/**
 * Copyright (C) 2007 EDIT
 * European Distributed Institute of Taxonomy
 * http://www.e-taxonomy.eu
 *
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * See LICENSE.TXT at the top of this package for the full license terms.
 */
package eu.etaxonomy.taxeditor.ui.section.agent;

import eu.etaxonomy.cdm.model.agent.Team;
import eu.etaxonomy.taxeditor.store.StoreUtil;
import eu.etaxonomy.taxeditor.ui.element.CacheRelevance;
import eu.etaxonomy.taxeditor.ui.element.CdmFormFactory;
import eu.etaxonomy.taxeditor.ui.element.CdmPropertyChangeEvent;
import eu.etaxonomy.taxeditor.ui.element.CheckboxElement;
import eu.etaxonomy.taxeditor.ui.element.ICdmFormElement;
import eu.etaxonomy.taxeditor.ui.element.LayoutConstants;
import eu.etaxonomy.taxeditor.ui.element.ToggleableTextElement;
import eu.etaxonomy.taxeditor.ui.section.AbstractIdentifiableEntityDetailElement;

/**
 * TeamDetailElement class.
 *
 * @author n.hoffmann
 * @created Apr 26, 2010
 */
public class TeamDetailElement
        extends AbstractIdentifiableEntityDetailElement<Team> {

	private ToggleableTextElement toggleable_nomenclaturalTitleCache;
	private ToggleableTextElement toggleable_collectorTitleCache;
	private CheckboxElement hasAdditionalMembers;

	private TeamMemberSection section_teamMembers;
	private boolean isNomenclaturalTeam;
	private boolean isCollectorTeam;
	
	/**
	 * Constructor for TeamDetailElement.
	 */
	public TeamDetailElement(CdmFormFactory cdmFormFactory,
			ICdmFormElement formElement, int style, boolean isNomenclatural, boolean isCollector) {
		super(cdmFormFactory, formElement);
		this.isNomenclaturalTeam = isNomenclatural;
		this.isCollectorTeam = isCollector;
	}

    @Override
    protected void createControls(ICdmFormElement formElement, Team entity, int style) {

    	setWarnForReferencingObjects(formElement);
        toggleable_cache = formFactory.createToggleableTextField(formElement, "Title Cache",
                entity.getTitleCache(), entity.isProtectedTitleCache(), style);
        toggleable_nomenclaturalTitleCache = formFactory.createToggleableTextField(formElement,
                "Abbrev. Title", entity.getNomenclaturalTitleCache(), entity.isProtectedNomenclaturalTitleCache(),
                CacheRelevance.CACHE2, style);
        toggleable_collectorTitleCache = formFactory.createToggleableTextField(formElement,
                "Collector Title", entity.getCollectorTitleCache(), entity.isProtectedCollectorTitleCache(),
                CacheRelevance.CACHE2, style);
        hasAdditionalMembers = formFactory.createCheckbox(formElement, "Additional Members", entity.isHasMoreMembers(), style);
        section_teamMembers = formFactory.createTeamMemberSection(getConversationHolder(), formElement, StoreUtil.getSectionStyle(TeamMemberSection.class, entity.getClass().getCanonicalName(), true), isNomenclaturalTeam, isCollectorTeam);
        section_teamMembers.setLayoutData(LayoutConstants.FILL_HORIZONTALLY(2, 1));

        //cache relevance
        //TODO
//        registerCacheRelevance(toggleable_nomenclaturalTitleCache);
        registerCacheRelevance(hasAdditionalMembers, toggleable_cache, toggleable_nomenclaturalTitleCache);
        registerCacheRelevance(section_teamMembers, toggleable_cache, toggleable_nomenclaturalTitleCache);
    }

	@Override
	protected void updateContent() {
		super.updateContent();
		if (toggleable_cache != null){
		    toggleable_cache.setCacheEnabled(getEntity().isProtectedTitleCache());
		}
		if (toggleable_nomenclaturalTitleCache != null){
		    toggleable_nomenclaturalTitleCache.setCacheEnabled(getEntity()
				.isProtectedNomenclaturalTitleCache());
		}
		if (toggleable_collectorTitleCache != null){
            toggleable_collectorTitleCache.setCacheEnabled(getEntity()
                .isProtectedCollectorTitleCache());
        }
		section_teamMembers.setEntity(getEntity());

		updateCacheRelevance();
	}

	@Override
	public void handleEvent(Object eventSource) {
		if (eventSource == toggleable_cache) {
			handleToggleableCacheField();

		} else if (eventSource == toggleable_nomenclaturalTitleCache) {
			updateToggleable_nomanclaturalTitleCacheField();
			updateCacheRelevance();
		} else if (eventSource == toggleable_collectorTitleCache) {
            updateToggleable_collectorTitleCacheField();
            updateCacheRelevance();
        } else if (eventSource == section_teamMembers) {
			toggleable_cache.setText(getEntity().getTitleCache());
			toggleable_nomenclaturalTitleCache.setText(getEntity()
					.getNomenclaturalTitleCache());
			toggleable_collectorTitleCache.setText(getEntity()
                    .getCollectorTitleCache());
		} else if (eventSource == hasAdditionalMembers){
			getEntity().setHasMoreMembers(hasAdditionalMembers.getSelection());
			updateToggleableCacheField();
			toggleable_nomenclaturalTitleCache.setText(getEntity()
					.getNomenclaturalTitleCache());
		}

		firePropertyChangeEvent(new CdmPropertyChangeEvent(this, null));
	}

	public void updateToggleable_nomanclaturalTitleCacheField() {
		 boolean pushedState = toggleable_nomenclaturalTitleCache.getState();

	        getEntity().setNomenclaturalTitleCache(toggleable_nomenclaturalTitleCache.getText(), pushedState);
	        updateCacheRelevance();

        if (!getEntity().isProtectedNomenclaturalTitleCache()) {
            toggleable_nomenclaturalTitleCache.setText(getEntity().getNomenclaturalTitleCache());
        }
    }

	public void updateToggleable_collectorTitleCacheField() {
        boolean pushedState = toggleable_collectorTitleCache.getState();

        getEntity().setCollectorTitleCache(toggleable_collectorTitleCache.getText(), pushedState);
        updateCacheRelevance();

       if (!getEntity().isProtectedCollectorTitleCache()) {
           toggleable_collectorTitleCache.setText(getEntity().getCollectorTitleCache());
       }
   }

    public boolean isNomenclaturalTeam() {
        return isNomenclaturalTeam;
    }
    public void setNomenclaturalTeam(boolean isNomenclaturalTeam) {
        this.isNomenclaturalTeam = isNomenclaturalTeam;
    }
}