/**
 * Copyright (C) 2007 EDIT
 * European Distributed Institute of Taxonomy
 * http://www.e-taxonomy.eu
 *
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * See LICENSE.TXT at the top of this package for the full license terms.
 */

package eu.etaxonomy.taxeditor.ui.section.agent;

import org.eclipse.swt.SWT;

import eu.etaxonomy.cdm.api.conversation.ConversationHolder;
import eu.etaxonomy.cdm.model.agent.Team;
import eu.etaxonomy.taxeditor.ui.element.AbstractCdmEntityWizardPage;
import eu.etaxonomy.taxeditor.ui.element.CdmFormFactory;
import eu.etaxonomy.taxeditor.ui.element.ICdmFormElement;
import eu.etaxonomy.taxeditor.ui.section.AbstractCdmDetailElement;

/**
 * <p>
 * TeamWizardPage class.
 * </p>
 *
 * @author n.hoffmann
 * @created Jun 1, 2010
 * @version 1.0
 */
public class TeamWizardPage extends AbstractCdmEntityWizardPage<Team> {

    private boolean isNomenclaturalTeam = false;
    private boolean isCollectorTeam = false;

	/**
	 * <p>
	 * Constructor for TeamWizardPage.
	 * </p>
	 *
	 * @param formFactory
	 *            a {@link eu.etaxonomy.taxeditor.ui.element.CdmFormFactory}
	 *            object.
	 * @param conversation
	 *            a {@link eu.etaxonomy.cdm.api.conversation.ConversationHolder}
	 *            object.
	 * @param entity
	 *            a {@link eu.etaxonomy.cdm.model.agent.Team} object.
	 */
	public TeamWizardPage(CdmFormFactory formFactory,
			ConversationHolder conversation, Team entity, boolean isNomenclaturalTeam, boolean isCollectorTeam) {
		super(formFactory, conversation, entity);
		setTitle("Team");
		setIsNomenclaturalTeam(isNomenclaturalTeam);
		setIsCollectorTeam(isCollectorTeam);
	}

	/** {@inheritDoc} */
	@Override
	public AbstractCdmDetailElement<Team> createElement(
			ICdmFormElement rootElement) {
		TeamDetailElement teamElement = formFactory.createTeamDetailElement(rootElement, SWT.NULL, isNomenclaturalTeam, isCollectorTeam);
		teamElement.setEntity(entity);
		return teamElement;
	}

    /**
     * @return the isNomenclaturalTeam
     */
    public boolean isNomenclaturalTeam() {
        return isNomenclaturalTeam;
    }

    public void setIsNomenclaturalTeam(boolean isNomenclaturalTeam){
        this.isNomenclaturalTeam = isNomenclaturalTeam;
    }
    
    public boolean isCollectorTeam() {
        return isCollectorTeam;
    }

    public void setIsCollectorTeam(boolean isCollectorTeam){
        this.isCollectorTeam = isCollectorTeam;
    }
}
