/**
* Copyright (C) 2016 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.ui.section.classification;

import java.util.Collection;
import java.util.Collections;
import java.util.List;
import java.util.Map;

import eu.etaxonomy.cdm.api.service.ITaxonNodeService;
import eu.etaxonomy.cdm.model.taxon.TaxonNode;
import eu.etaxonomy.cdm.persistence.dto.MergeResult;
import eu.etaxonomy.taxeditor.event.EventUtility;
import eu.etaxonomy.taxeditor.event.WorkbenchEventConstants;
import eu.etaxonomy.taxeditor.newWizard.IWizardPageListener;
import eu.etaxonomy.taxeditor.session.ICdmEntitySession;
import eu.etaxonomy.taxeditor.session.ICdmEntitySessionEnabled;
import eu.etaxonomy.taxeditor.store.CdmStore;

/**
 * @author k.luther
 * @date 23.03.2016
 */
public class EditTaxonNodeWizard 
		extends AbstractEditWizard<TaxonNode> 
		implements ICdmEntitySessionEnabled<TaxonNode>{

    private TaxonNodeWizardPage taxonNodePage;
    private IWizardPageListener wizardPageListener;
    private ICdmEntitySession cdmEntitySession;


//    @Override
//    public void init(IWorkbench workbench, IStructuredSelection selection) {
//        formFactory = new CdmFormFactory(getContext().get(Shell.class).getDisplay());
//        conversation = CdmStore.createConversation();
//    }

    @Override
    protected String getEntityName() {
        return "taxon node";
    }

    @Override
    protected void saveEntity() {
        getConversationHolder().bind();
        TaxonNode node = getEntity();
        if (node.getSource().checkEmpty()){
            node.setSource(null);
        }
        MergeResult result = CdmStore.getService(ITaxonNodeService.class).merge(node, true);
        EventUtility.postEvent(WorkbenchEventConstants.REFRESH_NAME_EDITOR, node.getTaxon().getUuid());

    }
    @Override
    public void addPages() {
        taxonNodePage = new TaxonNodeWizardPage(formFactory, getConversationHolder(), getEntity(), wizardPageListener, false);

        addPage(taxonNodePage);
    }

    public void addWizardPageListener(IWizardPageListener wizardPageListener){
        this.wizardPageListener = wizardPageListener;
    }

    @Override
    public void setEntity(TaxonNode entity) {
        setWindowTitle(String.format("Edit %s", getEntityName()));
        if (CdmStore.isActive() && cdmEntitySession == null) {

            cdmEntitySession = CdmStore.getCurrentSessionManager().newSession(this, true);
            cdmEntitySession.bind();
        }
        entity = CdmStore.getService(ITaxonNodeService.class).load(entity.getUuid());
        super.setEntity(entity);
    }

    @Override
    public ICdmEntitySession getCdmEntitySession() {
        return cdmEntitySession;
    }

    @Override
    public Collection<TaxonNode> getRootEntities() {
        return Collections.singleton(this.getEntity());
    }

    @Override
    public Map<Object, List<String>> getPropertyPathsMap() {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public void dispose() {
        cdmEntitySession.dispose();
        super.dispose();
    }
}