/**
* Copyright (C) 2016 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.ui.section.classification;

import java.util.Collection;
import java.util.Comparator;

import org.eclipse.ui.forms.widgets.TableWrapData;

import eu.etaxonomy.cdm.api.conversation.ConversationHolder;
import eu.etaxonomy.cdm.model.taxon.TaxonNode;
import eu.etaxonomy.cdm.model.taxon.TaxonNodeAgentRelation;
import eu.etaxonomy.taxeditor.ui.element.CdmFormFactory;
import eu.etaxonomy.taxeditor.ui.element.ICdmFormElement;
import eu.etaxonomy.taxeditor.ui.section.AbstractEntityCollectionSection;
import eu.etaxonomy.taxeditor.ui.section.DefaultCdmBaseComparator;

/**
 * @author k.luther
 * @date 22.03.2016
 *
 */
public class TaxonNodeAgentRelationCollectionSection extends AbstractEntityCollectionSection<TaxonNode, TaxonNodeAgentRelation> {



    public TaxonNodeAgentRelationCollectionSection (CdmFormFactory formFactory, ConversationHolder conversation,
            ICdmFormElement parentElement, int style) {
        super(formFactory, conversation, parentElement,  "Related persons or teams", style);

    }

    @Override
    public String getEmptyString() {
        return "No related persons or teams yet.";
    }

    @Override
    protected String getTooltipString() {
        return "Add a new person/team to this taxon node.";
    }

    @Override
    public Collection<TaxonNodeAgentRelation> getCollection(TaxonNode entity) {
        //why Do I have to give the method an entity???
       Collection<TaxonNodeAgentRelation> allAgentRelationships = getEntity().getAgentRelations();
       return allAgentRelationships;
    }

    @Override
    public Comparator<TaxonNodeAgentRelation> getComparator() {
        return new DefaultCdmBaseComparator<>();
    }

    @Override
    public void addElement(TaxonNodeAgentRelation element) {


    }

    @Override
    public void removeElement(TaxonNodeAgentRelation element) {
        getEntity().removeAgentRelation(element);
      //  CdmStore.getService(ITaxonNodeService.class).deleteAgentrelation(element);
    }


    @Override
    public TaxonNodeAgentRelation createNewElement() {
        TaxonNodeAgentRelation rel = getEntity().addAgentRelation(null, null);
        return rel;
    }

    public boolean isComplete(){

        return false;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public TaxonNodeAgentRelation addExisting() {
        return null;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public boolean allowAddExisting() {
        return false;
    }

    /**
     * @param i
     */
    public void setIndent(int i) {
        TableWrapData tableWrapData = (TableWrapData)getLayoutData();
        if (tableWrapData == null){
            tableWrapData = new TableWrapData();
        }
        tableWrapData.indent = i;
        setLayoutData(tableWrapData);

    }

}
