/**
 * Copyright (C) 2007 EDIT
 * European Distributed Institute of Taxonomy
 * http://www.e-taxonomy.eu
 *
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * See LICENSE.TXT at the top of this package for the full license terms.
 */
package eu.etaxonomy.taxeditor.ui.section.classification;

import java.util.UUID;

import org.eclipse.jface.util.PropertyChangeEvent;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Listener;

import eu.etaxonomy.cdm.api.conversation.ConversationHolder;
import eu.etaxonomy.cdm.api.service.dto.CreateTaxonDTO;
import eu.etaxonomy.cdm.common.CdmUtils;
import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.cdm.model.taxon.TaxonNode;
import eu.etaxonomy.taxeditor.l10n.Messages;
import eu.etaxonomy.taxeditor.newWizard.IWizardPageListener;
import eu.etaxonomy.taxeditor.newWizard.NewTaxonNodeWizard;
import eu.etaxonomy.taxeditor.preference.PreferencesUtil;
import eu.etaxonomy.taxeditor.ui.element.AbstractCdmEntityWizardPage;
import eu.etaxonomy.taxeditor.ui.element.CdmFormFactory;
import eu.etaxonomy.taxeditor.ui.element.ICdmFormElement;

/**
 * @author n.hoffmann
 * @created Sep 15, 2009
 */
public class TaxonNodeWizardPage extends AbstractCdmEntityWizardPage<TaxonNode> implements Listener{

    private final IWizardPageListener wizardPageListener;
    private final boolean isCreateNew;

	public TaxonNodeWizardPage(CdmFormFactory formFactory,
			ConversationHolder conversation, TaxonNode entity, IWizardPageListener closeOnEnterListener, boolean isCreateNew) {
		super(formFactory, conversation, entity);
		this.wizardPageListener = closeOnEnterListener;

		if (isCreateNew){
		    setTitle(Messages.TaxonNodeWizardPage_new);
		}else{
		    setTitle(Messages.TaxonNodeWizardPage_edit);
		}
		this.isCreateNew = isCreateNew;
	}

	@Override
	public TaxonNodeDetailElement createElement(ICdmFormElement rootElement) {
	    TaxonNodeDetailElement detailElement = formFactory.createTaxonNodeDetailElement(rootElement, this.isCreateNew);

	    if (isCreateNew){
		    detailElement.setParentTreeNode(((NewTaxonNodeWizard)getWizard()).getParentEntity());
		}else{
		    detailElement.setEntity(entity);
		}
//	    detailElement.setPersistentBackground(Abstrac);;

		formFactory.addPropertyChangeListener(this);
//		if (isCreateNew){
//		    detailElement.getMainControl().addListener(SWT.KeyDown, this);
//		}
		return detailElement;
	}

	@Override
	public void propertyChange(PropertyChangeEvent event) {
		checkComplete();
	}

	@Override
	protected void checkComplete() {
		TaxonNodeDetailElement detailElement = (TaxonNodeDetailElement) getDetailElement();

		if (detailElement.getClassification() == null) {
			setMessage(Messages.TaxonNodeWizardPage_no_classification, WARNING);
		}
		else if(isCreateNew ){
		    if(CdmUtils.isBlank(detailElement.getTaxonName())){
		        setMessage(Messages.TaxonNodeWizardPage_no_taxon_name, ERROR);
		    } else{
		        setMessage(null);
	            setPageComplete(true);
	            return;
		    }
		}
		else if(!detailElement.isComplete()){
		    setMessage(Messages.TaxonNodeWizardPage_not_all_required_fields, ERROR);
		}
		else {
			setMessage(null);
			setPageComplete(true);
			return;
		}

		setPageComplete(false);
	}

	public TaxonNode getParentTreeNode() {
		return ((TaxonNodeDetailElement) getDetailElement()).getParentTreeNode();
	}

	public CreateTaxonDTO createTaxon() {

	    UUID nameUuid = null;
	    UUID taxonUuid = null;
	    String taxonNameString = null;
	    if (((TaxonNodeDetailElement) getDetailElement()).getExistingName() != null){
	        nameUuid = ((TaxonNodeDetailElement) getDetailElement()).getExistingName().getUuid();
	    }else if (((TaxonNodeDetailElement) getDetailElement()).getTaxon() != null){
	        taxonUuid = ((TaxonNodeDetailElement) getDetailElement()).getTaxon().getUuid();
	    }else {
	        taxonNameString = ((TaxonNodeDetailElement) getDetailElement()).getTaxonName();
	    }

	    UUID secUuid = null;
	    if (((TaxonNodeDetailElement) getDetailElement()).getSecReference() != null){
	        secUuid = ((TaxonNodeDetailElement) getDetailElement()).getSecReference().getUuid();
	    }
	    boolean isPublish = ((TaxonNodeDetailElement) getDetailElement()).is_publish();
	    String secMicroReference = ((TaxonNodeDetailElement) getDetailElement()).getMicroReference();
	    CreateTaxonDTO createDto = null;
	    if (taxonUuid != null){
	        createDto = new CreateTaxonDTO(taxonUuid, false, null,isPublish,
	                taxonNameString, PreferencesUtil.getPreferredNomenclaturalCode(), null);
	    }else{
	        createDto = new CreateTaxonDTO(nameUuid, secUuid, secMicroReference,false, null,isPublish,
	            taxonNameString, PreferencesUtil.getPreferredNomenclaturalCode(), null);
	    }

        return createDto;
	}

	public TaxonNode getTaxonNode(){
	   return ((TaxonNodeDetailElement) getDetailElement()).getEntity();
	}

	public Taxon getExistingTaxon(){
	       return ((TaxonNodeDetailElement) getDetailElement()).getTaxon();
	    }

	@Override
	public void dispose() {
		formFactory.removePropertyChangeListener(this);
		super.dispose();
	}

    @Override
    public void handleEvent(Event event) {
        TaxonNodeDetailElement detailElement = (TaxonNodeDetailElement) getDetailElement();
        if(event.widget == detailElement.getMainControl()
                && event.keyCode == SWT.CR && isPageComplete()){
            wizardPageListener.close();
        }
    }
}
