/**
* Copyright (C) 2018 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.ui.section.common;

import java.util.Collection;
import java.util.Comparator;

import org.eclipse.ui.forms.widgets.TableWrapData;

import eu.etaxonomy.cdm.api.conversation.ConversationHolder;
import eu.etaxonomy.cdm.common.URI;
import eu.etaxonomy.cdm.model.media.ExternalLink;
import eu.etaxonomy.cdm.model.media.ExternalLinkType;
import eu.etaxonomy.cdm.model.reference.OriginalSourceBase;
import eu.etaxonomy.taxeditor.ui.element.CdmFormFactory;
import eu.etaxonomy.taxeditor.ui.element.ICdmFormElement;
import eu.etaxonomy.taxeditor.ui.section.AbstractEntityCollectionSection;

/**
 * @author k.luther
 * @since 23.08.2018
 */
public class ExternalLinksSection extends AbstractEntityCollectionSection<OriginalSourceBase, ExternalLink> {

    private String emptySectionString = "No external link yet.";

    private boolean isShowDescription = true;

    public ExternalLinksSection(CdmFormFactory cdmFormFactory, ConversationHolder conversation, ICdmFormElement parentElement, int style) {
        super(cdmFormFactory, conversation, parentElement, "External Links", style);
    }

    public ExternalLinksSection(CdmFormFactory cdmFormFactory, ConversationHolder conversation,
            ICdmFormElement parentElement, boolean isAdvanced, String string, int style) {
        super(cdmFormFactory, conversation, parentElement, string, style);
        this.isShowDescription = isAdvanced;
    }

    @Override
    //TODO think about moving to cdmlib
    public Comparator<ExternalLink> getComparator() {
        return (o1, o2)-> {
            URI uri1 = o1 == null? null: o1.getUri();
            URI uri2 = o2 == null? null: o2.getUri();

                if (uri1 == uri2){
                    return 0;
                }
                if (uri1 == null){
                    return -1;
                }
                if (uri2 == null){
                    return 1;
                }
                return uri1.toString().compareTo(uri2.toString());
        };
    }

    @Override
    public Collection<ExternalLink> getCollection(OriginalSourceBase entity) {
        if (entity != null){
            return entity.getLinks();
        } else {
            return null;
        }
    }

    @Override
    public ExternalLink createNewElement() {
        ExternalLink link = ExternalLink.NewInstance(ExternalLinkType.Unknown, null);
        return link;
    }

    @Override
    public void addElement(ExternalLink element) {
        getEntity().addLink(element);
    }

    @Override
    public ExternalLink addExisting() {
        return null;
    }

    @Override
    public boolean allowAddExisting() {
        return false;
    }

    @Override
    public void removeElement(ExternalLink element) {
        getEntity().removeLink(element);
    }

    @Override
    public String getEmptyString() {
        return emptySectionString;
    }

    @Override
    protected String getTooltipString() {
        return null;
    }

    public void setEmptySectionString(String emptySectionString) {
        this.emptySectionString = emptySectionString;
    }

    public void setIndent(int indent) {
       TableWrapData tableWrapData = (TableWrapData)this.getLayoutData();
       if (tableWrapData == null){
           tableWrapData = new TableWrapData();
       }
       tableWrapData.indent = indent;
       this.setLayoutData(tableWrapData);
       getLayoutComposite().layout();
    }

    public boolean isAdvanced() {
        return isShowDescription;
    }

    public void setAdvanced(boolean isAdvanced) {
        this.isShowDescription = isAdvanced;
    }
}