/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.ui.section.description;

import org.eclipse.e4.ui.di.UISynchronize;
import org.eclipse.e4.ui.workbench.modeling.EPartService;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.action.ToolBarManager;
import org.eclipse.jface.viewers.ISelectionProvider;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Control;

import eu.etaxonomy.cdm.api.conversation.ConversationHolder;
import eu.etaxonomy.cdm.model.description.DescriptionElementBase;
import eu.etaxonomy.cdm.model.description.Feature;
import eu.etaxonomy.cdm.model.description.TextData;
import eu.etaxonomy.taxeditor.model.AbstractUtility;
import eu.etaxonomy.taxeditor.model.MessagingUtils;
import eu.etaxonomy.taxeditor.preference.PreferencesUtil;
import eu.etaxonomy.taxeditor.store.CdmStore;
import eu.etaxonomy.taxeditor.store.StoreUtil;
import eu.etaxonomy.taxeditor.ui.element.CdmFormFactory;
import eu.etaxonomy.taxeditor.ui.element.ICdmFormElement;
import eu.etaxonomy.taxeditor.ui.element.IEnableableFormElement;
import eu.etaxonomy.taxeditor.ui.section.AbstractCdmDetailElement;
import eu.etaxonomy.taxeditor.ui.section.AbstractCdmDetailSection;
import eu.etaxonomy.taxeditor.ui.section.AbstractEntityCollectionElement;
import eu.etaxonomy.taxeditor.ui.section.description.operation.ChangeDescriptionElementType;
import eu.etaxonomy.taxeditor.view.e4.details.DetailsPartE4;

/**
 * <p>DescriptionElementDetailSection class.</p>
 *
 * @author n.hoffmann
 * @created Mar 5, 2010
 * @version 1.0
 */
public class DescriptionElementDetailSection extends
		AbstractCdmDetailSection<DescriptionElementBase> {



	/**
	 * <p>Constructor for DescriptionElementDetailSection.</p>
	 *
	 * @param cdmFormFactory a {@link eu.etaxonomy.taxeditor.ui.element.CdmFormFactory} object.
	 * @param conversation a {@link eu.etaxonomy.cdm.api.conversation.ConversationHolder} object.
	 * @param parentElement a {@link eu.etaxonomy.taxeditor.ui.element.ICdmFormElement} object.
	 * @param selectionProvider a {@link org.eclipse.jface.viewers.ISelectionProvider} object.
	 * @param style a int.
	 */
	public DescriptionElementDetailSection(CdmFormFactory cdmFormFactory, ConversationHolder conversation,
			ICdmFormElement parentElement, ISelectionProvider selectionProvider, int style) {
		super(cdmFormFactory, conversation, parentElement, selectionProvider, style);
	}


	@Override
    protected Control createToolbar() {

		ToolBarManager toolBarManager = new ToolBarManager(SWT.FLAT);

		final String destinationString = (getEntity().isInstanceOf(TextData.class)) ? "the specific element" : "free text";

		final String label = "Change to " + destinationString;

		Action addAction = new Action("Change Type", IAction.AS_PUSH_BUTTON) {
			/* (non-Javadoc)
			 * @see org.eclipse.jface.action.Action#run()
			 */
			@Override
			public void run() {
				boolean confirmed = MessagingUtils.confirmDialog("Confirmation", "Do you really want to change to "
						+ destinationString + "? Current data will be lost.");
				if (confirmed) {
					EPartService partService = formFactory.getContext().get(EPartService.class);
                    DetailsPartE4 detailsView = AbstractUtility.getDetailsView(partService);
                    ChangeDescriptionElementType operation = new ChangeDescriptionElementType(label,
							getEntity(), detailsView, StoreUtil.getUndoContext());
					AbstractUtility.executeOperation(operation, formFactory.getContext().get(UISynchronize.class));
				}
			}
		};
		// TODO enable this once a proper icon has been found
		// ticket #5043
//		addAction.setImageDescriptor(new ImageDescriptor() {
//
//			@Override
//			public ImageData getImageData() {
//				return ImageResources.getImage(ImageResources.ADD_ICON).getImageData();
//			}
//		});
		addAction.setToolTipText(label);

		toolBarManager.add(addAction);

		return toolBarManager.createControl(this);
	}

	/** {@inheritDoc} */
	@Override
	public String getHeading() {
		return "Description Element";
	}

	/** {@inheritDoc} */
	@Override
	protected void setSectionTitle() {
		this.setText(getHeading() + ": " + getEntity().getFeature().getPreferredRepresentation(CdmStore.getDefaultLanguage()));
		        //getLabel(CdmStore.getDefaultLanguage()));

        if(IsMoreThanOneTypeSupported()){
            setTextClient(createToolbar());
        }
	}

	private boolean IsMoreThanOneTypeSupported(){
	    int count = 0;
	    Feature feature = getEntity().getFeature();
	    if(feature.isSupportsCategoricalData()){count++;}
	    if(feature.isSupportsCommonTaxonName()){count++;}
	    if(feature.isSupportsDistribution()){count++;}
	    if(feature.isSupportsIndividualAssociation()){count++;}
	    if(feature.isSupportsQuantitativeData()){count++;}
	    if(feature.isSupportsTaxonInteraction()){count++;}
	    if(feature.isSupportsTextData()){count++;}
	    if(feature.isSupportsTemporalData()){count++;}
	    return count > 1;
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.ui.section.AbstractCdmDetailSection#createCdmDetailElement(eu.etaxonomy.taxeditor.ui.section.AbstractCdmDetailSection, int)
	 */
	@Override
	protected AbstractCdmDetailElement<DescriptionElementBase> createCdmDetailElement(AbstractCdmDetailSection<DescriptionElementBase> parentElement, int style) {
	    return formFactory.createDescriptionElementDetailElement(parentElement, style);
	}
	@Override
    public void setEntity(DescriptionElementBase entity) {
        super.setEntity(entity);
        boolean isEnabled = true;
        if ((entity.getInDescription().isComputed() || entity.getInDescription().isCloneForSource())&& PreferencesUtil.isComputedDesciptionHandlingDisabled() ){
            isEnabled = false;
        }
        setEnabled(isEnabled);

    }


}
