/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.ui.section.description;

import java.util.Comparator;

import eu.etaxonomy.cdm.model.description.DescriptionElementSource;
import eu.etaxonomy.cdm.model.reference.OriginalSourceBase;
import eu.etaxonomy.cdm.model.reference.OriginalSourceType;
import eu.etaxonomy.cdm.model.reference.Reference;

/**
 * @author pplitzner
 * @date Apr 13, 2016
 */
public class SourceComparator  <T extends OriginalSourceBase> implements Comparator<T> {

    @Override
    public int compare(T o1, T o2) {

        //same and null compare, to be on the save side
        if (o1 == o2){
            return 0;
        }else if (o1 == null){
            return -1;
        }else if (o2 == null){
            return 1;
        }

        boolean isDescriptionElementSource1 = false;
        boolean isDescriptionElementSource2 = false;
        if (o1.isInstanceOf(DescriptionElementSource.class)){
            isDescriptionElementSource1 = true;
        }
        if (o2.isInstanceOf(DescriptionElementSource.class)){
            isDescriptionElementSource2 = true;
        }

        if (isDescriptionElementSource1 != isDescriptionElementSource2){
            if (isDescriptionElementSource1){
                return -1;
            }else{
                return 1;
            }
        }

        // the newly created should always be on top
        if (!o1.isPersited() && o2.isPersited()) {
            return -1;
        } else if(o1.isPersited() && !o2.isPersited()){
            return 1;
        }

        // sort by type (Primary taxonomic > Primary Media > others
        // alphabetically by reference title cache)
        OriginalSourceType type1 = o1.getType();
        OriginalSourceType type2 = o2.getType();
        if (type1 == null){
            if (type2!=null){
                return -1;
            }
        } else if (type2 == null){
            return 1;
        } else if(type1.equals(type2)){
            //continue with citation compare
        } else if (type1.equals(OriginalSourceType.PrimaryTaxonomicSource)){
            return 1;
        } else if (type2.equals(OriginalSourceType.PrimaryTaxonomicSource)){
            return -1;
        } else if (type1.equals(OriginalSourceType.PrimaryMediaSource)){
            return 1;
        } else if (type2.equals(OriginalSourceType.PrimaryMediaSource)){
            return -1;
        }

        int result;

        //sort by citation title cache if types are equal
        Reference citation1 = o1.getCitation();
        Reference citation2 = o2.getCitation();
        if(citation1!=null && citation2!=null){
            result = citation1.getTitleCache().compareTo(citation2.getTitleCache());
            if (result != 0){
                return result;
            }
        }

        //sort by created
        if(o2.getCreated()!=null && o1.getCreated()!=null){
            result = o1.getCreated().compareTo(o2.getCreated());
            if (result != 0){
                return result;
            }
        }else if (o1.getCreated() == null){
            if (o2.getCreated() != null){
                return -1;
            }
        }else if (o2.getCreated() == null){
            return 1;
        }

        //default fallback
        return o1.getUuid().compareTo(o2.getUuid());
    }
}