/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.ui.section.description;

import java.util.Collection;
import java.util.Comparator;

import eu.etaxonomy.cdm.api.conversation.ConversationHolder;
import eu.etaxonomy.cdm.model.description.CategoricalData;
import eu.etaxonomy.cdm.model.description.StateData;
import eu.etaxonomy.taxeditor.ui.element.CdmFormFactory;
import eu.etaxonomy.taxeditor.ui.element.ICdmFormElement;
import eu.etaxonomy.taxeditor.ui.element.IEnableableFormElement;
import eu.etaxonomy.taxeditor.ui.section.AbstractEntityCollectionSection;
import eu.etaxonomy.taxeditor.ui.section.DefaultCdmBaseComparator;

/**
 * @author n.hoffmann
 * @created Sep 15, 2010
 */
public class StateDataSection extends AbstractEntityCollectionSection<CategoricalData, StateData> implements IEnableableFormElement {

	public StateDataSection(CdmFormFactory formFactory,
			ConversationHolder conversation, ICdmFormElement parentElement,
			int style) {
		super(formFactory, conversation, parentElement, "State Data", style);
	}

	@Override
	public Collection<StateData> getCollection(CategoricalData entity) {
		return entity.getStateData();
	}

	@Override
	public Comparator<StateData> getComparator() {
        return new DefaultCdmBaseComparator<>();
	}

	@Override
	public StateData createNewElement() {
		return StateData.NewInstance();
	}

	@Override
	public void addElement(StateData element) {
		getEntity().addStateData(element);
	}

	@Override
	public void removeElement(StateData element) {
		getEntity().removeStateData(element);
	}

	@Override
	public String getEmptyString() {
		return "No state data yet.";
	}

	@Override
	protected String getTooltipString() {
		return "Create new state data";
	}

    @Override
    public StateData addExisting() {
        return null;
    }

    @Override
    public boolean allowAddExisting() {
        return false;
    }
}