/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.ui.section.description;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Set;

import org.eclipse.swt.events.SelectionListener;

import eu.etaxonomy.cdm.model.description.StatisticalMeasure;
import eu.etaxonomy.cdm.model.description.StatisticalMeasurementValue;
import eu.etaxonomy.cdm.model.term.TermType;
import eu.etaxonomy.taxeditor.store.StoreUtil;
import eu.etaxonomy.taxeditor.ui.combo.term.TermComboElement;
import eu.etaxonomy.taxeditor.ui.element.AbstractFormSection;
import eu.etaxonomy.taxeditor.ui.element.BigDecimalWithLabelElement;
import eu.etaxonomy.taxeditor.ui.element.CdmFormFactory;
import eu.etaxonomy.taxeditor.ui.element.ICdmFormElement;
import eu.etaxonomy.taxeditor.ui.element.IEnableableFormElement;
import eu.etaxonomy.taxeditor.ui.element.LayoutConstants;
import eu.etaxonomy.taxeditor.ui.section.AbstractEntityCollectionElement;

/**
 * <p>StatisticalMeasurementValueElement class.</p>
 *
 * @author n.hoffmann
 * @created Sep 15, 2010
 */
public class StatisticalMeasurementValueElement extends
		AbstractEntityCollectionElement<StatisticalMeasurementValue> {

	private BigDecimalWithLabelElement number_value;
	private TermComboElement<StatisticalMeasure> combo_type;
	private ModifierSection section_modifiers;

	/**
	 * <p>Constructor for StatisticalMeasurementValueElement.</p>
	 *
	 * @param formFactory a {@link eu.etaxonomy.taxeditor.ui.element.CdmFormFactory} object.
	 * @param section a {@link eu.etaxonomy.taxeditor.ui.element.AbstractFormSection} object.
	 * @param entity a {@link eu.etaxonomy.cdm.model.description.StatisticalMeasurementValue} object.
	 * @param removeListener a {@link org.eclipse.swt.events.SelectionListener} object.
	 * @param style a int.
	 */
	public StatisticalMeasurementValueElement(CdmFormFactory formFactory,
			AbstractFormSection section, StatisticalMeasurementValue entity,
			SelectionListener removeListener, int style) {
		super(formFactory, section, entity, removeListener, null, style);
	}

	@Override
	public void setEntity(StatisticalMeasurementValue entity) {
		this.entity = entity;
		if (combo_type != null){
			Set<StatisticalMeasure> statisiticalMeasures = getEntity().getQuantitativeData().getFeature().getRecommendedStatisticalMeasures();
	        combo_type.setTerms(new ArrayList<StatisticalMeasure>(statisiticalMeasures));
			combo_type.setSelection(entity.getType());
			number_value.setNumber(entity.getValue());
			section_modifiers.setEntity(entity);
		}
	}

	@Override
	public void createControls(ICdmFormElement element, int style) {
		number_value = formFactory.createBigDecimalTextWithLabelElement(element, "Value", 0, style);
		combo_type = formFactory.createDefinedTermComboElement(TermType.StatisticalMeasure, element, "Statistical Measure", null, style);
		section_modifiers = formFactory.createModifierSection(getConversationHolder(), element, StoreUtil.getSectionStyle(ModifierSection.class, StatisticalMeasurementValue.class.getCanonicalName()));
		section_modifiers.setLayoutData(LayoutConstants.FILL_HORIZONTALLY(2, 1));
		if (entity != null){
			setEntity(entity);
		}
	}

	@Override
	public void handleEvent(Object eventSource) {
		if(eventSource == number_value){
			BigDecimal bigDecimalNumber = number_value.getBigDecimal();
//			if(bigDecimalNumber==null){
//			    String zero = "0.0";
//			    bigDecimalNumber = new BigDecimal(zero);
//			    number_value.setText(zero);
//			}
            getEntity().setValue(bigDecimalNumber);
		}
		else if(eventSource == combo_type){
			getEntity().setType(combo_type.getSelection());
			combo_type.removeEmptyElement();
		}
	}

//	@Override
//	public void setEnabled(boolean enabled) {
//		combo_type.setEnabled(enabled);
//		number_value.setEnabled(enabled);
//		section_modifiers.setEnabled(enabled);
//	}
//
//	@Override
//	public boolean isEnabled() {		
//		return combo_type.isEnabled();
//	}
}
