/**
 *
 */
package eu.etaxonomy.taxeditor.ui.section.description.detail;

import java.util.ArrayList;

import eu.etaxonomy.cdm.model.description.MeasurementUnit;
import eu.etaxonomy.cdm.model.description.QuantitativeData;
import eu.etaxonomy.cdm.model.term.TermType;
import eu.etaxonomy.taxeditor.store.StoreUtil;
import eu.etaxonomy.taxeditor.ui.combo.term.TermComboElement;
import eu.etaxonomy.taxeditor.ui.element.CdmFormFactory;
import eu.etaxonomy.taxeditor.ui.element.ICdmFormElement;
import eu.etaxonomy.taxeditor.ui.element.LayoutConstants;
import eu.etaxonomy.taxeditor.ui.section.description.StatisticalMeasurementValueSection;

/**
 * <p>QuantitativeDataDetailElement class.</p>
 *
 * @author n.hoffmann
 */
public class QuantitativeDataDetailElement extends
		AbstractDetailedDescriptionDetailElement<QuantitativeData> {

	private TermComboElement<MeasurementUnit> comboMeasurementUnit;
	private StatisticalMeasurementValueSection section_statisticalMeasurementValues;

	/**
	 * <p>Constructor for QuantitativeDataDetailElement.</p>
	 *
	 * @param formFactory a {@link eu.etaxonomy.taxeditor.ui.element.CdmFormFactory} object.
	 * @param formElement a {@link eu.etaxonomy.taxeditor.ui.element.ICdmFormElement} object.
	 * @param entity a {@link eu.etaxonomy.cdm.model.description.QuantitativeData} object.
	 * @param style a int.
	 */
	public QuantitativeDataDetailElement(CdmFormFactory formFactory,
			ICdmFormElement formElement, QuantitativeData entity, boolean enabled, int style) {
		super(formFactory, formElement, entity, enabled, style);
	}

	@Override
	protected void createControls(ICdmFormElement formElement,
			QuantitativeData entity, int style) {
		comboMeasurementUnit = formFactory.createDefinedTermComboElement(TermType.MeasurementUnit, formElement, "Measurement Unit", null, style);
        comboMeasurementUnit.setTerms(new ArrayList<MeasurementUnit>(entity.getFeature().getRecommendedMeasurementUnits()));
        comboMeasurementUnit.setSelection(entity.getUnit());
		section_statisticalMeasurementValues = formFactory.createStatisticalMeasurementValueSection(getConversationHolder(), formElement, StoreUtil.getSectionStyle(StatisticalMeasurementValueSection.class, entity.getClass().getCanonicalName(), true));
		section_statisticalMeasurementValues.setEntity(entity);
		section_statisticalMeasurementValues.setLayoutData(LayoutConstants.FILL_HORIZONTALLY(2, 1));
		if (entity.getFeature().getRecommendedModifierEnumeration() != null && !entity.getFeature().getRecommendedModifierEnumeration().isEmpty()){
            createModifier(formElement, entity, style);
        }else{
            createFreetextModifier(formElement, entity, style);
        }
	}

	@Override
	public void handleEvent(Object eventSource) {
		if(eventSource == comboMeasurementUnit){
			getEntity().setUnit(comboMeasurementUnit.getSelection());
			comboMeasurementUnit.removeEmptyElement();
		}else if (eventSource == section_statisticalMeasurementValues){
			// FIXME
		}
	}
}
