/**
* Copyright (C) 2019 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.ui.section.feature;

import java.util.Collections;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Color;

import eu.etaxonomy.cdm.model.description.Feature;
import eu.etaxonomy.cdm.model.description.FeatureState;
import eu.etaxonomy.cdm.model.description.State;
import eu.etaxonomy.taxeditor.ui.combo.term.TermComboElement;
import eu.etaxonomy.taxeditor.ui.element.AbstractFormSection;
import eu.etaxonomy.taxeditor.ui.element.CdmFormFactory;
import eu.etaxonomy.taxeditor.ui.element.ICdmFormElement;
import eu.etaxonomy.taxeditor.ui.section.AbstractEntityCollectionElement;
import eu.etaxonomy.taxeditor.ui.selection.EntitySelectionElement;

/**
 * @author pplitzner
 * @since Aug 16, 2019
 *
 */
public abstract class AbstractApplicableElement extends AbstractEntityCollectionElement<FeatureState> {

    private TermComboElement<State> comboState;
    private EntitySelectionElement<Feature> selectFeature;

    public AbstractApplicableElement(CdmFormFactory formFactory, AbstractFormSection section, FeatureState entity,
            SelectionListener removeListener, Color backgroundColor, int style) {
        super(formFactory, section, entity, removeListener, backgroundColor, style);

    }

    @Override
    public void createControls(ICdmFormElement element, int style) {
        selectFeature = formFactory.createSelectionElement(Feature.class, element, "Feature", null, EntitySelectionElement.SELECTABLE, SWT.NONE);
        comboState = formFactory.createDefinedTermComboElement(Collections.EMPTY_LIST, element, getComboLabel(), null, style);
        comboState.setEnabled(false);
        selectFeature.setEnabled(false);
        
        selectFeature.setEntity(entity.getFeature());
        comboState.setSelection(entity.getState());
    }

    protected abstract String getComboLabel();

    @Override
    public void setEntity(FeatureState entity) {
        this.entity = entity;
        if (selectFeature != null){
	        selectFeature.setEntity(entity.getFeature());
	        comboState.setSelection(entity.getState());
        }
    }

    @Override
    public void handleEvent(Object eventSource) {
    }
}
