/**
* Copyright (C) 2009 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.ui.section.feature;

import eu.etaxonomy.cdm.model.description.Character;
import eu.etaxonomy.cdm.model.term.TermNode;
import eu.etaxonomy.taxeditor.editor.definedterm.TermBasePropertyTester;
import eu.etaxonomy.taxeditor.store.StoreUtil;
import eu.etaxonomy.taxeditor.ui.element.CdmFormFactory;
import eu.etaxonomy.taxeditor.ui.element.ICdmFormElement;
import eu.etaxonomy.taxeditor.ui.element.LayoutConstants;
import eu.etaxonomy.taxeditor.ui.section.AbstractCdmDetailElement;

/**
 *
 * @author pplitzner
 * @since Aug 19, 2019
 *
 */
public class CharacterNodeDetailElement extends AbstractCdmDetailElement<TermNode<Character>> {
    private CharacterDetailSection characterDetails;

    private ICdmFormElement parentFormElement;
//
//    protected RepresentationElement element_Representation;
//
//    private TermComboElement<DefinedTerm> comboStructuralModifier;
//
//    private CheckboxElement supportsQuantitativeData;
//
//    private CheckboxElement supportsCategoricalData;
//
//    private StateVocabularyCollectionSection sectionStateVocabularies;
//
//    private MeasurementUnitCollectionSection sectionMeasurementUnits;
//
//    private StatisticalMeasureCollectionSection sectionStatisticalMeasures;
//
//    private RecommendedModifierVocabulariesCollectionSection sectionModifierVocabularies;

    private InapplicableIfEntityCollectionSectionForNode sectionInapplicableIf;
    private OnlyApplicableIfEntityCollectionSectionForNode sectionOnlyApplicableIf;

	public CharacterNodeDetailElement(CdmFormFactory formFactory,
			ICdmFormElement formElement) {
		super(formFactory, formElement);
	}

	@Override
	protected void createControls(ICdmFormElement formElement, TermNode<Character> entity, int style) {
	    this.parentFormElement = formElement;
//	    characterDetails = formFactory.createCharacterDetailElement(formElement, style);
	    characterDetails = formFactory.createCharacterDetailSection(getConversationHolder(), formElement, null, StoreUtil.getSectionStyle(CharacterDetailSection.class, entity.getClass().getCanonicalName(), true));
	    characterDetails.setLayoutData(LayoutConstants.FILL_HORIZONTALLY(2, 1));
	    addElement(characterDetails);
	    addControl(characterDetails);
	    characterDetails.setEntity(entity.getTerm());
//	    element_Representation = formFactory.createTranslatableRepresentationElement(formElement,  entity.getTerm().getPreferredRepresentation(CdmStore.getDefaultLanguage()),entity.getTerm(), 100, style, true);
//
//	    createText(formElement, "Structure", getStructureText(entity.getTerm()));
//	    createText(formElement, "Property", getPropertyText(entity.getTerm()));
//
//        comboStructuralModifier = formFactory.createDefinedTermComboElement(TermType.StructureModifier, formElement, "Structural Modifier", entity.getTerm().getStructureModifier(), style);
//
//	    supportsQuantitativeData = formFactory.createCheckbox(formElement, "Supports Quantitative Data", entity.getTerm().isSupportsQuantitativeData(), style);
//	    supportsCategoricalData = formFactory.createCheckbox(formElement, "Supports Categorical Data", entity.getTerm().isSupportsCategoricalData(), style);
//
//        if (supportsCategoricalData.getSelection()) {
//            sectionStateVocabularies = formFactory.createStateVocabulariesSection(getConversationHolder(),
//                    parentFormElement, StoreUtil.getSectionStyle(StateVocabularyCollectionSection.class, entity.getClass().getCanonicalName(), true));
//            sectionStateVocabularies.setLayoutData(LayoutConstants.FILL_HORIZONTALLY(2, 1));
//            sectionStateVocabularies.setEntity(entity.getTerm());
//        }
//        if (supportsQuantitativeData.getSelection()) {
//            sectionMeasurementUnits = formFactory.createMeasurementUnitCollectionSection(getConversationHolder(),
//                    parentFormElement, StoreUtil.getSectionStyle(MeasurementUnitCollectionSection.class, entity.getClass().getCanonicalName(), true));
//            sectionMeasurementUnits.setLayoutData(LayoutConstants.FILL_HORIZONTALLY(2, 1));
//            sectionMeasurementUnits.setEntity(entity.getTerm());
//
//            sectionStatisticalMeasures = formFactory.createStatisticalMeasureCollectionSection(getConversationHolder(),
//                    parentFormElement, StoreUtil.getSectionStyle(StatisticalMeasureCollectionSection.class, entity.getClass().getCanonicalName(), true));
//            sectionStatisticalMeasures.setLayoutData(LayoutConstants.FILL_HORIZONTALLY(2, 1));
//            sectionStatisticalMeasures.setEntity(entity.getTerm());
//        }
//        if (supportsCategoricalData.getSelection() || supportsQuantitativeData.getSelection()) {
//            sectionModifierVocabularies = formFactory.createRecommendedModifierVocabulariesCollectionSection(
//                    getConversationHolder(), parentFormElement, StoreUtil.getSectionStyle(RecommendedModifierVocabulariesCollectionSection.class, entity.getClass().getCanonicalName(), true));
//            sectionModifierVocabularies.setLayoutData(LayoutConstants.FILL_HORIZONTALLY(2, 1));
//            sectionModifierVocabularies.setEntity(entity.getTerm());
//        }

        // applicability
        sectionInapplicableIf = formFactory.createInapplicableIfEntityCollectionSectionForNode(getConversationHolder(),
                formElement, StoreUtil.getSectionStyle(InapplicableIfEntityCollectionSection.class, entity.getTerm().getClass().getCanonicalName()));
        sectionInapplicableIf.setLayoutData(LayoutConstants.FILL_HORIZONTALLY(2, 1));
        sectionInapplicableIf.setEntity(getEntity());
        sectionOnlyApplicableIf = formFactory.createOnlyApplicableIfEntityCollectionSectionForNode(getConversationHolder(),
                formElement, StoreUtil.getSectionStyle(OnlyApplicableIfEntityCollectionSection.class, entity.getTerm().getClass().getCanonicalName()));
        sectionOnlyApplicableIf.setLayoutData(LayoutConstants.FILL_HORIZONTALLY(2, 1));
        sectionOnlyApplicableIf.setEntity(entity);
        
    }



    public static String getPropertyText(Character entity) {
        return entity.getProperty()!=null?StoreUtil.getPath(entity.getProperty()):"[no property]";
    }

    public static String getStructureText(Character entity) {
        return entity.getStructure()!=null?StoreUtil.getPath(entity.getStructure()):"[no structure]";
    }

    public static String getRatioToText(Character entity) {
        return entity.getRatioToStructure()!=null?StoreUtil.getPath(entity.getRatioToStructure()):"[no ratio to structure]";
    }

    @Override
    public void setEntity(TermNode<Character> entity) {
        super.setEntity(entity);
        characterDetails.setEntity(entity.getTerm());
        setEnabled(TermBasePropertyTester.isModifiable(entity));
        
    }
    
    

	@Override
	public void handleEvent(Object eventSource) {

//	    if(eventSource == characterDetails){
//	        getEntity().getTerm().setStructureModifier(comboStructuralModifier.getSelection());
//	    }
//	    if(eventSource == supportsQuantitativeData || eventSource == supportsCategoricalData){
//	        //if any type is selected disable supportsTextData
//	        getEntity().getTerm().setSupportsTextData(false);
//	    }
//	    if(eventSource == supportsQuantitativeData){
//	        getEntity().getTerm().setSupportsQuantitativeData(supportsQuantitativeData.getSelection());
//	        if(supportsQuantitativeData.getSelection()){
//	            activateQuantitativeWidgets();
//	        }
//	        else{
//	            removeQuantitativeWidgets();
//	        }
//            StoreUtil.reflowParentScrolledForm(getLayoutComposite(), true);
//	    }
//	    else if(eventSource == supportsCategoricalData){
//	        getEntity().getTerm().setSupportsCategoricalData(supportsCategoricalData.getSelection());
//            if (supportsCategoricalData.getSelection()) {
//                activateCategoricalWidgets();
//            }
//	        else{
//	            removeCategoricalWidgets();
//	        }
//            StoreUtil.reflowParentScrolledForm(getLayoutComposite(), true);
//	    }
//	    else if (eventSource == element_Representation) {
//            Representation selectedRepresentation = element_Representation.getSelectedRepresentation();
//            if(selectedRepresentation!=null){
//                Language representationLanguage = selectedRepresentation.getLanguage();
//                if(representationLanguage==null){
//                    representationLanguage = Language.getDefaultLanguage();
//                }
//                getEntity().getTerm().setLabel(selectedRepresentation.getLabel(),  representationLanguage);
//                getEntity().getTerm().getRepresentation(representationLanguage).setAbbreviatedLabel(selectedRepresentation.getAbbreviatedLabel());
//                getEntity().getTerm().getRepresentation(representationLanguage).setText(selectedRepresentation.getDescription());
//            }
//            getEntity().getTerm().setTitleCache(null);
//        }
	}



//	private void activateQuantitativeWidgets(){
//	    //disable categorical widgets
//	    supportsCategoricalData.setSelection(false);
//	    getEntity().getTerm().setSupportsCategoricalData(false);
//	    removeCategoricalWidgets();
//
//	    //measurement units
//	    sectionMeasurementUnits = formFactory.createMeasurementUnitCollectionSection(getConversationHolder(),
//	            parentFormElement, StoreUtil.getSectionStyle(MeasurementUnitCollectionSection.class, getEntity().getClass().getCanonicalName(), true));
//	    sectionMeasurementUnits.setLayoutData(LayoutConstants.FILL_HORIZONTALLY(2, 1));
//	    sectionMeasurementUnits.setEntity(getEntity().getTerm());
//
//	    //statistical measures
//	    sectionStatisticalMeasures = formFactory.createStatisticalMeasureCollectionSection(
//	            getConversationHolder(), parentFormElement, StoreUtil.getSectionStyle(StatisticalMeasureCollectionSection.class, getEntity().getClass().getCanonicalName(), true));
//	    sectionStatisticalMeasures.setLayoutData(LayoutConstants.FILL_HORIZONTALLY(2, 1));
//	    sectionStatisticalMeasures.setEntity(getEntity().getTerm());
//
//	    //modifiers
//	    sectionModifierVocabularies = formFactory.createRecommendedModifierVocabulariesCollectionSection(
//	            getConversationHolder(), parentFormElement, StoreUtil.getSectionStyle(RecommendedModifierVocabulariesCollectionSection.class, getEntity().getClass().getCanonicalName(), true));
//	    sectionModifierVocabularies.setLayoutData(LayoutConstants.FILL_HORIZONTALLY(2, 1));
//	    sectionModifierVocabularies.setEntity(getEntity().getTerm());
//	}
//
//	private void removeQuantitativeWidgets(){
//	    if(sectionMeasurementUnits!=null){
//	        removeElementsAndControls(sectionMeasurementUnits);
//	        removeElementsAndControls(sectionStatisticalMeasures);
//	        removeElementsAndControls(sectionModifierVocabularies);
//	    }
//	}
//
//	private void activateCategoricalWidgets(){
//	    //disable quantitative widgeets
//	    supportsQuantitativeData.setSelection(false);
//	    getEntity().getTerm().setSupportsQuantitativeData(false);
//	    removeQuantitativeWidgets();
//
//	    //states
//	    sectionStateVocabularies = formFactory.createStateVocabulariesSection(getConversationHolder(),
//                parentFormElement, StoreUtil.getSectionStyle(StateVocabularyCollectionSection.class, getEntity().getClass().getCanonicalName(), true));
//        sectionStateVocabularies.setLayoutData(LayoutConstants.FILL_HORIZONTALLY(2, 1));
//        sectionStateVocabularies.setEntity(getEntity().getTerm());
//
//        //modifiers
//        sectionModifierVocabularies = formFactory.createRecommendedModifierVocabulariesCollectionSection(
//                getConversationHolder(), parentFormElement, StoreUtil.getSectionStyle(RecommendedModifierVocabulariesCollectionSection.class, getEntity().getClass().getCanonicalName(), true));
//        sectionModifierVocabularies.setLayoutData(LayoutConstants.FILL_HORIZONTALLY(2, 1));
//        sectionModifierVocabularies.setEntity(getEntity().getTerm());
//	}
//
//	private void removeCategoricalWidgets(){
//	    if(sectionStateVocabularies!=null){
//	        removeElementsAndControls(sectionStateVocabularies);
//	        removeElementsAndControls(sectionModifierVocabularies);
//	    }
//	}

}
