// $Id$
/**
* Copyright (C) 2018 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.ui.section.feature;

import java.util.Collection;
import java.util.Comparator;

import org.eclipse.core.runtime.IStatus;
import org.eclipse.jface.wizard.WizardDialog;

import eu.etaxonomy.cdm.api.conversation.ConversationHolder;
import eu.etaxonomy.cdm.model.description.FeatureState;
import eu.etaxonomy.cdm.model.term.TermNode;
import eu.etaxonomy.cdm.persistence.dto.FeatureStateDto;
import eu.etaxonomy.cdm.persistence.dto.TermNodeDto;
import eu.etaxonomy.taxeditor.ui.element.CdmFormFactory;
import eu.etaxonomy.taxeditor.ui.element.ICdmFormElement;
import eu.etaxonomy.taxeditor.ui.section.AbstractEntityCollectionSection;
import eu.etaxonomy.taxeditor.ui.section.DefaultCdmBaseComparator;

/**
 * @author pplitzner
 * @date 06.03.2018
 *
 */
public class InapplicableIfEntityCollectionSectionForNode extends AbstractEntityCollectionSection<TermNode, FeatureState>{

    public InapplicableIfEntityCollectionSectionForNode(CdmFormFactory formFactory, ConversationHolder conversation,
            ICdmFormElement parentElement, int style) {
        super(formFactory, conversation, parentElement, "Inapplicable If", style);
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public Collection<FeatureState> getCollection(TermNode entity) {
        return entity.getInapplicableIf();
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public Comparator<FeatureState> getComparator() {
        return new DefaultCdmBaseComparator<>();
    }


    /**
     * {@inheritDoc}
     */
    @Override
    public FeatureState createNewElement() {
        FeatureStateWizard wizard = new FeatureStateWizard(TermNodeDto.fromNode(getEntity(), null), formFactory);
        WizardDialog dialog = new WizardDialog(getLayoutComposite().getShell(), wizard);
        int status = dialog.open();
        if(status == IStatus.OK) {
            return wizard.getFeatureState();
        }
        return null;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public void addElement(FeatureState element) {
        getEntity().getInapplicableIf().add(element);
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public FeatureState addExisting() {
        return null;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public boolean allowAddExisting() {
        return false;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public void removeElement(FeatureState element) {
        getEntity().getInapplicableIf().remove(element);
    }

    /** {@inheritDoc} */
    @Override
    public String getEmptyString() {
        return "No feature state added yet.";
    }

    /** {@inheritDoc} */
    @Override
    protected String getTooltipString() {
        return "Add a feature state";
    }

}
