/**
 *
 */
package eu.etaxonomy.taxeditor.ui.section.key;

import eu.etaxonomy.cdm.model.description.PolytomousKey;
import eu.etaxonomy.taxeditor.store.StoreUtil;
import eu.etaxonomy.taxeditor.ui.element.CdmFormFactory;
import eu.etaxonomy.taxeditor.ui.element.CdmPropertyChangeEvent;
import eu.etaxonomy.taxeditor.ui.element.ICdmFormElement;
import eu.etaxonomy.taxeditor.ui.element.LayoutConstants;
import eu.etaxonomy.taxeditor.ui.element.NumberWithLabelElement;
import eu.etaxonomy.taxeditor.ui.element.TextWithLabelElement;
import eu.etaxonomy.taxeditor.ui.section.AbstractCdmDetailElement;
import eu.etaxonomy.taxeditor.ui.section.occurrence.GeoScopePolyKeyDetailSection;

/**
 * @author n.hoffmann
 *
 */
public class PolytomousKeyDetailElement extends
		AbstractCdmDetailElement<PolytomousKey> {

    private TextWithLabelElement textLabel;
    private NumberWithLabelElement numberStartNumber;
    private TaxonomicScopeSection sectionTaxonomicScope;
    private GeoScopePolyKeyDetailSection sectionGeoScopes;
    private ScopeRestrictionSection sectionScopeRestriction;

	public PolytomousKeyDetailElement(CdmFormFactory formFactory,
			ICdmFormElement formElement) {
		super(formFactory, formElement);
	}

	@Override
	protected void createControls(ICdmFormElement formElement,
			PolytomousKey entity, int style) {
	    textLabel = formFactory.createTextWithLabelElement(formElement, "Title", entity.getTitleCache(), style);

	    numberStartNumber = formFactory.createFloatTextWithLabelElement(formElement, "Start Number", entity.getStartNumber(), style);

	    sectionTaxonomicScope = formFactory.createTaxonomicScopeSection(getConversationHolder(), formElement, StoreUtil.getSectionStyle(TaxonomicScopeSection.class, entity.getClass().getCanonicalName()));
	    sectionTaxonomicScope.setLayoutData(LayoutConstants.FILL_HORIZONTALLY(2, 1));
	    sectionTaxonomicScope.setEntity(entity);

	    sectionGeoScopes = formFactory.createGeoScopePolyKeyDetailSection(getConversationHolder(), formElement, StoreUtil.getSectionStyle(GeoScopePolyKeyDetailSection.class, entity.getClass().getCanonicalName()));
	    sectionGeoScopes.setLayoutData(LayoutConstants.FILL_HORIZONTALLY(2, 1));
	    sectionGeoScopes.setEntity(entity);

	    sectionScopeRestriction = formFactory.createScopeRestrictionSection(getConversationHolder(), formElement, StoreUtil.getSectionStyle(ScopeRestrictionSection.class, entity.getClass().getCanonicalName()));
	    sectionScopeRestriction.setLayoutData(LayoutConstants.FILL_HORIZONTALLY(2, 1));
	    sectionScopeRestriction.setEntity(entity);
	}

	public String getText() {
	    return textLabel.getText();
	}

	@Override
	public void handleEvent(Object eventSource) {
		if(eventSource==textLabel){
		    getEntity().setTitleCache(textLabel.getText(), true);
		}
		else if(eventSource==numberStartNumber){
			int startNumber = numberStartNumber.getInteger() == null? 0 : numberStartNumber.getInteger();
		    getEntity().setStartNumber(startNumber);
		}
		firePropertyChangeEvent(new CdmPropertyChangeEvent(this, null));
	}
}
