/**
 *
 */
package eu.etaxonomy.taxeditor.ui.section.media;

import java.util.Collection;
import java.util.Comparator;

import eu.etaxonomy.cdm.api.conversation.ConversationHolder;
import eu.etaxonomy.cdm.model.media.IdentifiableMediaEntity;
import eu.etaxonomy.cdm.model.media.Media;
import eu.etaxonomy.taxeditor.ui.dialog.selection.MediaSelectionDialog;
import eu.etaxonomy.taxeditor.ui.element.CdmFormFactory;
import eu.etaxonomy.taxeditor.ui.element.ICdmFormElement;
import eu.etaxonomy.taxeditor.ui.section.AbstractEntityCollectionSection;
import eu.etaxonomy.taxeditor.ui.section.DefaultCdmBaseComparator;

/**
 * <p>MediaSection class.</p>
 *
 * @author nho
 */
public class MediaSection extends AbstractEntityCollectionSection<IdentifiableMediaEntity, Media>{

	/**
	 * <p>Constructor for MediaSection.</p>
	 *
	 * @param parentElement a {@link eu.etaxonomy.taxeditor.ui.element.ICdmFormElement} object.
	 * @param style a int.
	 * @param cdmFormFactory a {@link eu.etaxonomy.taxeditor.ui.element.CdmFormFactory} object.
	 * @param conversation a {@link eu.etaxonomy.cdm.api.conversation.ConversationHolder} object.
	 */
	public MediaSection(CdmFormFactory cdmFormFactory, ConversationHolder conversation, ICdmFormElement parentElement, int style) {
		super(cdmFormFactory, conversation, parentElement, "Media", style);
	}

	@Override
	public Media createNewElement() {
		return Media.NewInstance();
	}

	@Override
	public Collection<Media> getCollection(IdentifiableMediaEntity entity) {
		return entity.getMedia();
	}

	@Override
	public Comparator<Media> getComparator() {
        return new DefaultCdmBaseComparator<>();
	}

	@Override
	public String getEmptyString() {
		return "No media yet.";
	}

	@Override
	protected String getTooltipString() {
		return "Create new media";
	}

	@Override
	public void addElement(Media element) {
		getEntity().addMedia(element);
	}

	@Override
	public void removeElement(Media element) {
		getEntity().removeMedia(element);
	}

    @Override
    public Media addExisting() {
        return MediaSelectionDialog.select(getShell(), null);
    }

    @Override
    public boolean allowAddExisting() {
        return true;
    }
}
