/**
 * Copyright (C) 2007 EDIT
 * European Distributed Institute of Taxonomy
 * http://www.e-taxonomy.eu
 *
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * See LICENSE.TXT at the top of this package for the full license terms.
 */
package eu.etaxonomy.taxeditor.ui.section.name;

import eu.etaxonomy.cdm.model.agent.TeamOrPersonBase;
import eu.etaxonomy.cdm.model.name.INonViralName;
import eu.etaxonomy.cdm.model.name.IZoologicalName;
import eu.etaxonomy.cdm.model.name.NomenclaturalCode;
import eu.etaxonomy.cdm.model.name.TaxonNameFactory;
import eu.etaxonomy.taxeditor.event.EventUtility;
import eu.etaxonomy.taxeditor.event.WorkbenchEventConstants;
import eu.etaxonomy.taxeditor.preference.NameDetailsConfigurator;
import eu.etaxonomy.taxeditor.preference.PreferencesUtil;
import eu.etaxonomy.taxeditor.ui.element.CdmFormFactory;
import eu.etaxonomy.taxeditor.ui.element.CdmPropertyChangeEvent;
import eu.etaxonomy.taxeditor.ui.element.ICdmFormElement;
import eu.etaxonomy.taxeditor.ui.element.NumberWithLabelElement;
import eu.etaxonomy.taxeditor.ui.element.ToggleableTextElement;
import eu.etaxonomy.taxeditor.ui.section.AbstractIdentifiableEntityDetailElement;
import eu.etaxonomy.taxeditor.ui.selection.EntitySelectionElement;

/**
 * AuthorshipDetailElement class.
 *
 * @author n.hoffmann
 * @created Mar 4, 2010
 */
public class AuthorshipDetailElement
		extends AbstractIdentifiableEntityDetailElement<INonViralName> {

    @SuppressWarnings("rawtypes")
    private EntitySelectionElement<TeamOrPersonBase> selectionExBasionymAuthor;
    @SuppressWarnings("rawtypes")
    private EntitySelectionElement<TeamOrPersonBase> selectionBasionymAuthor;
    @SuppressWarnings("rawtypes")
    private EntitySelectionElement<TeamOrPersonBase> selectionExCombinationAuthor;
    @SuppressWarnings("rawtypes")
    private EntitySelectionElement<TeamOrPersonBase> selectionCombinationAuthor;

    private NumberWithLabelElement text_publicationYear;
    private NumberWithLabelElement text_originalPublicationYear;

    /**
	 * Constructor for AuthorshipDetailElement.
	 */
	public AuthorshipDetailElement(CdmFormFactory cdmFormFactory,
			ICdmFormElement formElement, int style) {
		super(cdmFormFactory, formElement);
	}

	@Override
	protected void createControls(ICdmFormElement formElement,
			INonViralName entity, int style) {
	    NameDetailsConfigurator config = PreferencesUtil.getPreferredNameDetailsConfiguration();
	    if (config == null || config.isAuthorCacheActivated()){
	        toggleable_cache = formFactory.createToggleableTextField(this,
				"Authorship Cache", entity.getAuthorshipCache(),
				entity.isProtectedAuthorshipCache(), style);
	        addElement(toggleable_cache);
	    }
	    NomenclaturalCode code = entity.getNameType();

	    if (config == null || config.isAuthorshipSectionActivated()){
            selectionCombinationAuthor = formFactory.createSelectionElementWithAbbreviatedTitle(TeamOrPersonBase.class,
    				getConversationHolder(),
    				formElement, "Author",
    				entity.getCombinationAuthorship(),
    				EntitySelectionElement.DELETABLE | EntitySelectionElement.SELECTABLE | EntitySelectionElement.EDITABLE, style);
    		addElement(selectionCombinationAuthor);
    		selectionExCombinationAuthor = formFactory.createSelectionElementWithAbbreviatedTitle(TeamOrPersonBase.class,
    				getConversationHolder(),
    				formElement, "Ex Author",
    				entity.getExCombinationAuthorship(),
    				EntitySelectionElement.DELETABLE | EntitySelectionElement.SELECTABLE | EntitySelectionElement.EDITABLE, style);
    		addElement(selectionExCombinationAuthor);
    		if (code != null){
                if (code.equals(NomenclaturalCode.ICZN)){
                    text_publicationYear = formFactory.createFloatTextWithLabelElement(formElement, "Publication Year", ((IZoologicalName)entity).getPublicationYear(), style);
                }
            }
    		selectionBasionymAuthor = formFactory.createSelectionElementWithAbbreviatedTitle(TeamOrPersonBase.class, getConversationHolder(),
                    formElement, "Basionym Author", entity.getBasionymAuthorship(),
                    EntitySelectionElement.DELETABLE | EntitySelectionElement.SELECTABLE | EntitySelectionElement.EDITABLE, style);
    		addElement(selectionBasionymAuthor);
    		selectionExBasionymAuthor = formFactory.createSelectionElementWithAbbreviatedTitle(TeamOrPersonBase.class, getConversationHolder(),
    		        formElement, "Ex Basionym Author",entity.getExBasionymAuthorship(),
    		        EntitySelectionElement.DELETABLE | EntitySelectionElement.SELECTABLE | EntitySelectionElement.EDITABLE, style);

    		addElement(selectionExBasionymAuthor);
    		if (code != null && code.equals(NomenclaturalCode.ICZN)){
    		    text_originalPublicationYear = formFactory.createFloatTextWithLabelElement(formElement, "Orig. Publication Year", ((IZoologicalName)entity).getOriginalPublicationYear(), style);
    	    }
	    }

	    //put to Cache
	    registerCacheRelevance(selectionCombinationAuthor);
        registerCacheRelevance(selectionBasionymAuthor);
        registerCacheRelevance(selectionExCombinationAuthor);
        registerCacheRelevance(selectionExBasionymAuthor);
        registerCacheRelevance(text_publicationYear);
        registerCacheRelevance(text_originalPublicationYear);

        registerCacheRelevance(toggleable_cache, cacheRelevanceHelper.getDependenciesArray());

	}

    @Override
    public void addDependsOnCache(ToggleableTextElement toggleElement) {
        super.addDependsOnCache(toggleElement);
        //register my cache on higher cache (if not null yet)
        registerCacheRelevance(toggleable_cache, toggleElement);
    }

    @Override
	public void updateContent() {
		if (getEntity() == null) {
			setEntity(TaxonNameFactory.NewNonViralInstance(null));
		}

		super.updateContent();
		if (toggleable_cache != null){
    		toggleable_cache.setCacheEnabled(getEntity().isProtectedAuthorshipCache());
    		updateCacheRelevance();
		}
	}

	@Override
	public void handleEvent(Object eventSource) {
		if (eventSource == toggleable_cache) {
			getEntity().setAuthorshipCache(toggleable_cache.getText(),
					toggleable_cache.getState());
			updateCacheRelevance();
		} else if (eventSource == selectionCombinationAuthor) {
			getEntity().setCombinationAuthorship(
					selectionCombinationAuthor.getSelection());
		} else if (eventSource == selectionExCombinationAuthor) {
			getEntity().setExCombinationAuthorship(
					selectionExCombinationAuthor.getSelection());
		} else if (eventSource == selectionBasionymAuthor) {
			getEntity().setBasionymAuthorship(
					selectionBasionymAuthor.getSelection());
		} else if (eventSource == selectionExBasionymAuthor) {
			getEntity().setExBasionymAuthorship(
					selectionExBasionymAuthor.getSelection());
		} else if (eventSource == text_originalPublicationYear) {
            ((IZoologicalName)getEntity()).setOriginalPublicationYear(text_originalPublicationYear.getInteger());
        } else if (eventSource == text_publicationYear) {
            ((IZoologicalName)getEntity()).setPublicationYear(text_publicationYear.getInteger());
        }

//		if (eventSource != toggleable_cache) {
//			toggleable_cache.setText(getEntity().getAuthorshipCache());
//		}

//		we have to notify the parent if this is embedded in the nonviral name
//		section
//		maybe we can handle this a little bit more elegant
//		if (getParentElement() instanceof AbstractCdmDetailSection)
			firePropertyChangeEvent(new CdmPropertyChangeEvent(
					getParentElement(), null));
			EventUtility.postEvent(WorkbenchEventConstants.REFRESH_NAME_EDITOR, this.getEntity());
	}

	@Override
	public void updateToggleableCacheField() {
	    if (toggleable_cache != null){
    		if(! getEntity().isProtectedAuthorshipCache()){
    			toggleable_cache.setText(getEntity().getAuthorshipCache());
    		}
	    }
	}
}