/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.ui.section.name;

import org.eclipse.jface.util.PropertyChangeEvent;

import eu.etaxonomy.cdm.model.name.INonViralName;
import eu.etaxonomy.cdm.model.name.Rank;
import eu.etaxonomy.taxeditor.ui.element.CdmFormFactory;
import eu.etaxonomy.taxeditor.ui.element.CheckboxElement;
import eu.etaxonomy.taxeditor.ui.element.ICdmFormElement;
import eu.etaxonomy.taxeditor.ui.section.AbstractCdmDetailElement;

/**
 * @author n.hoffmann
 * @created Jan 5, 2011
 */
public class HybridDetailElement extends AbstractCdmDetailElement<INonViralName> {

	private CheckboxElement checkboxMonomHybrid;
	private CheckboxElement checkboxBinomHybrid;
	private CheckboxElement checkboxTrinomHybrid;
	private CheckboxElement checkboxHybridFormula;

	public HybridDetailElement(CdmFormFactory formFactory,
			ICdmFormElement formElement) {
		super(formFactory, formElement);
	}

	@Override
	protected void createControls(ICdmFormElement formElement, INonViralName entity,
			int style) {
	    boolean enableBinom = false;
	    boolean enableTrinom = false;
	    if (entity.getRank() != null){
    	    if (entity.getRank().isLower(Rank.GENUS())){
    	        enableBinom = true;
    	    }

    	    if (entity.getRank().isLower(Rank.SPECIES())){
                enableTrinom = true;
            }
	    }
		checkboxMonomHybrid = formFactory.createCheckbox(formElement, "Monom hybrid", entity.isMonomHybrid(), style);
		checkboxBinomHybrid = formFactory.createCheckbox(formElement, "Binom hybrid", entity.isBinomHybrid(), style);
		checkboxBinomHybrid.setEnabled(enableBinom);
		checkboxTrinomHybrid = formFactory.createCheckbox(formElement, "Trinom hybrid", entity.isTrinomHybrid(), style);
		checkboxTrinomHybrid.setEnabled(enableTrinom);
		checkboxHybridFormula = formFactory.createCheckbox(formElement, "Hybrid formula", entity.isHybridFormula(), style);
		checkboxHybridFormula.setEnabled(false);//read-only

        registerCacheRelevance(checkboxMonomHybrid, cacheRelevanceHelper.getDependenciesArray());
        registerCacheRelevance(checkboxBinomHybrid, cacheRelevanceHelper.getDependenciesArray());
        registerCacheRelevance(checkboxTrinomHybrid, cacheRelevanceHelper.getDependenciesArray());
        registerCacheRelevance(checkboxHybridFormula, cacheRelevanceHelper.getDependenciesArray());

	}

	@Override
	public void handleEvent(Object eventSource) {
		if(eventSource == checkboxMonomHybrid){
			getEntity().setMonomHybrid(checkboxMonomHybrid.getSelection());
		}
		else if(eventSource == checkboxBinomHybrid){
			getEntity().setBinomHybrid(checkboxBinomHybrid.getSelection());
		}
		else if(eventSource == checkboxTrinomHybrid){
			getEntity().setTrinomHybrid(checkboxTrinomHybrid.getSelection());
		}
	}

    @Override
    public void propertyChange(PropertyChangeEvent event) {
        if (event == null) {
            return;
        }

        Object obj = event.getSource();
        handleEvent(obj);
        if (obj instanceof NameDetailElement){
            if (getEntity().getRank() != null){
                checkboxBinomHybrid.setEnabled(getEntity().getRank().isLower(Rank.GENUS()));
                checkboxTrinomHybrid.setEnabled(getEntity().getRank().isLower(Rank.SPECIES()));
            }
        }
    }
}