/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.ui.section.name;

import org.eclipse.jface.viewers.ISelectionProvider;

import eu.etaxonomy.cdm.api.conversation.ConversationHolder;
import eu.etaxonomy.cdm.hibernate.HibernateProxyHelper;
import eu.etaxonomy.cdm.model.name.INonViralName;
import eu.etaxonomy.cdm.model.taxon.TaxonBase;
import eu.etaxonomy.taxeditor.ui.element.CdmFormFactory;
import eu.etaxonomy.taxeditor.ui.element.ICdmFormElement;
import eu.etaxonomy.taxeditor.ui.section.AbstractCdmDetailElement;
import eu.etaxonomy.taxeditor.ui.section.AbstractCdmDetailSection;
import eu.etaxonomy.taxeditor.ui.section.ITaxonBaseDetailSection;

/**
 * <p>NameDetailSection class.</p>
 *
 * @author n.hoffmann
 * @created 05.06.2009
 */
public class NameDetailSection 
		extends AbstractCdmDetailSection<INonViralName> 
		implements ITaxonBaseDetailSection{

	private TaxonBase<?> taxonBase;

	/**
	 * <p>Constructor for NameDetailSection.</p>
	 *
	 * @param parentElement a {@link eu.etaxonomy.taxeditor.ui.element.ICdmFormElement} object.
	 * @param style a int.
	 * @param cdmFormFactory a {@link eu.etaxonomy.taxeditor.ui.element.CdmFormFactory} object.
	 * @param conversation a {@link eu.etaxonomy.cdm.api.conversation.ConversationHolder} object.
	 * @param selectionProvider a {@link org.eclipse.jface.viewers.ISelectionProvider} object.
	 */
	public NameDetailSection(CdmFormFactory cdmFormFactory, ConversationHolder conversation, ICdmFormElement parentElement,
			ISelectionProvider selectionProvider, int style) {
		super(cdmFormFactory, conversation, parentElement, selectionProvider, style);
	}

	@Override
    public void setTaxonBase(TaxonBase entity) {
		this.taxonBase = entity;
		INonViralName name = (INonViralName) HibernateProxyHelper.deproxy(entity.getName());
		setEntity(name);
	}

	@Override
	public String getHeading() {
		return "Scientific Name";
	}

	@Override
	protected void setSectionTitle() {
		if(!isDisposed()){
			String title = "";
			if(getEntity() != null){
				 title = ": " + getEntity().getNameCache();
			}
			this.setText(getHeading() + title);
		}
	}

	@Override
	public TaxonBase<?> getTaxonBase() {
		return taxonBase;
	}

	@Override
	protected AbstractCdmDetailElement<INonViralName> createCdmDetailElement(
			AbstractCdmDetailSection<INonViralName> parentElement, int style) {
	    return formFactory.createNameDetailElement(parentElement, style);

	}
	
	public void updateContent(){
		((NameDetailElement)this.detailElement).updateContent();
	}
}
