/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.ui.section.name;

import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Color;

import eu.etaxonomy.cdm.model.name.NameRelationship;
import eu.etaxonomy.cdm.model.name.NameRelationshipType;
import eu.etaxonomy.taxeditor.model.MessagingUtils;
import eu.etaxonomy.taxeditor.preference.NameDetailsConfigurator;
import eu.etaxonomy.taxeditor.preference.PreferencesUtil;
import eu.etaxonomy.taxeditor.ui.combo.term.TermComboElement;
import eu.etaxonomy.taxeditor.ui.element.AbstractFormSection;
import eu.etaxonomy.taxeditor.ui.element.CdmFormFactory;
import eu.etaxonomy.taxeditor.ui.element.ICdmFormElement;
import eu.etaxonomy.taxeditor.ui.element.LabelElement;
import eu.etaxonomy.taxeditor.ui.element.LayoutConstants;
import eu.etaxonomy.taxeditor.ui.section.supplemental.AbstractSourcedEntityElement;
import eu.etaxonomy.taxeditor.ui.selection.EntitySelectionElement;

/**
 * <p>NameRelationshipDetailElement class.</p>
 *
 * @author n.hoffmann
 * @created Mar 18, 2010
 */
public class NameRelationshipDetailElement extends AbstractSourcedEntityElement<NameRelationship>{

	private RuleConsideredElement ruleConsidered;
	private EntitySelectionElement<?> selection_toName;
	private TermComboElement<NameRelationshipType> combo_relationshipType;
	private LabelElement labelElement;

//	private OriginalSourceElement sourceElement;

	/**
	 * <p>Constructor for NameRelationshipDetailElement.</p>
	 *
	 * @param cdmFormFactory a {@link eu.etaxonomy.taxeditor.ui.element.CdmFormFactory} object.
	 * @param formElement a {@link eu.etaxonomy.taxeditor.ui.element.AbstractFormSection} object.
	 * @param entity a {@link eu.etaxonomy.cdm.model.name.NameRelationship} object.
	 * @param removeListener a {@link org.eclipse.swt.events.SelectionListener} object.
	 * @param style a int.
	 */
	public NameRelationshipDetailElement(CdmFormFactory cdmFormFactory,
	        AbstractFormSection formElement,
            NameRelationship entity, SelectionListener removeListener, int style){
	    super(cdmFormFactory, formElement, entity, removeListener, "Source", style);

	}

	@Override
	public void createControls(ICdmFormElement element, int style) {

	    NameDetailsConfigurator config = PreferencesUtil.getPreferredNameDetailsConfiguration();
		labelElement = formFactory.createLabel(element, null);

		if (config == null || config.isNameRelationsRuleConsideredActivated()){
		    ruleConsidered = formFactory.createRuleConsideredElement(
		            element, "Rule Considered", config == null
		            || config.isNameRelationsRuleConsideredCodeEditionActivated(), style);
		}
		LabelElement spacer = formFactory.createLabel(element, null);
		spacer.getLayoutComposite().setLayoutData(LayoutConstants.FILL_HORIZONTALLY(2, 1));
		addElement(spacer);
		super.createControls(element, style);
		setSourceLabel("Source");
		if (entity != null){
			setEntity(entity);
		}
	}

	@Override
	public void setEntity(NameRelationship entity) {
		if (labelElement != null){
		   String text = (entity.getFromName() == null? "-":entity.getFromName().getTitleCache())
			        + " " +(entity.getType()==null?"(no type)":entity.getType().getLabel())
			        + " " + (entity.getToName()==null?"-":entity.getToName().getTitleCache());
			labelElement.setText(text);
			if (ruleConsidered != null){
			    ruleConsidered.setElement(entity);
			}
		}

		super.setEntity(entity);
	}

	@Override
	public void handleEvent(Object eventSource) {
	    if(eventSource == combo_relationshipType){
			getEntity().setType(combo_relationshipType.getSelection());
		}
		else if(eventSource == selection_toName){
			MessagingUtils.error(getClass(), "No API call available.", null);
		}


	}

	@Override
	public void setBackground(Color color) {
		if(labelElement != null) {
            labelElement.setBackground(color);
        }
		super.setBackground(color);
	}


}
