/**
 * Copyright (C) 2007 EDIT
 * European Distributed Institute of Taxonomy
 * http://www.e-taxonomy.eu
 *
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * See LICENSE.TXT at the top of this package for the full license terms.
 */
package eu.etaxonomy.taxeditor.ui.section.name;

import eu.etaxonomy.cdm.model.name.IBacterialName;
import eu.etaxonomy.cdm.model.name.NomenclaturalCode;
import eu.etaxonomy.cdm.model.name.TaxonName;
import eu.etaxonomy.taxeditor.l10n.Messages;
import eu.etaxonomy.taxeditor.model.MessagingUtils;
import eu.etaxonomy.taxeditor.preference.NameDetailsConfigurator;
import eu.etaxonomy.taxeditor.preference.PreferencesUtil;
import eu.etaxonomy.taxeditor.store.StoreUtil;
import eu.etaxonomy.taxeditor.ui.combo.EnumComboElement;
import eu.etaxonomy.taxeditor.ui.element.CdmFormFactory;
import eu.etaxonomy.taxeditor.ui.element.ICdmFormElement;
import eu.etaxonomy.taxeditor.ui.element.LayoutConstants;
import eu.etaxonomy.taxeditor.ui.element.LsidWithExceptionLabelElement;
import eu.etaxonomy.taxeditor.ui.element.TextWithLabelElement;
import eu.etaxonomy.taxeditor.ui.section.AbstractCdmDetailSection;
import eu.etaxonomy.taxeditor.ui.section.AbstractIdentifiableEntityDetailElement;

/**
 * @author n.hoffmann
 * @created May 20, 2010
 */
public class NonViralNameDetailElement
		extends AbstractIdentifiableEntityDetailElement<TaxonName>  {

	private NameDetailSection section_name;
	private AuthorshipDetailSection section_author;
	private EnumComboElement<NomenclaturalCode> combo_nomenclaturalCode;
	private HybridDetailSection section_hybrid;
	private LsidWithExceptionLabelElement textLsid;
	private TextWithLabelElement text_nameApprobation;

	public NonViralNameDetailElement(CdmFormFactory formFactory,
			ICdmFormElement formElement) {
		super(formFactory, formElement);
	}

	@Override
    protected void createControls(ICdmFormElement formElement, final TaxonName entity, int style) {
	    NameDetailsConfigurator config = PreferencesUtil.getPreferredNameDetailsConfiguration();

        if (config == null){
            toggleable_cache = formFactory.createToggleableTextField(formElement, "Cache", entity.getTitleCache(), entity.isProtectedTitleCache() || entity.isProtectedFullTitleCache(), style);

            combo_nomenclaturalCode = formFactory.createEnumComboElement(NomenclaturalCode.class, formElement, style);
            combo_nomenclaturalCode.setSelection(entity.getNameType());
            if (entity.getNameType().equals(NomenclaturalCode.ICNP)){
            	text_nameApprobation = formFactory.createTextWithLabelElement(formElement, "Name Approbation", entity.getNameApprobation(),style);
            }
            textLsid = formFactory.createLsidWithExceptionLabelElement(formElement, "Lsid", entity.getLsid(), style);
        }else{
        	if (config.isNameCacheActivated()){
        		toggleable_cache = formFactory.createToggleableTextField(formElement, "Cache", entity.getTitleCache(), entity.isProtectedTitleCache() || entity.isProtectedFullTitleCache(), style);
          	}
        	if (config.isNomenclaturalCodeActived()){
        		combo_nomenclaturalCode = formFactory.createEnumComboElement(NomenclaturalCode.class, formElement, style);
        		combo_nomenclaturalCode.setSelection(entity.getNameType());
        	}
        	if (entity.getNameType().equals(NomenclaturalCode.ICNP)){
        		if (config.isNameApprobiationActivated()){
        			text_nameApprobation = formFactory.createTextWithLabelElement(formElement, "Name Approbation", entity.getNameApprobation(),style);
        		}
        	}
        	if (config.isLSIDActivated()){
        		textLsid = formFactory.createLsidWithExceptionLabelElement(formElement, "Lsid", entity.getLsid(), style);
        	}
        }

	    section_name = formFactory.createNameDetailSection(getConversationHolder(), formElement, null, StoreUtil.getSectionStyle(NameDetailSection.class, entity.getClass().getCanonicalName(), true));
	    section_name.setLayoutData(LayoutConstants.FILL_HORIZONTALLY(2, 1));
	    addControl(section_name);
	    addElement(section_name);
//	    if (isAdvancedView || PreferencesUtil.getBooleanValue(IPreferenceKeys.SHOW_NAME_DETAILS_SECTION_AUTHORSHIP)){
    	    section_author = formFactory.createAuthorshipDetailSection(getConversationHolder(), formElement, null, StoreUtil.getSectionStyle(AuthorshipDetailSection.class, entity.getClass().getCanonicalName(), true));
    	    section_author.setLayoutData(LayoutConstants.FILL_HORIZONTALLY(2, 1));
    	    addControl(section_author);
    	    addElement(section_author);
//	    }
	    //TODO RL
	    if (config == null || config.isHybridActivated()){
    	   section_hybrid = formFactory.createHybridDetailSection(getConversationHolder(), formElement, null, StoreUtil.getSectionStyle(HybridDetailSection.class, entity.getClass().getCanonicalName()));
    	   section_hybrid.setLayoutData(LayoutConstants.FILL_HORIZONTALLY(2, 1));
    	   addControl(section_hybrid);
    	   addElement(section_hybrid);
    	}

	    //cache relevance
	    registerCacheRelevance(section_name);
	    registerCacheRelevance(section_author);
	    registerCacheRelevance(combo_nomenclaturalCode);
	    registerCacheRelevance(section_hybrid);
    }

	@Override
	protected void updateContent() {
		super.updateContent();

		if(toggleable_cache != null){
		    toggleable_cache.setCacheEnabled(getEntity().isProtectedTitleCache() || getEntity().isProtectedFullTitleCache());
		    updateCacheRelevance();
		}
	}

	@Override
	public void setEntity(TaxonName entity) {
		super.setEntity(entity);
		if (section_name!=null) {
            section_name.setEntity(entity);
        }
		if (section_author!=null) {
            section_author.setEntity(entity);
        }
		if (section_hybrid!=null) {
            section_hybrid.setEntity(entity);
        }
	}

	@Override
	public void removeElements() {
		super.removeElements();
		if (section_name != null) {
			section_name.removeElements();
			removeControl(section_name);
			section_name.dispose();
			section_name = null;
		}
		if (section_author != null) {
			section_author.removeElements();
			removeControl(section_author);
			section_author.dispose();
			section_author = null;
		}
	}

	@Override
	public void handleEvent(Object eventSource) {
		if (eventSource == toggleable_cache) {
			handleToggleableCacheField();
			// we never want the fullTitleCache to be protected since we only
			// use it for
			// initiating the free text name editor
			getEntity().setProtectedFullTitleCache(false);
		} else if (eventSource == section_name || eventSource == section_author) {
			if (getParentElement() instanceof AbstractCdmDetailSection) {
                ((AbstractCdmDetailSection<?>) getParentElement()).updateTitle();
            }
			if (toggleable_cache != null){
    			if (!toggleable_cache.getState()) {
    				toggleable_cache.setText(getEntity().getTitleCache());
    			}
			}
		} else if (eventSource == section_name) {
			section_name.setEntity(getEntity());
			getLayoutComposite().layout();
		} else if(eventSource==textLsid){
		    getEntity().setLsid(textLsid.parseText());
		} else if (eventSource == combo_nomenclaturalCode){
			boolean change = checkForDeletableParts(combo_nomenclaturalCode.getSelection());
			if (change){
				getEntity().setNameType(combo_nomenclaturalCode.getSelection());

				this.updateContent();
	            //this is needed for the "new" wizards in order to redraw correctly
	            StoreUtil.reflowParentScrolledForm(getLayoutComposite(), true);
	            if (section_hybrid != null){
	            	section_hybrid.setEntity(getEntity());
	            }
	            if (section_author != null){
	            	section_author.setEntity(getEntity());
	            }
	            if (section_name != null){
	            	section_name.setEntity(getEntity());
	            }
			}else{
				combo_nomenclaturalCode.setSelection(getEntity().getNameType());
			}
		} else if(eventSource == this.text_nameApprobation){
            ((IBacterialName)getEntity()).setNameApprobation(text_nameApprobation.getText());
        }
	}

	private boolean checkForDeletableParts(NomenclaturalCode selection) {
		NomenclaturalCode actualCode =  getEntity().getNameType();
		if (selection.equals(actualCode)){
			return true;
		}
		if (actualCode.equals(NomenclaturalCode.ICZN) ){
			if(getEntity().getOriginalPublicationYear() != null || getEntity().getPublicationYear() != null || getEntity().getBreed() != null){
				if (MessagingUtils.confirmDialog(Messages.NonViralNameDetails_confirmDeleteOfZoologicalNameParts, Messages.NonViralNameDetails_descriptionDeleteZoologicalNameParts)){
					getEntity().setOriginalPublicationYear(null);
					getEntity().setPublicationYear(null);
					getEntity().setBreed(null);
					return true;
				}
			}else {
				return true;
			}
		}
		if (actualCode.equals(NomenclaturalCode.ICNP)){
			if (getEntity().getNameApprobation() != null || getEntity().getSubGenusAuthorship() != null){
				if (MessagingUtils.confirmDialog(Messages.NonViralNameDetails_confirmDeleteOfBacterialNameParts, Messages.NonViralNameDetails_desciptionDeleteOfBacterialNameParts)){
					getEntity().setNameApprobation(null);
					getEntity().setSubGenusAuthorship(null);
					return true;
				}
			}else{
				return true;
			}
		}
		if (actualCode.equals(NomenclaturalCode.NonViral)){
		    return true;
        }
		if (actualCode.equals(NomenclaturalCode.Fungi)){
			if (getEntity().isAnamorphic() ){
				if (MessagingUtils.confirmDialog(Messages.NonViralNameDetails_confirmDeleteOfFungiNameParts, Messages.NonViralNameDetails_descriptionDeleteOfFungiNameParts)){
					getEntity().setAnamorphic(false);
					return true;
				}
			}else{
				return true;
			}
		}
		if (actualCode.equals(NomenclaturalCode.ICNCP)){
			if (getEntity().getCultivarEpithet() != null || getEntity().getCultivarGroupEpithet() != null){
				if (MessagingUtils.confirmDialog(Messages.NonViralNameDetails_confirmDeleteOfCultivarNameParts, Messages.NonViralNameDetails_descriptionDeleteOfCultivarNameParts)){
					getEntity().setCultivarEpithet(null);
					getEntity().setCultivarGroupEpithet(null);
					return true;
				}
			}else{
				return true;
			}
		}
		if (actualCode.equals(NomenclaturalCode.ICNAFP)){
			return true;
		}
		return false;
	}

	@Override
	protected void handleToggleableCacheField() {
        getEntity().setTitleCache(toggleable_cache.getText(), toggleable_cache.getState());
        updateCacheRelevance();
        updateToggleableCacheField();
    }
}