/**
 * Copyright (C) 2007 EDIT
 * European Distributed Institute of Taxonomy
 * http://www.e-taxonomy.eu
 *
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * See LICENSE.TXT at the top of this package for the full license terms.
 */
package eu.etaxonomy.taxeditor.ui.section.name;

import org.eclipse.jface.util.PropertyChangeEvent;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionListener;

import eu.etaxonomy.cdm.model.name.TextualTypeDesignation;
import eu.etaxonomy.taxeditor.ui.element.AbstractFormSection;
import eu.etaxonomy.taxeditor.ui.element.CdmFormFactory;
import eu.etaxonomy.taxeditor.ui.element.CheckboxElement;
import eu.etaxonomy.taxeditor.ui.element.ICdmFormElement;
import eu.etaxonomy.taxeditor.ui.element.MultilanguageTextElement;

/**
 * @author pplitzner
 * @since May 15, 2019
 */
public class TextTypeDesignationElement
        extends AbstractTypeDesignationElement<TextualTypeDesignation> {

    private MultilanguageTextElement multilanguageTextElement;
    private CheckboxElement checkIsVerbatim;

	public TextTypeDesignationElement(CdmFormFactory formFactory,
			AbstractFormSection section, TextualTypeDesignation entity,
			SelectionListener removeListener, int style) {
		super(formFactory, section, entity, removeListener, style);
	}

	@Override
	public void createControls(ICdmFormElement formElement, int style) {
        multilanguageTextElement = formFactory.createMultiLanguageTextElement(this, null, entity.getText(), 100, SWT.WRAP);
        checkIsVerbatim = formFactory.createCheckbox(formElement, "Is Verbatim", getEntity().isVerbatim(), style);
        super.createControls(formElement, style);
        if (entity != null){
        	setEntity(entity);
        }
	}

    @Override
    public void propertyChange(PropertyChangeEvent event) {
        if(event.getSource()==multilanguageTextElement){
            multilanguageTextElement.getMultilanguageText().values()
                    .forEach(languageString -> getEntity().putText(languageString));
        }
        super.propertyChange(event);
    }

	@Override
	public void setEntity(TextualTypeDesignation entity) {
	    super.setEntity(entity);
	    if (checkbox_notDesignated != null){
	    	removeNotDesignatedControls();
	    }
	}

	@Override
	public void handleEvent(Object eventSource) {
	    super.handleEvent(eventSource);
	    if(eventSource==checkIsVerbatim){
	        getEntity().setVerbatim(checkIsVerbatim.getSelection());
	    }
	}
}