/**
* Copyright (C) 2019 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.ui.section.name.type;


import java.util.Collection;
import java.util.stream.Collectors;

import org.eclipse.jface.wizard.Wizard;

import eu.etaxonomy.cdm.common.URI;
import eu.etaxonomy.cdm.model.name.SpecimenTypeDesignation;
import eu.etaxonomy.cdm.model.name.SpecimenTypeDesignationStatus;
import eu.etaxonomy.cdm.model.name.TaxonName;

/**
 * @author pplitzner
 * @since Mar 13, 2019
 *
 */
public class CloneTypeWizard extends Wizard{

    private CloneTypeWizardPage page;

    private final Collection<SpecimenTypeDesignation> typeDesignations;

    private final TaxonName taxonName;

    private SpecimenTypeDesignation baseTypeDesignation;

    private String accessionNumber;

    private String barcode;

    private String catalogNumber;

    private eu.etaxonomy.cdm.model.occurrence.Collection collection;

    private SpecimenTypeDesignationStatus typeStatus;

    private URI stableIdentifier;

    public CloneTypeWizard(TaxonName taxonName) {
        this.taxonName = taxonName;
        this.typeDesignations = taxonName.getTypeDesignations().stream()
                .filter(designation -> designation instanceof SpecimenTypeDesignation)
                .map(designation -> (SpecimenTypeDesignation)designation)
                .filter(specimenTypeDesignation->specimenTypeDesignation.getTypeSpecimen()!=null)
                .collect(Collectors.toList());
    }

    @Override
    public void addPages() {
        page = new CloneTypeWizardPage(typeDesignations);
        addPage(page);
    }

    @Override
    public boolean performFinish() {
        this.baseTypeDesignation = page.getBaseTypeDesignation();
        this.accessionNumber = page.getAccessionNumber();
        this.barcode = page.getBarcode();
        this.catalogNumber = page.getCatalogNumber();
        this.collection = page.getCollection();
        this.typeStatus = page.getTypeStatus();
        this.stableIdentifier = page.getStableIdentifier();
        return true;
    }

    public TaxonName getTaxonName() {
        return taxonName;
    }

    public SpecimenTypeDesignation getBaseTypeDesignation() {
        return baseTypeDesignation;
    }

    public String getAccessionNumber() {
        return accessionNumber;
    }

    public String getBarcode() {
        return barcode;
    }

    public String getCatalogNumber() {
        return catalogNumber;
    }

    public eu.etaxonomy.cdm.model.occurrence.Collection getCollection() {
        return collection;
    }

    public SpecimenTypeDesignationStatus getTypeStatus() {
        return typeStatus;
    }

    public URI getStableIdentifier() {
        return stableIdentifier;
    }

    public void setStableIdentifier(URI stableIdentifier) {
        this.stableIdentifier = stableIdentifier;
    }

}
