/**
* Copyright (C) 2019 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.ui.section.name.type;

import java.util.Collection;

import org.eclipse.jface.viewers.ArrayContentProvider;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.widgets.Composite;

import eu.etaxonomy.cdm.common.URI;
import eu.etaxonomy.cdm.model.name.SpecimenTypeDesignation;
import eu.etaxonomy.cdm.model.name.SpecimenTypeDesignationStatus;
import eu.etaxonomy.taxeditor.ui.dialog.selection.CollectionSelectionDialog;

/**
 * @author pplitzner
 * @since Mar 13, 2019
 *
 */
public class CloneTypeWizardPage extends WizardPage implements ISelectionChangedListener, ModifyListener {

    private CloneTypeWizardComposite composite;

    private eu.etaxonomy.cdm.model.occurrence.Collection selectedCollection = null;
    private SpecimenTypeDesignation baseDesignation = null;

    private final Collection<SpecimenTypeDesignation> typeDesignations;

    protected CloneTypeWizardPage(Collection<SpecimenTypeDesignation> typeDesignations) {
        super("Duplicate Type");
        setTitle("Duplicate Type");
        setDescription("Create a type duplicate based on an existing type specimen.");
        this.typeDesignations = typeDesignations;
    }

    @Override
    public void createControl(Composite parent) {

        composite = new CloneTypeWizardComposite(parent, SWT.NONE);

        // fill combo with available types
        composite.getComboViewerBaseType().setContentProvider(new ArrayContentProvider());
        composite.getComboViewerBaseType().setLabelProvider(new LabelProvider(){
            @Override
            public String getText(Object element) {
                if(element instanceof SpecimenTypeDesignation){
                    return ((SpecimenTypeDesignation) element).getTypeSpecimen().getTitleCache();
                }
                return super.getText(element);
            }
        });
        composite.getComboViewerBaseType().setInput(typeDesignations);
        if(typeDesignations.size()==1){
            SpecimenTypeDesignation next = typeDesignations.iterator().next();
            baseDesignation = next;
            composite.getComboViewerBaseType().setSelection(new StructuredSelection(next));
        }
        composite.getComboViewerBaseType().addSelectionChangedListener(e->{
            baseDesignation = (SpecimenTypeDesignation) ((StructuredSelection) e.getSelection()).getFirstElement();
            getWizard().getContainer().updateButtons();
        });

        composite.getBtnBrowseCollection().addSelectionListener(new SelectionAdapter() {
            @Override
            public void widgetSelected(SelectionEvent e) {
                selectedCollection = CollectionSelectionDialog.select(getShell(), selectedCollection);
                composite.getTxtCollection().setText(selectedCollection.getTitleCache());
                getWizard().getContainer().updateButtons();
            }
        });

        // add listeners for update buttons
        composite.getTxtAccNumber().addModifyListener(this);
        composite.getComboTypeStatus().addSelectionChangedListener(this);

        setControl(composite);
    }

    @Override
    public boolean isPageComplete() {
        return composite.getComboViewerBaseType().getSelection()!=null
                && selectedCollection!=null
                && composite.getComboTypeStatus().getSelection()!=null;
    }

    @Override
    public void selectionChanged(SelectionChangedEvent event) {
        getWizard().getContainer().updateButtons();
    }

    @Override
    public void modifyText(ModifyEvent e) {
        getWizard().getContainer().updateButtons();

    }

    public SpecimenTypeDesignation getBaseTypeDesignation() {
        return baseDesignation;
    }

    public String getAccessionNumber() {
        return composite.getTxtAccNumber().getText();
    }

    public String getBarcode() {
        return composite.getTextBarcode().getText();
    }

    public String getCatalogNumber() {
        return composite.getTextCatalogNumber().getText();
    }

    public eu.etaxonomy.cdm.model.occurrence.Collection getCollection() {
        return selectedCollection;
    }

    public SpecimenTypeDesignationStatus getTypeStatus() {
        return (SpecimenTypeDesignationStatus) composite.getComboTypeStatus().getSelection();
    }

    /**
     * @return
     */
    public URI getStableIdentifier() {
        return composite.getStableIdentifier();
    }

}
