/**
* Copyright (C) 2020 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.ui.section.occurrence;

import java.util.Collection;
import java.util.Comparator;

import eu.etaxonomy.cdm.api.conversation.ConversationHolder;
import eu.etaxonomy.cdm.api.facade.DerivedUnitFacade;
import eu.etaxonomy.cdm.model.media.Media;
import eu.etaxonomy.taxeditor.ui.dialog.selection.MediaSelectionDialog;
import eu.etaxonomy.taxeditor.ui.element.CdmFormFactory;
import eu.etaxonomy.taxeditor.ui.element.ICdmFormElement;
import eu.etaxonomy.taxeditor.ui.section.AbstractEntityCollectionSection;
import eu.etaxonomy.taxeditor.ui.section.DefaultCdmBaseComparator;

/**
 * @author k.luther
 * @since Aug 4, 2020
 */
public class DerivedUnitMediaSection extends AbstractEntityCollectionSection<DerivedUnitFacade, Media> {

    public DerivedUnitMediaSection(CdmFormFactory cdmFormFactory, ConversationHolder conversation, ICdmFormElement parentElement, int style) {
        super(cdmFormFactory, conversation, parentElement, "Media", style);
    }

    @Override
    public Media createNewElement() {
        return Media.NewInstance();
    }

    @Override
    public Collection<Media> getCollection(DerivedUnitFacade entity) {
        return entity.getDerivedUnitMedia();
    }

    @Override
    public Comparator<Media> getComparator() {
        return new DefaultCdmBaseComparator<>();
    }

    @Override
    public String getEmptyString() {
        return "No media yet.";
    }

    @Override
    protected String getTooltipString() {
        return "Create new media";
    }

    @Override
    public void addElement(Media element) {
        getEntity().addDerivedUnitMedia(element);
    }

    @Override
    public void removeElement(Media element) {
        getEntity().removeDerivedUnitMedia(element);
    }

    @Override
    public Media addExisting() {
        return MediaSelectionDialog.select(getShell(), null);
    }

    @Override
    public boolean allowAddExisting() {
        return true;
    }

}
