/**
 * Copyright (C) 2013 EDIT
 * European Distributed Institute of Taxonomy
 * http://www.e-taxonomy.eu
 *
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * See LICENSE.TXT at the top of this package for the full license terms.
 */
package eu.etaxonomy.taxeditor.ui.section.occurrence;

import java.util.Collection;
import java.util.HashSet;
import java.util.Set;

import eu.etaxonomy.cdm.api.facade.DerivedUnitFacade;
import eu.etaxonomy.cdm.model.agent.AgentBase;
import eu.etaxonomy.cdm.model.agent.Person;
import eu.etaxonomy.cdm.model.agent.Team;
import eu.etaxonomy.cdm.model.common.CdmBase;
import eu.etaxonomy.cdm.model.common.LanguageString;
import eu.etaxonomy.cdm.model.location.NamedArea;
import eu.etaxonomy.cdm.model.occurrence.SpecimenOrObservationType;
import eu.etaxonomy.taxeditor.preference.PreferencesUtil;
import eu.etaxonomy.taxeditor.store.CdmStore;
import eu.etaxonomy.taxeditor.store.StoreUtil;
import eu.etaxonomy.taxeditor.ui.combo.EnumComboElement;
import eu.etaxonomy.taxeditor.ui.element.CdmFormFactory;
import eu.etaxonomy.taxeditor.ui.element.CheckboxElement;
import eu.etaxonomy.taxeditor.ui.element.GatheringEventUnitElement;
import eu.etaxonomy.taxeditor.ui.element.ICdmFormElement;
import eu.etaxonomy.taxeditor.ui.element.LanguageStringWithLabelElement;
import eu.etaxonomy.taxeditor.ui.element.LayoutConstants;
import eu.etaxonomy.taxeditor.ui.element.MinMaxTextSection;
import eu.etaxonomy.taxeditor.ui.element.NumberWithLabelElement;
import eu.etaxonomy.taxeditor.ui.element.PointElement;
import eu.etaxonomy.taxeditor.ui.element.TextWithLabelElement;
import eu.etaxonomy.taxeditor.ui.element.TimePeriodElement;
import eu.etaxonomy.taxeditor.ui.element.ToggleableTextElement;
import eu.etaxonomy.taxeditor.ui.section.AbstractCdmDetailElement;
import eu.etaxonomy.taxeditor.ui.selection.CollectorSelectionElement;
import eu.etaxonomy.taxeditor.ui.selection.EntitySelectionElement;

/**
 * @author pplitzner
 * @date 05.11.2013
 */
public class FieldUnitFacadeGeneralDetailElement extends AbstractCdmDetailElement<DerivedUnitFacade> {

    private ToggleableTextElement toggleableText_titleCache;
    private EnumComboElement<SpecimenOrObservationType> combo_specorobstype;
    private EntitySelectionElement<NamedArea> selection_country;
    private CollectingAreasDetailSection section_collectingAreas;
    private LanguageStringWithLabelElement languageText_locality;
    private PointElement element_point;
    private NumberWithLabelElement number_elevation;
    private TimePeriodElement element_date;
    private CollectorSelectionElement selection_collector;
    private CollectorSelectionElement selection_primary_collector;
    private TextWithLabelElement text_collectingNumber;
    private GatheringEventUnitElement element_elevation;
    private CheckboxElement checkIsPublish;

    public FieldUnitFacadeGeneralDetailElement(CdmFormFactory formFactory, ICdmFormElement formElement) {
        super(formFactory, formElement);
    }

    @Override
    protected void createControls(ICdmFormElement formElement, DerivedUnitFacade entity, int style) {
        toggleableText_titleCache = formFactory.createToggleableTextField(formElement, "Title Cache",
                entity.getTitleCache(), entity.isProtectedTitleCache(), style);

        if (!entity.getType().equals(SpecimenOrObservationType.FieldUnit)){
           combo_specorobstype = formFactory.createEnumComboElement(SpecimenOrObservationType.class, formElement, style);
           combo_specorobstype.setSelection(entity.getType());
        }

        selection_country = formFactory.createSelectionElement(NamedArea.class, //getConversationHolder(),
                formElement, "Country", entity.getCountry(), EntitySelectionElement.DELETABLE, style);
        if(PreferencesUtil.isCollectingAreaInGeneralSection()){
            section_collectingAreas = formFactory.createCollectingAreasDetailSection(getConversationHolder(), formElement, StoreUtil.getSectionStyle(CollectingAreasDetailSection.class, entity.getClass().getCanonicalName()));
            section_collectingAreas.setLayoutData(LayoutConstants.FILL_HORIZONTALLY(2, 1));
            section_collectingAreas.setEntity(entity);
        }

        languageText_locality = formFactory.createLanguageStringWithLabelElement(formElement, "Locality",
                entity.getLocality(), style);
        element_point = formFactory.createPointElement(formElement, entity.getExactLocation(), style);
        element_elevation = formFactory.createGatheringEventUnitElement(
				formElement, "Elevation : ", entity, MinMaxTextSection.UnitType.ELEVATION,  style);

        element_date = formFactory.createTimePeriodElement(formElement, "Date", entity.getGatheringPeriod(), style);
        selection_collector = formFactory.createCollectorSelectionElement(formElement, "Collector", entity.getCollector(), EntitySelectionElement.ALL, style);
        selection_primary_collector = formFactory
	            .createCollectorSelectionElement(
	                    formElement, "Primary collector",
	                    entity.getCollector(), entity.getPrimaryCollector(), EntitySelectionElement.ALL,
	                    style);
        if (!(entity.getCollector() instanceof Team)){
	    	selection_primary_collector.setEnabled(false);
	    }
        selection_primary_collector.setIndent(8);
        text_collectingNumber = formFactory.createTextWithLabelElement(formElement, "Collecting number",
                entity.getFieldNumber(), style);
        checkIsPublish = formFactory.createCheckbox(formElement, "Publish", entity.innerFieldUnit().isPublish(), style);
    }

    @Override
    public void handleEvent(Object eventSource) {
        if (eventSource == toggleableText_titleCache) {
            getEntity().setTitleCache(toggleableText_titleCache.getText(), toggleableText_titleCache.getState());

        } else if (eventSource == selection_country) {
            getEntity().setCountry(selection_country.getSelection());
        } else if (eventSource == languageText_locality) {
            LanguageString locality = languageText_locality.updateLanguageString(getEntity().getLocality());
            getEntity().setLocality(locality);
        } else if (eventSource == element_point) {
            getEntity().setExactLocation(element_point.getPoint());
        } else if (eventSource == number_elevation) {
            getEntity().setAbsoluteElevation(number_elevation.getInteger());
        } else if (eventSource == element_date) {
            getEntity().setGatheringPeriod(element_date.getTimePeriod());
        } else if (eventSource == selection_collector) {
            getEntity().setCollector(selection_collector.getSelection());
            
            if (getEntity().getCollector() instanceof Team){
            	if (selection_primary_collector.getCollectorTeam() != null && selection_primary_collector.getEntity() != null ){
            		boolean collInNewTeam = false;
            		for (Person person:((Team)getEntity().getCollector()).getTeamMembers()){
            			collInNewTeam = person.getId() == selection_primary_collector.getEntity().getId();
            			if (collInNewTeam){
            				break;
            			}
            		}
            		if (!collInNewTeam){
            			selection_primary_collector.setEntity(null);
            		}
            	}
            	selection_primary_collector.setCollectorTeam((Team)getEntity().getCollector());
            }else{
            	selection_primary_collector.setEntity(null);
            }
            selection_primary_collector.setEnabled((getEntity().getCollector() instanceof Team));
            
        } else if (eventSource == selection_primary_collector) {
            getEntity().setPrimaryCollector((Person)selection_primary_collector.getSelection());
        } else if (eventSource == text_collectingNumber) {
            getEntity().setFieldNumber(text_collectingNumber.getText());
        } else if (eventSource == combo_specorobstype) {
            getEntity().setType(combo_specorobstype.getSelection());
        } else if (eventSource == checkIsPublish) {
            getEntity().innerFieldUnit().setPublish(checkIsPublish.getSelection());
        }

        toggleableText_titleCache.setText(getEntity().getTitleCache());

    }
    protected void updateControlStates(){
        if (getEntity() != null ){
            enabled = enabled && CdmStore.currentAuthentiationHasPermission(StoreUtil.getCdmEntity(getEntity()), requiredCrud);
        }else{
            enabled = true;
        }
        Set<Object> except = new HashSet<>(); 
        if (!selection_primary_collector.isEnabled() && enabled){
        	except.add(selection_primary_collector);
        }
        setEnabled(enabled, except);
        
    }

}