/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.ui.section.occurrence;

import java.util.Collection;
import java.util.Comparator;

import eu.etaxonomy.cdm.api.conversation.ConversationHolder;
import eu.etaxonomy.cdm.model.location.NamedArea;
import eu.etaxonomy.cdm.model.taxon.Classification;
import eu.etaxonomy.taxeditor.ui.dialog.selection.NamedAreaSelectionDialog;
import eu.etaxonomy.taxeditor.ui.element.CdmFormFactory;
import eu.etaxonomy.taxeditor.ui.element.ICdmFormElement;
import eu.etaxonomy.taxeditor.ui.section.AbstractEntityCollectionSection;
import eu.etaxonomy.taxeditor.ui.section.DefaultCdmBaseComparator;

/**
 * <p>
 * CollectingAreasDetailSection class.
 * </p>
 *
 * @author n.hoffmann
 * @created Oct 14, 2010
 * @version 1.0
 */
public class GeoScopeDetailSection extends AbstractEntityCollectionSection<Classification, NamedArea> {

	/**
	 * <p>Constructor for CollectingAreasDetailSection.</p>
	 *
	 * @param formFactory a {@link eu.etaxonomy.taxeditor.ui.element.CdmFormFactory} object.
	 * @param conversation a {@link eu.etaxonomy.cdm.api.conversation.ConversationHolder} object.
	 * @param parentElement a {@link eu.etaxonomy.taxeditor.ui.element.ICdmFormElement} object.
	 * @param style a int.
	 */
	public GeoScopeDetailSection(CdmFormFactory formFactory,
			ConversationHolder conversation, ICdmFormElement parentElement, int style) {
		super(formFactory, conversation, parentElement, "Geo Scopes", style);
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.section.AbstractEntityCollectionSection#getCollection(java.lang.Object)
	 */
	/** {@inheritDoc} */
	@Override
	public Collection<NamedArea> getCollection(Classification entity) {
		return entity.getGeoScopes();
	}

	@Override
	public Comparator<NamedArea> getComparator() {
        return new DefaultCdmBaseComparator<>();
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.section.AbstractEntityCollectionSection#createNewElement()
	 */
	/** {@inheritDoc} */
	@Override
	public NamedArea createNewElement() {
		NamedArea selection = NamedAreaSelectionDialog.select(getLayoutComposite().getShell(), //getConversationHolder(),
		        null, Classification.class.getCanonicalName());

		return selection;
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.section.AbstractEntityCollectionSection#addElement(eu.etaxonomy.cdm.model.common.IVersionableEntity)
	 */
	/** {@inheritDoc} */
	@Override
	public void addElement(NamedArea element) {
		getEntity().addGeoScope(element);
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.section.AbstractEntityCollectionSection#removeElement(eu.etaxonomy.cdm.model.common.IVersionableEntity)
	 */
	/** {@inheritDoc} */
	@Override
	public void removeElement(NamedArea element) {
		getEntity().removeGeoScope(element);
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.section.AbstractEntityCollectionSection#getEmptyString()
	 */
	/** {@inheritDoc} */
	@Override
	public String getEmptyString() {
		return "No geo scopes yet.";
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.section.AbstractEntityCollectionSection#getTooltipString()
	 */
	/** {@inheritDoc} */
	@Override
	protected String getTooltipString() {
		return "Add a geo scope";
	}

    /**
     * {@inheritDoc}
     */
    @Override
    public NamedArea addExisting() {
        return null;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public boolean allowAddExisting() {
        // TODO Auto-generated method stub
        return false;
    }

}
