/**
 * Copyright (C) 2007 EDIT
 * European Distributed Institute of Taxonomy
 * http://www.e-taxonomy.eu
 *
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * See LICENSE.TXT at the top of this package for the full license terms.
 */

package eu.etaxonomy.taxeditor.ui.section.occurrence;

import org.eclipse.swt.events.SelectionListener;

import eu.etaxonomy.cdm.model.occurrence.DerivedUnit;
import eu.etaxonomy.taxeditor.ui.element.AbstractFormSection;
import eu.etaxonomy.taxeditor.ui.element.CdmFormFactory;
import eu.etaxonomy.taxeditor.ui.element.ICdmFormElement;
import eu.etaxonomy.taxeditor.ui.section.AbstractEntityCollectionElement;
import eu.etaxonomy.taxeditor.ui.selection.EntitySelectionElement;

/**
 * <p>
 * SpecimenCollectionDetailElement class.
 * </p>
 *
 * @author n.hoffmann
 * @created Oct 14, 2010
 * @version 1.0
 */
public class SpecimenCollectionDetailElement extends
		AbstractEntityCollectionElement<DerivedUnit> {

	private EntitySelectionElement<DerivedUnit> selection_derivedUnit;

	/**
	 * <p>
	 * Constructor for SpecimenCollectionDetailElement.
	 * </p>
	 *
	 * @param formFactory
	 *            a {@link eu.etaxonomy.taxeditor.ui.element.CdmFormFactory}
	 *            object.
	 * @param section
	 *            a {@link eu.etaxonomy.taxeditor.ui.element.AbstractFormSection}
	 *            object.
	 * @param entity
	 *            a {@link eu.etaxonomy.cdm.model.occurrence.Specimen} object.
	 * @param removeListener
	 *            a {@link org.eclipse.swt.events.SelectionListener} object.
	 * @param style
	 *            a int.
	 */
	public SpecimenCollectionDetailElement(CdmFormFactory formFactory,
			AbstractFormSection section, DerivedUnit entity,
			SelectionListener removeListener, int style) {
		super(formFactory, section, entity, removeListener, null, style);
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see
	 * eu.etaxonomy.taxeditor.section.AbstractEntityCollectionElement#setEntity
	 * (java.lang.Object)
	 */
	/** {@inheritDoc} */
	@Override
	public void setEntity(DerivedUnit entity) {
		this.entity = entity;
		if (selection_derivedUnit != null){
			selection_derivedUnit.setEntity(entity);
		}
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see
	 * eu.etaxonomy.taxeditor.section.AbstractEntityCollectionElement#createControls
	 * (eu.etaxonomy.taxeditor.forms.ICdmFormElement, int)
	 */
	/** {@inheritDoc} */
	@Override
	public void createControls(ICdmFormElement element, int style) {
		selection_derivedUnit = formFactory
				.createSelectionElement(DerivedUnit.class,//getConversationHolder(),
				        element, "Derived Unit", null,
						EntitySelectionElement.EDITABLE | EntitySelectionElement.SELECTABLE, style);
		if (entity != null){
			setEntity(entity);
		}
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see
	 * eu.etaxonomy.taxeditor.section.AbstractEntityCollectionElement#handleEvent
	 * (java.lang.Object)
	 */
	/** {@inheritDoc} */
	@Override
	public void handleEvent(Object eventSource) {
		if (eventSource == selection_derivedUnit) {
			this.setEntity(selection_derivedUnit.getSelection());
		}
	}

}
