/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.ui.section.occurrence;

import java.util.Collection;
import java.util.Comparator;

import eu.etaxonomy.cdm.api.conversation.ConversationHolder;
import eu.etaxonomy.cdm.api.facade.DerivedUnitFacade;
import eu.etaxonomy.cdm.model.common.IdentifiableSource;
import eu.etaxonomy.cdm.model.occurrence.DerivedUnit;
import eu.etaxonomy.cdm.model.occurrence.OccurrenceStatus;
import eu.etaxonomy.cdm.model.reference.OriginalSourceType;
import eu.etaxonomy.taxeditor.store.CdmStore;
import eu.etaxonomy.taxeditor.ui.element.CdmFormFactory;
import eu.etaxonomy.taxeditor.ui.element.ICdmFormElement;
import eu.etaxonomy.taxeditor.ui.section.AbstractEntityCollectionSection;
import eu.etaxonomy.taxeditor.ui.section.DefaultCdmBaseComparator;

/**
 * <p>SourceCollectionDetailSection class.</p>
 *
 * @author n.hoffmann
 * @created Oct 15, 2010
 * @version 1.0
 */
public class StatusCollectionDetailSection extends
		AbstractEntityCollectionSection<DerivedUnitFacade, OccurrenceStatus> {

	/**
	 * <p>Constructor for SourceCollectionDetailSection.</p>
	 *
	 * @param formFactory a {@link eu.etaxonomy.taxeditor.ui.element.CdmFormFactory} object.
	 * @param conversation a {@link eu.etaxonomy.cdm.api.conversation.ConversationHolder} object.
	 * @param parentElement a {@link eu.etaxonomy.taxeditor.ui.element.ICdmFormElement} object.
	 * @param style a int.
	 */
	public StatusCollectionDetailSection(CdmFormFactory formFactory,
			ConversationHolder conversation, ICdmFormElement parentElement, int style) {
		super(formFactory, conversation, parentElement, "Status", style);
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.section.AbstractEntityCollectionSection#getCollection(java.lang.Object)
	 */
	/** {@inheritDoc} */
	@Override
	public Collection<OccurrenceStatus> getCollection(DerivedUnitFacade entity) {
		if (entity.baseUnit() instanceof DerivedUnit){
			return ((DerivedUnit)entity.baseUnit()).getStatus();
		}
		return null;
	}

	@Override
	public Comparator<OccurrenceStatus> getComparator() {
        return new DefaultCdmBaseComparator<>();
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.section.AbstractEntityCollectionSection#createNewElement()
	 */
	/** {@inheritDoc} */
	@Override
	public OccurrenceStatus createNewElement() {
		return OccurrenceStatus.NewInstance(null);
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.section.AbstractEntityCollectionSection#addElement(eu.etaxonomy.cdm.model.common.IVersionableEntity)
	 */
	/** {@inheritDoc} */
	@Override
	public void addElement(OccurrenceStatus element) {
		 ((DerivedUnit)getEntity().baseUnit()).addStatus(element);
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.section.AbstractEntityCollectionSection#removeElement(eu.etaxonomy.cdm.model.common.IVersionableEntity)
	 */
	/** {@inheritDoc} */
	@Override
	public void removeElement(OccurrenceStatus element) {
		((DerivedUnit)getEntity().baseUnit()).removeStatus(element);
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.section.AbstractEntityCollectionSection#getEmptyString()
	 */
	/** {@inheritDoc} */
	@Override
	public String getEmptyString() {
		return "No status yet.";
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.section.AbstractEntityCollectionSection#getTooltipString()
	 */
	/** {@inheritDoc} */
	@Override
	protected String getTooltipString() {
		return "Add status";
	}

    /**
     * {@inheritDoc}
     */
    @Override
    public OccurrenceStatus addExisting() {
        return null;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public boolean allowAddExisting() {
        return false;
    }
    
    @Override
    protected void setSectionTitle() {
	    DerivedUnitFacade entity = getEntity();
		Collection<OccurrenceStatus> collection = getCollection(entity);
		if(collection != null && collection.size() > 0){
			String concatenatedLabels = "";
			for (OccurrenceStatus status: collection){
				concatenatedLabels += ", "+ (status != null && status.getType() != null? status.getType().getPreferredRepresentation(CdmStore.getDefaultLanguage()) : " - ");
			}
			concatenatedLabels = concatenatedLabels.replaceFirst(", ", ": ");
			this.setText(getTitleString() + concatenatedLabels );
		}else{
			this.setText(getTitleString());
		}
	}

}
