/**
 * Copyright (C) 2007 EDIT
 * European Distributed Institute of Taxonomy
 * http://www.e-taxonomy.eu
 *
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * See LICENSE.TXT at the top of this package for the full license terms.
 */

package eu.etaxonomy.taxeditor.ui.section.occurrence.association;

import java.util.Set;

import org.eclipse.swt.events.SelectionListener;

import eu.etaxonomy.cdm.hibernate.HibernateProxyHelper;
import eu.etaxonomy.cdm.model.term.TermType;
import eu.etaxonomy.cdm.model.name.SpecimenTypeDesignation;
import eu.etaxonomy.cdm.model.name.SpecimenTypeDesignationStatus;
import eu.etaxonomy.cdm.model.name.TaxonName;
import eu.etaxonomy.taxeditor.ui.combo.term.TermComboElement;
import eu.etaxonomy.taxeditor.ui.element.AbstractFormSection;
import eu.etaxonomy.taxeditor.ui.element.CdmFormFactory;
import eu.etaxonomy.taxeditor.ui.element.ICdmFormElement;
import eu.etaxonomy.taxeditor.ui.section.name.AbstractTypeDesignationElement;
import eu.etaxonomy.taxeditor.ui.selection.EntitySelectionElement;

/**
 * @author n.hoffmann
 * @created May 17, 2010
 */
public class DerivedUnitTypeDesignationElement
		extends AbstractTypeDesignationElement<SpecimenTypeDesignation> {

    private TermComboElement<SpecimenTypeDesignationStatus> combo_typeStatus;

//    private List<EntitySelectionElement<TaxonNameBase>> selectionTaxonNames;
    private ICdmFormElement parentFormElement;
    private int style;

    private EntitySelectionElement<TaxonName> selectionTaxonName;

    public DerivedUnitTypeDesignationElement(CdmFormFactory formFactory,
            AbstractFormSection<?> section, SpecimenTypeDesignation entity,
            SelectionListener removeListener, int style) {
        super(formFactory, section, entity, removeListener, style);
    }

    @Override
    public void setEntity(SpecimenTypeDesignation entity) {
        super.setEntity(entity);
        if (combo_typeStatus != null){
	        Set<TaxonName> typifiedNames = entity.getTypifiedNames();
	        if(typifiedNames.size()==1){
	            selectionTaxonName.setEntity(typifiedNames.iterator().next());
	        }
	        else if(typifiedNames.size()>1){
	            formFactory.createLabel(getLayoutComposite(), "!!!!!!!!");
	        }
	
	        SpecimenTypeDesignationStatus typeStatus = HibernateProxyHelper
	        		.deproxy(entity.getTypeStatus());
	        combo_typeStatus.setSelection(typeStatus);
	        checkbox_notDesignated.setSelection(entity.isNotDesignated());
        }

//        for (TaxonNameBase taxonNameBase : typifiedNames) {
//            EntitySelectionElement<TaxonNameBase> selectionElement = formFactory.createSelectionElement(
//                    TaxonNameBase.class, getConversationHolder(), parentFormElement, "Scientific Name", null,
//                    EntitySelectionElement.EDITABLE | EntitySelectionElement.SELECTABLE, style);
//            selectionElement.setEntity(taxonNameBase);
//        }


    }

    /** {@inheritDoc} */
    @Override
    public void createControls(ICdmFormElement element, int style) {
        this.parentFormElement = element;
        this.style = style;

        selectionTaxonName = formFactory.createSelectionElement(
                TaxonName.class, //getConversationHolder(),
                parentFormElement, "Scientific Name", null,
                EntitySelectionElement.EDITABLE | EntitySelectionElement.SELECTABLE, style);

        combo_typeStatus = formFactory.createDefinedTermComboElement(TermType.SpecimenTypeDesignationStatus,
                parentFormElement, "Designation Status", null, style);

        super.createControls(element, style);
        if (entity != null){
        	setEntity(entity);
        }
    }

    /** {@inheritDoc} */
    @Override
    public void handleEvent(Object eventSource) {
        if (eventSource == selectionTaxonName) {
            selectionTaxonName.getSelection().addTypeDesignation(getEntity(), false);
        } else if (eventSource == combo_typeStatus) {
            getEntity().setTypeStatus(combo_typeStatus.getSelection());
        } else if (eventSource == checkbox_notDesignated) {
            getEntity().setNotDesignated(checkbox_notDesignated.getSelection());
        }
//        else if (eventSource == selection_reference) {
//            getEntity().setCitation(selection_reference.getSelection());
//        } else if (eventSource == text_referenceDetail) {
//            getEntity().setCitationMicroReference(
//                    text_referenceDetail.getText());
//        }
    }

}
