/**
* Copyright (C) 2020 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.ui.section.reference;

import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.widgets.Display;

import eu.etaxonomy.cdm.model.metadata.PreferencePredicate;
import eu.etaxonomy.cdm.model.name.TaxonName;
import eu.etaxonomy.cdm.model.reference.NamedSourceBase;
import eu.etaxonomy.cdm.model.reference.OriginalSourceBase;
import eu.etaxonomy.taxeditor.preference.PreferencesUtil;
import eu.etaxonomy.taxeditor.ui.element.CdmFormFactory;
import eu.etaxonomy.taxeditor.ui.element.ICdmFormElement;
import eu.etaxonomy.taxeditor.ui.element.TextWithLabelElement;
import eu.etaxonomy.taxeditor.ui.section.AbstractCdmDetailElement;
import eu.etaxonomy.taxeditor.ui.selection.EntitySelectionElement;

/**
 * @author k.luther
 * @since Jul 2, 2020
 */
public class AdvancedSourceElement extends AbstractCdmDetailElement<OriginalSourceBase> {

    protected TextWithLabelElement text_idInSource;
    protected TextWithLabelElement text_idNamespace;
    protected TextWithLabelElement text_originalInfo;
    protected TextWithLabelElement text_cdmsource;
    protected EntitySelectionElement<TaxonName> select_nameUsedInSource;

//    protected ExternalLinksSection externalLinks;

    public AdvancedSourceElement(CdmFormFactory formFactory, ICdmFormElement formElement) {
        super(formFactory, formElement);
    }

    @Override
    protected void createControls(ICdmFormElement formElement, OriginalSourceBase entity, int style) {
        Display display = Display.getCurrent();
        Color background = display.getSystemColor(SWT.COLOR_WIDGET_BACKGROUND);

        if (entity instanceof NamedSourceBase){
            select_nameUsedInSource = formFactory.createSelectionElement(TaxonName.class, formElement, "Name in Source", entity != null? ((NamedSourceBase)entity).getNameUsedInSource(): null, EntitySelectionElement.DELETABLE, style);
            select_nameUsedInSource.setBackground(background);
        }
        text_originalInfo = formFactory.createTextWithLabelElement(
                formElement, "Original Information", entity != null?entity.getOriginalInfo():null, SWT.NULL);

        text_originalInfo.setBackground(background);

        if (PreferencesUtil.getBooleanValue(PreferencePredicate.ShowNamespaceInSource.getKey())){
            text_idNamespace = formFactory.createTextWithLabelElement(formElement, "ID Namespace", entity != null?entity.getIdNamespace():null, style);
            text_idNamespace.setBackground(background);
        }
        if (PreferencesUtil.getBooleanValue(PreferencePredicate.ShowIdInSource.getKey())){
            text_idInSource = formFactory.createTextWithLabelElement(formElement, "ID in Source", entity != null?entity.getIdInSource():null, style);
            text_idInSource.setBackground(background);
        }
    }

    @Override
    public void setEntity(OriginalSourceBase entity) {
        super.setEntity(entity);
    }

    @Override
    public void handleEvent(Object eventSource) {
        if (eventSource.equals(text_idInSource)){
             getEntity().setIdInSource(text_idInSource.getText());
         }else if (eventSource.equals(text_idNamespace)){
             getEntity().setIdNamespace(text_idNamespace.getText());
         }else if (eventSource.equals(text_originalInfo)){
             getEntity().setOriginalInfo(text_originalInfo.getText());
         }else if (eventSource.equals(select_nameUsedInSource)){
             ((NamedSourceBase)getEntity()).setNameUsedInSource(select_nameUsedInSource.getEntity());
         }
    }
}