/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.ui.section.reference;

import org.eclipse.jface.viewers.ISelectionProvider;

import eu.etaxonomy.cdm.api.conversation.ConversationHolder;
import eu.etaxonomy.cdm.hibernate.HibernateProxyHelper;
import eu.etaxonomy.cdm.model.name.NomenclaturalSource;
import eu.etaxonomy.cdm.model.name.TaxonName;
import eu.etaxonomy.cdm.model.taxon.TaxonBase;
import eu.etaxonomy.taxeditor.ui.element.CdmFormFactory;
import eu.etaxonomy.taxeditor.ui.element.ICdmFormElement;
import eu.etaxonomy.taxeditor.ui.section.AbstractCdmDetailElement;
import eu.etaxonomy.taxeditor.ui.section.AbstractCdmDetailSection;
import eu.etaxonomy.taxeditor.ui.section.ITaxonBaseDetailSection;

/**
 * <p>NomenclaturalReferenceDetailSection class.</p>
 *
 * @author n.hoffmann
 * @created Mar 12, 2010
 */
public class NomenclaturalSourceDetailSection extends
		AbstractCdmDetailSection<TaxonName>  implements ITaxonBaseDetailSection{

	private TaxonBase<?> taxonBase;

	/**
	 * <p>Constructor for NomenclaturalReferenceDetailSection.</p>
	 *
	 * @param cdmFormFactory a {@link eu.etaxonomy.taxeditor.ui.element.CdmFormFactory} object.
	 * @param conversation a {@link eu.etaxonomy.cdm.api.conversation.ConversationHolder} object.
	 * @param parentElement a {@link eu.etaxonomy.taxeditor.ui.element.ICdmFormElement} object.
	 * @param selectionProvider a {@link org.eclipse.jface.viewers.ISelectionProvider} object.
	 * @param style a int.
	 */
	public NomenclaturalSourceDetailSection(CdmFormFactory cdmFormFactory, ConversationHolder conversation,
			ICdmFormElement parentElement, ISelectionProvider selectionProvider, int style) {
		super(cdmFormFactory, conversation, parentElement, selectionProvider, style);
	}

	@Override
	public String getHeading() {
		return "Nomenclatural Source";
	}

	@Override
    public void setTaxonBase(TaxonBase entity) {
		this.taxonBase = entity;
		TaxonName name = HibernateProxyHelper.deproxy(entity.getName());
		setEntity(name);
	}

	@Override
	protected void setSectionTitle() {
		String title = "";
		if(getEntity() != null && getEntity().getNomenclaturalSource() != null){
			NomenclaturalSource reference = getEntity().getNomenclaturalSource();
			if (reference != null && reference.getCitation() != null){
			    String nomenclaturalCitation = reference.getCitation().getNomenclaturalCitation(reference.getCitationMicroReference());
			    title = ": " + nomenclaturalCitation;
			}

		}
		this.setText(getHeading() + title);
	}

	@Override
	public TaxonBase getTaxonBase() {
		return taxonBase;
	}

	@Override
	protected AbstractCdmDetailElement<TaxonName> createCdmDetailElement(
			AbstractCdmDetailSection<TaxonName> parentElement, int style) {
	    return formFactory.createNomenclaturalSourceDetailElement(parentElement, style);
	}
}
