/**
* Copyright (C) 2020 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.ui.section.reference;

import org.eclipse.jface.viewers.ISelectionProvider;
import org.eclipse.ui.forms.widgets.TableWrapLayout;

import eu.etaxonomy.cdm.api.conversation.ConversationHolder;
import eu.etaxonomy.cdm.model.reference.OriginalSourceBase;
import eu.etaxonomy.taxeditor.l10n.Messages;
import eu.etaxonomy.taxeditor.ui.element.CdmFormFactory;
import eu.etaxonomy.taxeditor.ui.element.ICdmFormElement;
import eu.etaxonomy.taxeditor.ui.section.AbstractCdmDetailElement;
import eu.etaxonomy.taxeditor.ui.section.AbstractCdmDetailSection;

/**
 * @author k.luther
 * @since Jul 1, 2020
 */
public class OriginalSourceAdvancedSection extends AbstractCdmDetailSection<OriginalSourceBase> {

    public OriginalSourceAdvancedSection(CdmFormFactory formFactory, ConversationHolder conversation,
            ICdmFormElement parentElement, ISelectionProvider selectionProvider, int style) {
        super(formFactory, conversation, parentElement, selectionProvider, style);
    }

    @Override
    protected AbstractCdmDetailElement<OriginalSourceBase> createCdmDetailElement(
            AbstractCdmDetailSection<OriginalSourceBase> parentElement, int style) {

        if (parentElement.getParentElement() instanceof NomenclaturalSourceElement){
            return getFormFactory().createAdvancedNomenclaturalSourceDetailElement(parentElement, style);
        }
        return getFormFactory().createAdvancedSourceDetailElement(parentElement, style);
    }

    @Override
    public String getHeading() {

        return Messages.OriginalSourceAdvancedSection_advanced;
    }
    @Override
    protected void createControlsByType(AbstractCdmDetailSection<OriginalSourceBase> formElement, Class<OriginalSourceBase> entityClass, int style) {
        super.createControlsByType(formElement, entityClass, style);
        int i = this.getTextClientHeightDifference();

        TableWrapLayout layout = new TableWrapLayout();
        layout.topMargin = 0;
        layout.bottomMargin = 0;
        layout.numColumns = DEFAULT_NUM_COLUMNS;
        getLayoutComposite().setLayout(layout);
//        setLayout(layout);
    }



//    protected ExternalLinksSection getExternalLinksElement(){
//        return ((AdvancedSourceElement)this.getDetailElement()).getExternalLinks();
//    }
}