/**
* Copyright (C) 2020 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.ui.section.reference;

import org.apache.commons.lang3.StringUtils;
import org.eclipse.jface.util.IPropertyChangeListener;
import org.eclipse.ui.forms.widgets.TableWrapData;

import eu.etaxonomy.cdm.hibernate.HibernateProxyHelper;
import eu.etaxonomy.cdm.model.common.CdmBase;
import eu.etaxonomy.cdm.model.common.TimePeriod;
import eu.etaxonomy.cdm.model.description.DescriptionElementSource;
import eu.etaxonomy.cdm.model.reference.INomenclaturalReference;
import eu.etaxonomy.cdm.model.reference.NamedSourceBase;
import eu.etaxonomy.cdm.model.reference.OriginalSourceBase;
import eu.etaxonomy.cdm.model.reference.Reference;
import eu.etaxonomy.taxeditor.model.AbstractUtility;
import eu.etaxonomy.taxeditor.preference.Resources;
import eu.etaxonomy.taxeditor.store.StoreUtil;
import eu.etaxonomy.taxeditor.ui.element.CdmFormFactory;
import eu.etaxonomy.taxeditor.ui.element.ICdmFormElement;
import eu.etaxonomy.taxeditor.ui.element.LayoutConstants;
import eu.etaxonomy.taxeditor.ui.element.TextWithLabelElement;
import eu.etaxonomy.taxeditor.ui.element.TimePeriodElement;
import eu.etaxonomy.taxeditor.ui.section.AbstractCdmDetailElement;
import eu.etaxonomy.taxeditor.ui.section.common.ExternalLinksSection;
import eu.etaxonomy.taxeditor.ui.selection.EntitySelectionElement;

/**
 * @author k.luther
 * @since Jul 1, 2020
 */
public class OriginalSourceElement<T extends OriginalSourceBase>
        extends AbstractCdmDetailElement<T>
        implements IPropertyChangeListener{

    protected EntitySelectionElement<Reference> selection_Ref;
    protected TextWithLabelElement microReference;
    protected TimePeriodElement accessed;
    protected OriginalSourceAdvancedSection advancedSection;
    protected String label = "Source";
    protected CdmBase cdmEntity;
    protected ExternalLinksSection externalLinks;

    public OriginalSourceElement(CdmFormFactory formFactory, ICdmFormElement formElement, CdmBase cdmEntity, String label) {
        super(formFactory, formElement);
        this.cdmEntity = cdmEntity;
        this.label = label;
    }

    @Override
    protected void createControls(ICdmFormElement formElement, T entity,  int style) {

        selection_Ref = formFactory.createSelectionElement(Reference.class,
                      formElement, label, null,
                      EntitySelectionElement.ALL, style, 100);

        if (entity != null){
            selection_Ref.setEntity(entity.getCitation());
        }
        selection_Ref.setBackground(this.getPersistentBackground());
        microReference = formFactory.createTextWithLabelElement(formElement, "Details", entity != null? entity.getCitationMicroReference(): "", style);
        microReference.setIndent(10);
        for (ICdmFormElement element: selection_Ref.getElements()){
            element.setBackground(getPersistentBackground());
        }

        //TODO preliminary #10057
        if (entity != null && entity.getCitation() != null && entity.getCitation().isDynamic()){
            TimePeriod acc =entity.getAccessed(); // (entity != null? entity.getAccessed(): null);
            accessed = formFactory.createTimePeriodElement(formElement, "Accessed", acc, style);
        }

        externalLinks = formFactory.createExternalLinksSection(getConversationHolder(), formElement, "Link(s)", StoreUtil.getSectionStyle(ExternalLinksSection.class, DescriptionElementSource.class.getCanonicalName()));
        externalLinks.setLayoutData(LayoutConstants.FILL_HORIZONTALLY(2, 3));
        externalLinks.setEntity(HibernateProxyHelper.deproxy(entity));
        externalLinks.setFont(AbstractUtility.getFont(Resources.COLOR_FONT_DEFAULT));
        externalLinks.setEmptySectionString("No links yet.");

        addControl(externalLinks);
        addElement(externalLinks);
        advancedSection = formFactory.createOriginalSourceAdvancedSection(getConversationHolder(), formElement, null, StoreUtil.getSectionStyle(OriginalSourceAdvancedSection.class, INomenclaturalReference.class.getCanonicalName()));
        TableWrapData layoutData = LayoutConstants.FILL_HORIZONTALLY(2, 1);
        advancedSection.setLayoutData(layoutData);
        advancedSection.setFont(AbstractUtility.getFont(Resources.COLOR_FONT_DEFAULT));
        addControl(advancedSection);
        addElement(advancedSection);
        advancedSection.setBackground(this.getPersistentBackground());
        advancedSection.setEntity(entity);
        if (entity == null){
            advancedSection.setExpanded(false);
        }else if (StringUtils.isNotBlank(entity.getIdInSource()) || StringUtils.isNotBlank(entity.getIdNamespace()) || StringUtils.isNotBlank(entity.getOriginalInfo()) || (entity.getLinks() != null && !entity.getLinks().isEmpty())  || (entity instanceof DescriptionElementSource && ((NamedSourceBase)entity).getNameUsedInSource() != null)){
            advancedSection.setExpanded(true);
        }else{
            advancedSection.setExpanded(false);
        }
    }

    @Override
    public void handleEvent(Object eventSource) {
        if (eventSource.equals(selection_Ref)){
            getEntity().setCitation(selection_Ref.getEntity());
        }else if (eventSource.equals(microReference)){
            getEntity().setCitationMicroReference(microReference.getText());
        }else if(eventSource == accessed){
            getEntity().setAccessed(accessed.getTimePeriod());
        }
    }

    public void setCdmEntity(CdmBase cdmEntity) {
        this.cdmEntity = cdmEntity;
    }

    public EntitySelectionElement<Reference> getReferenceSelectionElement(){
        return selection_Ref;
    }

    public void setIndent(int indent){
        selection_Ref.setIndent(indent);
        microReference.setIndent(indent+10);
        externalLinks.setIndent(indent+10);
        advancedSection.setIndent(indent+10);
    }
}