/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.ui.section.reference;

import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Composite;

import eu.etaxonomy.cdm.api.conversation.ConversationHolder;
import eu.etaxonomy.cdm.model.reference.Reference;
import eu.etaxonomy.taxeditor.ui.element.AbstractCdmEntityWizardPage;
import eu.etaxonomy.taxeditor.ui.element.CdmFormFactory;
import eu.etaxonomy.taxeditor.ui.element.CdmPropertyChangeEvent;
import eu.etaxonomy.taxeditor.ui.element.ICdmFormElement;
import eu.etaxonomy.taxeditor.ui.element.IExceptionHandler;
import eu.etaxonomy.taxeditor.ui.section.AbstractCdmDetailElement;

/**
 * <p>ReferenceBaseWizardPage class.</p>
 *
 * @author n.hoffmann
 * @created Jun 1, 2010
 */
public class ReferenceWizardPage extends AbstractCdmEntityWizardPage<Reference> implements IExceptionHandler{
	private boolean isNomenclaturalReference = false;

	/**
	 * <p>Constructor for ReferenceBaseWizardPage.</p>
	 *
	 * @param formFactory a {@link eu.etaxonomy.taxeditor.ui.element.CdmFormFactory} object.
	 * @param conversation a {@link eu.etaxonomy.cdm.api.conversation.ConversationHolder} object.
	 * @param entity a {@link eu.etaxonomy.cdm.model.reference.ReferenceBase} object.
	 */
	public ReferenceWizardPage(CdmFormFactory formFactory, ConversationHolder conversation, Reference entity, boolean isNomenclaturalReference) {
		super(formFactory, conversation, entity);
		setTitle("Reference");
		this.isNomenclaturalReference = isNomenclaturalReference;
	}

	@Override
    public void createControl(Composite parent) {
	    super.createControl(parent);
	}

	@Override
    public AbstractCdmDetailElement<Reference> createElement(ICdmFormElement rootElement){
		ReferenceDetailElement referenceElement;
		if (isNomenclaturalReference){
			referenceElement = formFactory.createNomenclaturalReferenceDetailElement(rootElement,  SWT.NULL);
		}else{
			referenceElement = formFactory.createReferenceDetailElement(rootElement, SWT.NULL);
		}
		referenceElement.addExceptionHandler(this);
		referenceElement.setEntity(entity);
		referenceElement.setWarnForReferencingObjectsVisible(true);
		return referenceElement;
	}

	@Override
	public void handleException(CdmPropertyChangeEvent event) {
		setMessage(event.getException().getMessage(), WARNING);
	}

	@Override
	public void clearException() {
		setMessage(null);
	}
}