/**
* Copyright (C) 2021 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.ui.section.reference;

import org.eclipse.swt.widgets.Label;

import eu.etaxonomy.cdm.hibernate.HibernateProxyHelper;
import eu.etaxonomy.cdm.model.common.CdmBase;
import eu.etaxonomy.cdm.model.common.TimePeriod;
import eu.etaxonomy.cdm.model.description.DescriptionElementSource;
import eu.etaxonomy.cdm.model.name.TaxonName;
import eu.etaxonomy.cdm.model.reference.NamedSourceBase;
import eu.etaxonomy.cdm.model.reference.Reference;
import eu.etaxonomy.cdm.model.taxon.SecundumSource;
import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.taxeditor.model.AbstractUtility;
import eu.etaxonomy.taxeditor.preference.Resources;
import eu.etaxonomy.taxeditor.store.StoreUtil;
import eu.etaxonomy.taxeditor.ui.element.CdmFormFactory;
import eu.etaxonomy.taxeditor.ui.element.ICdmFormElement;
import eu.etaxonomy.taxeditor.ui.element.LayoutConstants;
import eu.etaxonomy.taxeditor.ui.section.common.ExternalLinksSection;
import eu.etaxonomy.taxeditor.ui.selection.EntitySelectionElement;

/**
 * @author k.luther
 * @since Mar 24, 2021
 */
public class SecundumSourceElement extends OriginalSourceElement<SecundumSource> {

	protected EntitySelectionElement<TaxonName> select_nameUsedInSource;

	public SecundumSourceElement(CdmFormFactory formFactory, ICdmFormElement formElement, CdmBase cdmEntity,
            String label) {
        super(formFactory, formElement, cdmEntity, label);
    }

    @Override
    protected void createControls(ICdmFormElement formElement, SecundumSource entity,  int style) {

        setEntityWithoutUpdate(HibernateProxyHelper.deproxy(entity));

        this.selection_Ref = formFactory.createSelectionElement(Reference.class, formElement, "Secundum", getEntity()!= null? getEntity().getCitation(): null, EntitySelectionElement.ALL, style);

        selection_Ref.setBackground(this.getPersistentBackground());
        for (ICdmFormElement element: selection_Ref.getElements()){
            element.setBackground(getPersistentBackground());
        }

        microReference = formFactory.createTextWithLabelElement(formElement, "Detail", getEntity() != null? getEntity().getCitationMicroReference(): "", style);

        //TODO preliminary #10057
        if (entity != null && entity.getCitation() != null && entity.getCitation().isDynamic()){
            TimePeriod acc =entity.getAccessed(); // (entity != null? entity.getAccessed(): null);
            accessed = formFactory.createTimePeriodElement(formElement, "Accessed", acc, style);
        }

        Label sep = formFactory.createLabel(formElement.getLayoutComposite(), "");
        sep.setLayoutData(LayoutConstants.FILL_HORIZONTALLY(2, 1));
        if (this.cdmEntity instanceof Taxon){
        	select_nameUsedInSource = formFactory.createSelectionElement(TaxonName.class, formElement, "Name in Source", entity != null? ((NamedSourceBase)entity).getNameUsedInSource(): null, EntitySelectionElement.DELETABLE, style);
        }

        externalLinks = formFactory.createExternalLinksSection(getConversationHolder(), formElement, "Links", false, StoreUtil.getSectionStyle(ExternalLinksSection.class, DescriptionElementSource.class.getCanonicalName()));
        externalLinks.setEmptySectionString("No links yet.");
        externalLinks.setLayoutData(LayoutConstants.FILL_HORIZONTALLY(2, 3));
        externalLinks.setEntity(getEntity());
        externalLinks.setFont(AbstractUtility.getFont(Resources.COLOR_FONT_DEFAULT));

        addControl(externalLinks);
        addElement(externalLinks);
        this.getLayoutComposite().layout();
    }

    @Override
    public void handleEvent(Object eventSource) {
        if (eventSource.equals(select_nameUsedInSource)){
        	getEntity().setNameUsedInSource(select_nameUsedInSource.getEntity());
        }else{
        	super.handleEvent(eventSource);
        }
    }
}