/**
 * Copyright (C) 2007 EDIT
 * European Distributed Institute of Taxonomy
 * http://www.e-taxonomy.eu
 *
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * See LICENSE.TXT at the top of this package for the full license terms.
 */
package eu.etaxonomy.taxeditor.ui.section.supplemental;

import org.eclipse.swt.events.SelectionListener;

import eu.etaxonomy.cdm.model.common.SingleSourcedEntityBase;
import eu.etaxonomy.cdm.model.reference.NamedSource;
import eu.etaxonomy.taxeditor.ui.element.AbstractFormSection;
import eu.etaxonomy.taxeditor.ui.element.CdmFormFactory;
import eu.etaxonomy.taxeditor.ui.element.ICdmFormElement;
import eu.etaxonomy.taxeditor.ui.element.ISelectableElement;
import eu.etaxonomy.taxeditor.ui.element.SelectionArbitrator;
import eu.etaxonomy.taxeditor.ui.section.AbstractEntityCollectionElement;
import eu.etaxonomy.taxeditor.ui.section.reference.OriginalSourceElement;

/**
 * @author n.hoffmann
 * @created Mar 25, 2010
 */
public abstract class AbstractSourcedEntityElement<T extends SingleSourcedEntityBase>
		extends AbstractEntityCollectionElement<T> implements ISelectableElement {

	private SelectionArbitrator selectionArbitrator;
	private OriginalSourceElement sourceElement;
	private String label = "Source";
//	protected EntitySelectionElement<Reference> selection_reference;
//	protected TextWithLabelElement text_referenceDetail;

	protected boolean isCommonNameReference = false;

	public AbstractSourcedEntityElement(CdmFormFactory formFactory,
			AbstractFormSection section, T entity,
			SelectionListener removeListener, String label, int style, boolean isCommonNameReference) {
		super(formFactory, section, entity, removeListener, null, style);
		// make this element selectable
		if(formFactory.getSelectionProvider() != null){
			selectionArbitrator = formFactory.createSelectionArbitrator(this);
		}
		this.isCommonNameReference = isCommonNameReference;
		


	}

    public AbstractSourcedEntityElement(CdmFormFactory formFactory,
            AbstractFormSection section, T entity,
            SelectionListener removeListener, String label, int style) {
        this(formFactory, section, entity, removeListener, label, style, false);
    }


    @Override
	public void createControls(ICdmFormElement formElement, int style) {

	    sourceElement = formFactory.createOriginalSourceElement(formElement, entity, label);

        if (entity.getSource() == null){
            NamedSource source = NamedSource.NewPrimarySourceInstance(null, null);
            entity.setSource(source);
            sourceElement.setEntity(source);
        }else{
            sourceElement.setEntity(entity.getSource());

        }
        
	}

	@Override
	public void setEntity(T entity) {
		this.entity = entity;
	}

	@Override
    public SelectionArbitrator getSelectionArbitrator() {
		return selectionArbitrator;
	}

	public void setSourceLabel(String sourceLabel){
	    this.label = sourceLabel;
	    if (sourceElement != null){
	        this.sourceElement.getReferenceSelectionElement().setLabel(sourceLabel);
	        this.sourceElement.refresh();
	    }
	}

}
