/**
 *
 */
package eu.etaxonomy.taxeditor.ui.section.supplemental;

import java.util.Collection;
import java.util.Comparator;

import eu.etaxonomy.cdm.api.conversation.ConversationHolder;
import eu.etaxonomy.cdm.model.common.Credit;
import eu.etaxonomy.cdm.model.common.IdentifiableEntity;
import eu.etaxonomy.taxeditor.ui.element.CdmFormFactory;
import eu.etaxonomy.taxeditor.ui.element.ICdmFormElement;
import eu.etaxonomy.taxeditor.ui.section.AbstractEntityCollectionSection;
import eu.etaxonomy.taxeditor.ui.section.DefaultCdmBaseComparator;

/**
 * <p>CreditSection class.</p>
 *
 * @author n.hoffmann
 * @created Nov 16, 2009
 */
public class CreditSection extends AbstractEntityCollectionSection<IdentifiableEntity, Credit> {

	/**
	 * <p>Constructor for CreditSection.</p>
	 *
	 * @param parentElement a {@link eu.etaxonomy.taxeditor.ui.element.ICdmFormElement} object.
	 * @param style a int.
	 * @param cdmFormFactory a {@link eu.etaxonomy.taxeditor.ui.element.CdmFormFactory} object.
	 * @param conversation a {@link eu.etaxonomy.cdm.api.conversation.ConversationHolder} object.
	 */
	public CreditSection(CdmFormFactory cdmFormFactory, ConversationHolder conversation, ICdmFormElement parentElement, int style) {
		super(cdmFormFactory, conversation, parentElement, "Credits", style);
	}

	/** {@inheritDoc} */
	@Override
	public Credit createNewElement() {
		return Credit.NewInstance(null, null, null);
	}

	/** {@inheritDoc} */
	@Override
	public Collection<Credit> getCollection(IdentifiableEntity entity) {
		return entity.getCredits();
	}

	@Override
	public Comparator<Credit> getComparator() {
        return new DefaultCdmBaseComparator<>();
	}

	/** {@inheritDoc} */
	@Override
	public String getEmptyString() {
		return  "No credits yet.";
	}

	/** {@inheritDoc} */
	@Override
	protected String getTooltipString() {
		return "Create a new credit entry";
	}

	/** {@inheritDoc} */
	@Override
	public void addElement(Credit element) {
		getEntity().addCredit(element);
	}

	/** {@inheritDoc} */
	@Override
	public void removeElement(Credit element) {
		getEntity().removeCredit(element);
	}

    /**
     * {@inheritDoc}
     */
    @Override
    public Credit addExisting() {
        // TODO Auto-generated method stub
        return null;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public boolean allowAddExisting() {
        // TODO Auto-generated method stub
        return false;
    }

}
