/**
 *
 */
package eu.etaxonomy.taxeditor.ui.section.supplemental;

import java.util.Collection;
import java.util.Comparator;

import eu.etaxonomy.cdm.api.conversation.ConversationHolder;
import eu.etaxonomy.cdm.model.common.Extension;
import eu.etaxonomy.cdm.model.common.IdentifiableEntity;
import eu.etaxonomy.taxeditor.ui.element.CdmFormFactory;
import eu.etaxonomy.taxeditor.ui.element.ICdmFormElement;
import eu.etaxonomy.taxeditor.ui.section.AbstractEntityCollectionSection;
import eu.etaxonomy.taxeditor.ui.section.DefaultCdmBaseComparator;

/**
 * <p>ExtensionSection class.</p>
 *
 * @author nho
 */
public class ExtensionSection extends AbstractEntityCollectionSection<IdentifiableEntity, Extension> {

	/**
	 * <p>Constructor for ExtensionSection.</p>
	 *
	 * @param parentElement a {@link eu.etaxonomy.taxeditor.ui.element.ICdmFormElement} object.
	 * @param style a int.
	 * @param cdmFormFactory a {@link eu.etaxonomy.taxeditor.ui.element.CdmFormFactory} object.
	 * @param conversation a {@link eu.etaxonomy.cdm.api.conversation.ConversationHolder} object.
	 */
	public ExtensionSection(CdmFormFactory cdmFormFactory, ConversationHolder conversation, ICdmFormElement parentElement, int style) {
		super(cdmFormFactory, conversation,parentElement, "Extensions", style);
	}

	@Override
	public Extension createNewElement() {
		return Extension.NewInstance();
	}

	@Override
	public Collection<Extension> getCollection(IdentifiableEntity entity) {
		return entity.getExtensions();
	}

	@Override
	public Comparator<Extension> getComparator() {
        return new DefaultCdmBaseComparator<>();
	}

	@Override
	public String getEmptyString() {
		return "No extensions yet.";
	}

	@Override
	protected String getTooltipString() {
		return "Create a new extension";
	}

	@Override
	public void addElement(Extension element) {
		getEntity().addExtension(element);
	}

	@Override
	public void removeElement(Extension element) {
		getEntity().removeExtension(element);
	}

    @Override
    public Extension addExisting() {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public boolean allowAddExisting() {
        // TODO Auto-generated method stub
        return false;
    }

}
