/**
* Copyright (C) 2021 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.ui.section.supplemental;

import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;

import eu.etaxonomy.cdm.model.common.IdentifiableSource;
import eu.etaxonomy.cdm.model.metadata.PreferencePredicate;
import eu.etaxonomy.taxeditor.preference.PreferencesUtil;
import eu.etaxonomy.taxeditor.store.StoreUtil;
import eu.etaxonomy.taxeditor.ui.element.AbstractFormSection;
import eu.etaxonomy.taxeditor.ui.element.CdmFormFactory;
import eu.etaxonomy.taxeditor.ui.element.ICdmFormElement;
import eu.etaxonomy.taxeditor.ui.element.LayoutConstants;
import eu.etaxonomy.taxeditor.ui.section.common.ExternalLinksSection;

/**
 * @author n.hoffmann
 * @created Nov 16, 2009
 */
public class IdentifiableSourceElement extends AbstractOriginalSourceElement<IdentifiableSource>{

    boolean isEnabled = true;

	public IdentifiableSourceElement(CdmFormFactory cdmFormFactory, AbstractFormSection formElement, IdentifiableSource element,
			SelectionListener removeListener, int style, boolean isEnabled) {
		super(cdmFormFactory, formElement, element, removeListener, style);
		this.isEnabled = isEnabled;
		for (Control control: this.getControls()){
            control.setEnabled(isEnabled);
            if (control instanceof Composite){
                for (Control co: ((Composite)control).getChildren()){
                    co.setEnabled(isEnabled);
                }
            }
        }
	}

	public IdentifiableSourceElement(CdmFormFactory cdmFormFactory, AbstractFormSection formElement, IdentifiableSource element,
            SelectionListener removeListener, int style) {
        super(cdmFormFactory, formElement, element, removeListener, style);
    }

	@Override
	public void createControls(ICdmFormElement formElement, int style) {
		super.createControls(formElement, style);
		externalLinks = formFactory.createExternalLinksSection(getConversationHolder(), formElement, StoreUtil.getSectionStyle(ExternalLinksSection.class, IdentifiableSource.class.getCanonicalName()));
        externalLinks.setLayoutData(LayoutConstants.FILL_HORIZONTALLY(2, 1));
        if (entity != null){
        	setEntity(this.entity);
        }
	}

	@Override
	public void setEntity(IdentifiableSource entity) {
		super.setEntity(entity);
		if (selection_reference != null){
			if (PreferencesUtil.getBooleanValue(PreferencePredicate.ShowIdInSource.getKey())){
			    text_idInSource.setText(entity.getIdInSource());
			}
			if (PreferencesUtil.getBooleanValue(PreferencePredicate.ShowNamespaceInSource.getKey())){
			    text_idNamespace.setText(entity.getIdNamespace());
			}

			selection_reference.setEntity(entity.getCitation());
			text_referenceDetail.setText(entity.getCitationMicroReference());
			combo_origsourcetype.setSelection(entity.getType());
			text_originalInfo.setText(entity.getOriginalInfo());
		}
	}

	@Override
	public void handleEvent(Object eventSource) {
	    super.handleEvent(eventSource);
	}
}
