package eu.etaxonomy.taxeditor.ui.section.supplemental;

import java.util.Collection;
import java.util.Comparator;

import eu.etaxonomy.cdm.api.conversation.ConversationHolder;
import eu.etaxonomy.cdm.model.common.AnnotatableEntity;
import eu.etaxonomy.cdm.model.common.Marker;
import eu.etaxonomy.taxeditor.ui.element.CdmFormFactory;
import eu.etaxonomy.taxeditor.ui.element.ICdmFormElement;
import eu.etaxonomy.taxeditor.ui.section.DefaultCdmBaseComparator;
import eu.etaxonomy.taxeditor.ui.section.occurrence.dna.AbstractUnboundEntityCollectionSection;

/**
 * <p>MarkerSection class.</p>
 *
 * @author nho
 */
public class MarkerSection extends AbstractUnboundEntityCollectionSection<AnnotatableEntity, Marker> {

	/**
	 * <p>Constructor for MarkerSection.</p>
	 *
	 * @param parentElement a {@link eu.etaxonomy.taxeditor.ui.element.ICdmFormElement} object.
	 * @param style a int.
	 * @param cdmFormFactory a {@link eu.etaxonomy.taxeditor.ui.element.CdmFormFactory} object.
	 * @param conversation a {@link eu.etaxonomy.cdm.api.conversation.ConversationHolder} object.
	 */
	public MarkerSection(CdmFormFactory cdmFormFactory, ConversationHolder conversation, ICdmFormElement parentElement, int style) {
		super(cdmFormFactory, conversation, parentElement, "Marker", style);
	}

	@Override
	public Marker createNewElement() {
		Marker marker = Marker.NewInstance();
		marker.setFlag(true);
        return marker;
	}

	@Override
	public String getEmptyString() {
		return "No markers yet.";
	}

	@Override
	protected String getTooltipString() {
		return "Create a new marker";
	}

	@Override
	public void addElement(Marker element) {
		getEntity().addMarker(element);
	}

	@Override
	public void removeElement(Marker element) {
		getEntity().removeMarker(element);
	}

    @Override
    protected Collection<Marker> getEntityCollection(AnnotatableEntity entity) {
        return entity.getMarkers();
    }

    @Override
    public Comparator<Marker> getComparator() {
        return new DefaultCdmBaseComparator<>();
    }

    @Override
    public Marker addExisting() {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public boolean allowAddExisting() {
        // TODO Auto-generated method stub
        return false;
    }

}
