/**
 *
 */
package eu.etaxonomy.taxeditor.ui.section.supplemental;

import java.util.Collection;
import java.util.Comparator;

import eu.etaxonomy.cdm.api.conversation.ConversationHolder;
import eu.etaxonomy.cdm.model.common.IdentifiableEntity;
import eu.etaxonomy.cdm.model.media.Rights;
import eu.etaxonomy.taxeditor.ui.dialog.selection.RightsSelectionDialog;
import eu.etaxonomy.taxeditor.ui.element.CdmFormFactory;
import eu.etaxonomy.taxeditor.ui.element.ICdmFormElement;
import eu.etaxonomy.taxeditor.ui.section.AbstractEntityCollectionSection;
import eu.etaxonomy.taxeditor.ui.section.DefaultCdmBaseComparator;

/**
 * <p>RightsSection class.</p>
 *
 * @author nho
 */
public class RightsSection extends AbstractEntityCollectionSection<IdentifiableEntity, Rights> {

	/**
	 * <p>Constructor for RightsSection.</p>
	 *
	 * @param parentElement a {@link eu.etaxonomy.taxeditor.ui.element.ICdmFormElement} object.
	 * @param style a int.
	 * @param cdmFormFactory a {@link eu.etaxonomy.taxeditor.ui.element.CdmFormFactory} object.
	 * @param conversation a {@link eu.etaxonomy.cdm.api.conversation.ConversationHolder} object.
	 */
	public RightsSection(CdmFormFactory cdmFormFactory, ConversationHolder conversation, ICdmFormElement parentElement, int style) {
		super(cdmFormFactory, conversation, parentElement, "Rights", style);
	}

	@Override
	public Rights createNewElement() {
		return Rights.NewInstance();
	}

	@Override
	public Collection<Rights> getCollection(IdentifiableEntity entity) {
		return entity.getRights();
	}

	@Override
	public Comparator<Rights> getComparator() {
        return new DefaultCdmBaseComparator<>();
	}

	@Override
	public String getEmptyString() {
		return "No rights yet.";
	}

	@Override
	protected String getTooltipString() {
		// TODO Auto-generated method stub
		return "Create new rights.";
	}

	@Override
	public void addElement(Rights element) {
		getEntity().addRights(element);
	}

	@Override
	public void removeElement(Rights element) {
		getEntity().removeRights(element);
	}

    @Override
    public Rights addExisting() {
        return RightsSelectionDialog.select(getShell(), null);
    }

    @Override
    public boolean allowAddExisting() {
        return true;
    }


}
