/**
 * Copyright (C) 2007 EDIT
 * European Distributed Institute of Taxonomy
 * http://www.e-taxonomy.eu
 *
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * See LICENSE.TXT at the top of this package for the full license terms.
 */
package eu.etaxonomy.taxeditor.ui.section.taxon;

import org.eclipse.jface.util.PropertyChangeEvent;
import org.eclipse.swt.SWT;

import eu.etaxonomy.cdm.model.taxon.SecundumSource;
import eu.etaxonomy.cdm.model.taxon.Synonym;
import eu.etaxonomy.cdm.model.taxon.SynonymType;
import eu.etaxonomy.cdm.model.taxon.TaxonBase;
import eu.etaxonomy.cdm.persistence.permission.Role;
import eu.etaxonomy.taxeditor.preference.NameDetailsConfigurator;
import eu.etaxonomy.taxeditor.preference.PreferencesUtil;
import eu.etaxonomy.taxeditor.store.CdmStore;
import eu.etaxonomy.taxeditor.ui.combo.term.TermComboElement;
import eu.etaxonomy.taxeditor.ui.element.CdmFormFactory;
import eu.etaxonomy.taxeditor.ui.element.CheckboxElement;
import eu.etaxonomy.taxeditor.ui.element.ICdmFormElement;
import eu.etaxonomy.taxeditor.ui.element.TextWithLabelElement;
import eu.etaxonomy.taxeditor.ui.section.AbstractIdentifiableEntityDetailElement;
import eu.etaxonomy.taxeditor.ui.section.reference.SecundumSourceElement;

/**
 * @author n.hoffmann
 * @created Feb 26, 2010
 */
public class TaxonBaseDetailElement extends AbstractIdentifiableEntityDetailElement<TaxonBase<?>> {

	private CheckboxElement checkbox_doubtful;

	private CheckboxElement checkbox_useNameCache;

//	private EntitySelectionElement<Reference> selection_secundum;
    private SecundumSourceElement secElement;


	private TextWithLabelElement text_appendedPhrase;

	private CheckboxElement checkbox_published;

    private TermComboElement<SynonymType> combo_relationshipType;

	public TaxonBaseDetailElement(CdmFormFactory cdmFormFactory,
			ICdmFormElement formElement, int style) {
		super(cdmFormFactory, formElement);
	}

	@Override
	protected void createControls(ICdmFormElement formElement, TaxonBase<?> entity, int style) {

	    NameDetailsConfigurator config = PreferencesUtil.getPreferredNameDetailsConfiguration();
		toggleable_cache = formFactory.createToggleableTextField(formElement,
					"Title Cache", entity.getTitleCache(),
					entity.isProtectedTitleCache(), style);

		text_appendedPhrase = formFactory.createTextWithLabelElement(
					formElement, "Appended Phrase", entity.getAppendedPhrase(),
					SWT.WRAP);

		secElement = formFactory.createSecundumSourceElement(formElement, null, "Secundum");

		secElement.setCdmEntity(entity);
		 if (entity.getSecSource() == null){
	            SecundumSource source = SecundumSource.NewSecundumInstance(entity);
	            entity.setSecSource(source);
	            secElement.setEntity(source);
	        }else{
	            secElement.setEntity(entity.getSecSource());
	        }

		if (config != null && !config.isSecEnabled()){
		    secElement.setEnabled(false);
		}


		checkbox_useNameCache = formFactory.createCheckbox(formElement,
					"Exclude Authorship", entity.isUseNameCache(), style);
		checkbox_doubtful = formFactory.createCheckbox(formElement,
					entity.getClass().getSimpleName() + " is doubtful", entity.isDoubtful(), style);


		checkbox_published = formFactory.createCheckbox(formElement,
		        entity.getClass().getSimpleName()+" is published", entity.isPublish(), style);


	    //cache relevance
        registerCacheRelevance(checkbox_doubtful);
        registerCacheRelevance(checkbox_useNameCache);
        registerCacheRelevance(secElement);
        registerCacheRelevance(text_appendedPhrase);
        registerCacheRelevance(combo_relationshipType);
	}

    private void updateCheckboxPublish() {
        checkbox_published.setEnabled(
            		CdmStore.currentAuthentiationHasOneOfRoles(Role.ROLE_PUBLISH, Role.ROLE_ADMIN)
            		);

    }

	@Override
    protected void updateControlStates() {
        super.updateControlStates();
        updateCheckboxPublish();
    }

	@Override
	protected void updateContent() {
		super.updateContent();
		toggleable_cache.setCacheEnabled(getEntity().isProtectedTitleCache());
		updateCacheRelevance();
	}

	@Override
	protected void handleTitleCacheRelevantChange(PropertyChangeEvent event) {
		String title = getEntity().generateTitle();
		getEntity().setTitleCache(title);
		super.handleTitleCacheRelevantChange(event);
	}

	@Override
	public void handleEvent(Object eventSource) {
		if (eventSource== toggleable_cache) {
			handleToggleableCacheField();
		} else if (eventSource == checkbox_doubtful) {
			getEntity().setDoubtful(checkbox_doubtful.getSelection());
		} else if (eventSource == checkbox_useNameCache) {
			getEntity().setUseNameCache(checkbox_useNameCache.getSelection());
		} else if (eventSource == secElement) {
			getEntity().setSecSource(secElement.getEntity());
		} else if (eventSource == text_appendedPhrase) {
			getEntity().setAppendedPhrase(text_appendedPhrase.getText());
		} else if (eventSource == checkbox_published) {
		    getEntity().setPublish(checkbox_published.getSelection());
		} else if(eventSource == combo_relationshipType){
            ((Synonym) getEntity()).setType(combo_relationshipType.getSelection());
		//#7334/5 removed pro parte and partial as they are handled as concept relationships now
//        } else if(eventSource == checkPartial){
//            ((Synonym) getEntity()).setPartial(checkPartial.getSelection());
//        } else if(eventSource == checkProParte){
//            ((Synonym) getEntity()).setProParte(checkProParte.getSelection());
        }
	}

	@Override
	protected void handleToggleableCacheField() {
        getEntity().setTitleCache(toggleable_cache.getText(), toggleable_cache.getState());
        updateCacheRelevance();
        updateToggleableCacheField();
    }
}