/**
* Copyright (C) 2009 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.ui.section.vocabulary;

import org.eclipse.swt.widgets.Button;

import eu.etaxonomy.cdm.model.common.Language;
import eu.etaxonomy.cdm.model.term.Representation;
import eu.etaxonomy.cdm.model.term.TermBase;
import eu.etaxonomy.cdm.model.term.TermVocabulary;
import eu.etaxonomy.taxeditor.editor.definedterm.TermBasePropertyTester;
import eu.etaxonomy.taxeditor.store.CdmStore;
import eu.etaxonomy.taxeditor.ui.element.CdmFormFactory;
import eu.etaxonomy.taxeditor.ui.element.ICdmFormElement;
import eu.etaxonomy.taxeditor.ui.element.RepresentationElement;
import eu.etaxonomy.taxeditor.ui.element.UriWithLabelElement;
import eu.etaxonomy.taxeditor.ui.section.AbstractCdmDetailElement;

/**
 * @author l.morris
 * @date 20 Dec 2011
 *
 */
public abstract class AbstractTermBaseDetailElement<T extends TermBase> extends AbstractCdmDetailElement<T> {

	protected UriWithLabelElement uri_uri;
	protected RepresentationElement element_Representation;
	private Button button;


	public AbstractTermBaseDetailElement(CdmFormFactory formFactory,
			ICdmFormElement formElement) {
		super(formFactory, formElement);
	}

	@Override
	protected void createControls(ICdmFormElement formElement,
			T entity, int style) {
		element_Representation = formFactory.createTranslatableRepresentationElement(formElement,  getEntity().getPreferredRepresentation(CdmStore.getDefaultLanguage()),getEntity(), 100, style, true);
		uri_uri = formFactory.createUriWithLabelElement(formElement, "URI", getEntity().getUri(), style);
	}

    @Override
	public void setEntity(T entity) {
		super.setEntity(entity);
		setEnabled(TermBasePropertyTester.isModifiable(entity));
	}

    @Override
    public void handleEvent(Object eventSource) {
        if (eventSource == uri_uri) {
            getEntity().setUri(uri_uri.parseText());
        }
        else if (eventSource == element_Representation) {
            Representation selectedRepresentation = element_Representation.getSelectedRepresentation();
            if(selectedRepresentation!=null){
                Language representationLanguage = selectedRepresentation.getLanguage();
                if(representationLanguage==null){
                    representationLanguage = Language.getDefaultLanguage();
                }
                getEntity().setLabel(selectedRepresentation.getLabel(),  representationLanguage);
                getEntity().getRepresentation(representationLanguage).setAbbreviatedLabel(selectedRepresentation.getAbbreviatedLabel());
                getEntity().getRepresentation(representationLanguage).setText(selectedRepresentation.getDescription());
            }
            getEntity().setTitleCache(null);
        }
    }

	public abstract TermVocabulary getVocabulary();

}
