/**
* Copyright (C) 2009 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.ui.section.vocabulary;

import org.apache.commons.lang3.StringUtils;
import org.eclipse.swt.widgets.Composite;

import eu.etaxonomy.cdm.model.term.DefinedTermBase;
import eu.etaxonomy.cdm.model.term.TermVocabulary;
import eu.etaxonomy.cdm.persistence.permission.TargetEntityStates;
import eu.etaxonomy.taxeditor.ui.element.CdmFormFactory;
import eu.etaxonomy.taxeditor.ui.element.ICdmFormElement;
import eu.etaxonomy.taxeditor.ui.element.TextWithLabelAndSetNullElement;
import eu.etaxonomy.taxeditor.ui.element.TextWithLabelElement;

/**
 * @author l.morris
 * @date 20 Dec 2011
 */
public class DefinedTermDetailElement<T extends DefinedTermBase> 
			extends AbstractTermBaseDetailElement<T> {

    protected TextWithLabelElement textIdInVocabulary;

    protected TextWithLabelAndSetNullElement textSymbol;

    protected TextWithLabelAndSetNullElement textSymbol2;

	/**
	 * @param formFactory
	 * @param formElement
	 */
	public DefinedTermDetailElement(CdmFormFactory formFactory,
			ICdmFormElement formElement) {
		super(formFactory, formElement);
	}

	@Override
    protected void createControls(ICdmFormElement formElement, T entity, int style) {
	    super.createControls(formElement, entity, style);
	    textIdInVocabulary = formFactory.createTextWithLabelElement(formElement, "ID in vocabulary", entity.getIdInVocabulary(), style);
	    String symbolString = entity.getSymbol();
	    if (entity.getSymbol() == null) {
	    	symbolString = "[NULL]";
	    }
	    String symbolString2 = entity.getSymbol2();
	    if (entity.getSymbol2() == null) {
	    	symbolString2 = "[NULL]";
	    }
	    textSymbol = formFactory.createTextWithLabelAndSetNullElement(formElement, "Symbol", symbolString, style);
	    textSymbol2 = formFactory.createTextWithLabelAndSetNullElement(formElement, "Symbol 2", symbolString2, style);
	};

	@Override
	public void handleEvent(Object eventSource) {
	    super.handleEvent(eventSource);
		if(eventSource==textIdInVocabulary){
			if (StringUtils.isBlank(textIdInVocabulary.getText())){
				getEntity().setIdInVocabulary(null);
			}else{
				getEntity().setIdInVocabulary(textIdInVocabulary.getText());
			}

		}
		if(eventSource==textSymbol){
			if (textSymbol.getText().equals("[NULL]")){
				getEntity().setSymbol(null);
			}else {
				getEntity().setSymbol(textSymbol.getText());
			}
        }

		if(eventSource==textSymbol2){
			if (textSymbol2.getText().equals("[NULL]")){
				getEntity().setSymbol2(null);
			}else {
				getEntity().setSymbol2(textSymbol2.getText());
			}            

        }
	}

	@Override
	public TermVocabulary getVocabulary() {
		return getEntity() != null ? getEntity().getVocabulary() : null;
	}

}
