/**
* Copyright (C) 2009 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.ui.section.vocabulary;

import org.eclipse.ui.forms.widgets.TableWrapData;

import eu.etaxonomy.cdm.model.description.Feature;
import eu.etaxonomy.taxeditor.store.StoreUtil;
import eu.etaxonomy.taxeditor.ui.element.CdmFormFactory;
import eu.etaxonomy.taxeditor.ui.element.CheckboxElement;
import eu.etaxonomy.taxeditor.ui.element.ICdmFormElement;
import eu.etaxonomy.taxeditor.ui.element.LabelElement;
import eu.etaxonomy.taxeditor.ui.element.LayoutConstants;

/**
 * @author l.morris
 * @date 20 Dec 2011
 *
 */
public class FeatureDetailElement extends DefinedTermDetailElement<Feature> {

    private ICdmFormElement parentFormElement;

    private CheckboxElement supportsTextData;

    private CheckboxElement supportsQuantitativeData;

    private CheckboxElement supportsDistribution;

    private CheckboxElement supportsIndividualAssociation;

    private CheckboxElement supportsTaxonInteraction;

    private CheckboxElement supportsCategoricalData;

    private CheckboxElement supportsCommonTaxonName;

    private CheckboxElement supportsTemporalData;

    private StateVocabularyCollectionSection sectionStateVocabularies;

    private MeasurementUnitCollectionSection sectionMeasurementUnits;

    private StatisticalMeasureCollectionSection sectionStatisticalMeasures;

    private RecommendedModifierVocabulariesCollectionSection sectionModifierVocabularies;
    private CheckboxElement availableForTaxon;
    private CheckboxElement availableForTaxonName;
    private CheckboxElement availableForSpecimenOrObservation;

    /**
	 * @param formFactory
	 * @param formElement
	 */
	public FeatureDetailElement(CdmFormFactory formFactory,
			ICdmFormElement formElement) {
		super(formFactory, formElement);
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.ui.section.vocabulary.AbstractTermBaseDetailElement#createControls(eu.etaxonomy.taxeditor.ui.element.ICdmFormElement, eu.etaxonomy.cdm.model.term.TermBase, int)
	 */
	@Override
	protected void createControls(ICdmFormElement formElement, Feature entity, int style) {
	    super.createControls(formElement, entity, style);
	    this.parentFormElement = formElement;
	    LabelElement supportsLabel = formFactory.createLabel(formElement, "Supports");
	    supportsLabel.setBold();
	    supportsTextData = formFactory.createCheckbox(formElement, "Text Data", entity.isSupportsTextData(), style);
	    supportsCategoricalData = formFactory.createCheckbox(formElement, "Categorical Data", entity.isSupportsCategoricalData(), style);
	    supportsQuantitativeData = formFactory.createCheckbox(formElement, "Quantitative Data", entity.isSupportsQuantitativeData(), style);
	    supportsDistribution = formFactory.createCheckbox(formElement, "Distribution", entity.isSupportsDistribution(), style);
	    supportsIndividualAssociation = formFactory.createCheckbox(formElement, "Individual Association", entity.isSupportsIndividualAssociation(), style);
	    supportsTaxonInteraction = formFactory.createCheckbox(formElement, "Taxon Interaction", entity.isSupportsTaxonInteraction(), style);
	    supportsCommonTaxonName = formFactory.createCheckbox(formElement, "Common Taxon Name", entity.isSupportsCommonTaxonName(), style);
	    supportsTemporalData = formFactory.createCheckbox(formElement, "Temporal Data", entity.isSupportsTemporalData(), style);

        if (supportsCategoricalData.getSelection()) {
            sectionStateVocabularies = formFactory.createStateVocabulariesSection(getConversationHolder(),
                    parentFormElement, StoreUtil.getSectionStyle(StateVocabularyCollectionSection.class, getEntity().getClass().getCanonicalName()));
            sectionStateVocabularies.setLayoutData(LayoutConstants.FILL_HORIZONTALLY(2, 1));
            sectionStateVocabularies.setEntity(getEntity());

        }
        if (supportsQuantitativeData.getSelection()) {
            sectionMeasurementUnits = formFactory.createMeasurementUnitCollectionSection(getConversationHolder(),
                    parentFormElement, StoreUtil.getSectionStyle(MeasurementUnitCollectionSection.class, getEntity().getClass().getCanonicalName()));
            sectionMeasurementUnits.setLayoutData(LayoutConstants.FILL_HORIZONTALLY(2, 1));
            sectionMeasurementUnits.setEntity(getEntity());

            sectionStatisticalMeasures = formFactory.createStatisticalMeasureCollectionSection(getConversationHolder(),
                    parentFormElement, StoreUtil.getSectionStyle(StatisticalMeasureCollectionSection.class, getEntity().getClass().getCanonicalName()));
            sectionStatisticalMeasures.setLayoutData(LayoutConstants.FILL_HORIZONTALLY(2, 1));
            sectionStatisticalMeasures.setEntity(getEntity());
        }
        if (supportsCategoricalData.getSelection() || supportsQuantitativeData.getSelection()) {
            sectionModifierVocabularies = formFactory.createRecommendedModifierVocabulariesCollectionSection(
                    getConversationHolder(), parentFormElement, StoreUtil.getSectionStyle(RecommendedModifierVocabulariesCollectionSection.class, getEntity().getClass().getCanonicalName()));
            sectionModifierVocabularies.setLayoutData(LayoutConstants.FILL_HORIZONTALLY(2, 1));
            sectionModifierVocabularies.setEntity(getEntity());
        }

        LabelElement noLabel = formFactory.createLabel(formElement, "");

        TableWrapData data = LayoutConstants.FILL_HORIZONTALLY(2, 1);
        data.maxHeight = 3;
        noLabel.setLayout(data);
        LabelElement availableLabel = formFactory.createLabel(formElement, "Available for");
        availableLabel.setBold();

        availableForTaxon = formFactory.createCheckbox(formElement, "Taxon", entity.isAvailableForTaxon(), style);
        availableForTaxonName = formFactory.createCheckbox(formElement, "Taxon Name", entity.isAvailableForTaxonName(), style);
        availableForSpecimenOrObservation = formFactory.createCheckbox(formElement, "Occurrence", entity.isAvailableForOccurrence(), style);
    }

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.ui.section.vocabulary.DefinedTermDetailElement#handleEvent(java.lang.Object)
	 */
	@Override
	public void handleEvent(Object eventSource) {
	    super.handleEvent(eventSource);
	    if(eventSource == supportsTextData){
	        getEntity().setSupportsTextData(supportsTextData.getSelection());
	    }
	    if(eventSource == supportsQuantitativeData){
            getEntity().setSupportsQuantitativeData(supportsQuantitativeData.getSelection());
            if(supportsQuantitativeData.getSelection()){
                activateQuantitativeWidgets();
            }
            else{
                removeQuantitativeWidgets();
            }
            StoreUtil.reflowParentScrolledForm(getLayoutComposite(), true);
        }
        else if(eventSource == supportsCategoricalData){
            getEntity().setSupportsCategoricalData(supportsCategoricalData.getSelection());
            if (supportsCategoricalData.getSelection()) {
                activateCategoricalWidgets();
            }
            else{
                removeCategoricalWidgets();
            }
            StoreUtil.reflowParentScrolledForm(getLayoutComposite(), true);
        }
	    else if(eventSource == supportsDistribution){
	        getEntity().setSupportsDistribution(supportsDistribution.getSelection());
	    }
	    else if(eventSource == supportsIndividualAssociation){
	        getEntity().setSupportsIndividualAssociation(supportsIndividualAssociation.getSelection());
	    }
	    else if(eventSource == supportsTaxonInteraction){
	        getEntity().setSupportsTaxonInteraction(supportsTaxonInteraction.getSelection());
	    }
	    else if(eventSource == supportsCommonTaxonName){
	        getEntity().setSupportsCommonTaxonName(supportsCommonTaxonName.getSelection());
	    }
	    else if(eventSource == supportsTemporalData){
            getEntity().setSupportsTemporalData(supportsTemporalData.getSelection());
        }
	    else if(eventSource == availableForSpecimenOrObservation){
            getEntity().setAvailableForOccurrence(availableForSpecimenOrObservation.getSelection());
            
        }
	    else if(eventSource == availableForTaxon){
            getEntity().setAvailableForTaxon(availableForTaxon.getSelection());
            
        }
	    else if(eventSource == availableForTaxonName){
            getEntity().setAvailableForTaxonName(availableForTaxonName.getSelection());
            
        }

        StoreUtil.reflowParentScrolledForm(getLayoutComposite(), true);
	}

	private void activateQuantitativeWidgets(){
        //disable categorical widgets
        supportsCategoricalData.setSelection(false);
        getEntity().setSupportsCategoricalData(false);
        removeCategoricalWidgets();

        //measurement units
        sectionMeasurementUnits = formFactory.createMeasurementUnitCollectionSection(getConversationHolder(),
                parentFormElement, StoreUtil.getSectionStyle(MeasurementUnitCollectionSection.class, getEntity().getClass().getCanonicalName()));
        sectionMeasurementUnits.setLayoutData(LayoutConstants.FILL_HORIZONTALLY(2, 1));
        sectionMeasurementUnits.setEntity(getEntity());

        //statistical measures
        sectionStatisticalMeasures = formFactory.createStatisticalMeasureCollectionSection(
                getConversationHolder(), parentFormElement,StoreUtil.getSectionStyle(StatisticalMeasureCollectionSection.class, getEntity().getClass().getCanonicalName()));
        sectionStatisticalMeasures.setLayoutData(LayoutConstants.FILL_HORIZONTALLY(2, 1));
        sectionStatisticalMeasures.setEntity(getEntity());

        //modifiers
        sectionModifierVocabularies = formFactory.createRecommendedModifierVocabulariesCollectionSection(
                getConversationHolder(), parentFormElement, StoreUtil.getSectionStyle(RecommendedModifierVocabulariesCollectionSection.class, getEntity().getClass().getCanonicalName()));
        sectionModifierVocabularies.setLayoutData(LayoutConstants.FILL_HORIZONTALLY(2, 1));
        sectionModifierVocabularies.setEntity(getEntity());
    }

    private void removeQuantitativeWidgets(){
        if(sectionMeasurementUnits!=null){
            removeElementsAndControls(sectionMeasurementUnits);
            removeElementsAndControls(sectionStatisticalMeasures);
            removeElementsAndControls(sectionModifierVocabularies);
        }
    }

    private void activateCategoricalWidgets(){
        //disable quantitative widgeets
        supportsQuantitativeData.setSelection(false);
        getEntity().setSupportsQuantitativeData(false);
        removeQuantitativeWidgets();

        //states
        sectionStateVocabularies = formFactory.createStateVocabulariesSection(getConversationHolder(),
                parentFormElement, StoreUtil.getSectionStyle(StateVocabularyCollectionSection.class, getEntity().getClass().getCanonicalName()));
        sectionStateVocabularies.setLayoutData(LayoutConstants.FILL_HORIZONTALLY(2, 1));
        sectionStateVocabularies.setEntity(getEntity());

        //modifiers
        sectionModifierVocabularies = formFactory.createRecommendedModifierVocabulariesCollectionSection(
                getConversationHolder(), parentFormElement, StoreUtil.getSectionStyle(RecommendedModifierVocabulariesCollectionSection.class, getEntity().getClass().getCanonicalName()));
        sectionModifierVocabularies.setLayoutData(LayoutConstants.FILL_HORIZONTALLY(2, 1));
        sectionModifierVocabularies.setEntity(getEntity());
    }

    private void removeCategoricalWidgets(){
        if(sectionStateVocabularies!=null){
            removeElementsAndControls(sectionStateVocabularies);
            removeElementsAndControls(sectionModifierVocabularies);
        }
    }

}
