/**
* Copyright (C) 2015 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.ui.section.vocabulary;

import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Color;

import eu.etaxonomy.cdm.model.term.DefinedTerm;
import eu.etaxonomy.cdm.model.term.TermType;
import eu.etaxonomy.cdm.model.term.TermVocabulary;
import eu.etaxonomy.cdm.persistence.dto.FeatureDto;
import eu.etaxonomy.cdm.persistence.dto.TermVocabularyDto;
import eu.etaxonomy.taxeditor.ui.combo.VocabularyComboElement;
import eu.etaxonomy.taxeditor.ui.element.AbstractFormSection;
import eu.etaxonomy.taxeditor.ui.element.CdmFormFactory;
import eu.etaxonomy.taxeditor.ui.element.ICdmFormElement;
import eu.etaxonomy.taxeditor.ui.section.AbstractEntityCollectionElement;

/**
 * @author pplitzner
 * @date Apr 27, 2015
 *
 */
public class RecommendedModifierDtoVocabulariesElement extends AbstractEntityCollectionElement<TermVocabularyDto> {


    private VocabularyComboElement<DefinedTerm, TermVocabulary<DefinedTerm>> comboStateVocabulary;

    /**
     * @param formFactory
     * @param section
     * @param entity
     * @param removeListener
     * @param isChoosableEntity
     * @param backgroundColor
     * @param style
     */
    public RecommendedModifierDtoVocabulariesElement(CdmFormFactory formFactory, AbstractFormSection section, TermVocabularyDto entity,
            SelectionListener removeListener, Color backgroundColor, int style) {
        super(formFactory, section, entity, removeListener, backgroundColor, style);

    }

    @Override
    public void createControls(ICdmFormElement element, int style) {
        comboStateVocabulary = formFactory.createVocabularyComboElement(TermType.Modifier, "Modifier Vocabulary", null, element, style);
        if (entity != null){
        	setEntity(entity);
        }
    }

    @Override
    public void setEntity(TermVocabularyDto entity) {
        this.entity = entity;
        if(comboStateVocabulary != null){
            comboStateVocabulary.setSelectionDto(entity);
            comboStateVocabulary.removeEmptyElement();
        }
    }


    @Override
    public void handleEvent(Object eventSource) {
        if(eventSource==comboStateVocabulary && comboStateVocabulary.getSelection()!=null){
            if(getParentElement() instanceof RecommendedModifierVocabulariesDtoCollectionSection){
                RecommendedModifierVocabulariesDtoCollectionSection parentElement = (RecommendedModifierVocabulariesDtoCollectionSection) getParentElement();
                FeatureDto feature = parentElement.getEntity();
                TermVocabularyDto remove = null;
                for (TermVocabularyDto voc: feature.getRecommendedModifierEnumeration()){
                    if (voc.getUuid().equals(getEntity().getUuid())){
                        remove = voc;
                        break;
                    }
                }
                feature.getRecommendedModifierEnumeration().remove(remove);
                TermVocabulary<DefinedTerm> vocabulary = comboStateVocabulary.getSelection();
                entity = TermVocabularyDto.fromVocabulary(vocabulary);
                feature.getRecommendedModifierEnumeration().add(entity);
                //update the parent section to re-set the listeners to the persisted measurement unit
//                parentElement.removeElementAndUpdate(null);
            }
        }
    }

}
