/**
* Copyright (C) 2015 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.ui.section.vocabulary;

import java.util.Collection;
import java.util.Comparator;

import eu.etaxonomy.cdm.api.conversation.ConversationHolder;
import eu.etaxonomy.cdm.model.description.State;
import eu.etaxonomy.cdm.model.term.TermType;
import eu.etaxonomy.cdm.model.term.TermVocabulary;
import eu.etaxonomy.cdm.persistence.dto.FeatureDto;
import eu.etaxonomy.cdm.persistence.dto.TermVocabularyDto;
import eu.etaxonomy.taxeditor.ui.element.CdmFormFactory;
import eu.etaxonomy.taxeditor.ui.element.ICdmFormElement;
import eu.etaxonomy.taxeditor.ui.section.occurrence.dna.AbstractUnboundEntityCollectionSection;

/**
 * @author pplitzner
 * @date Apr 27, 2015
 *
 */
public class RecommendedModifierVocabulariesDtoCollectionSection extends AbstractUnboundEntityCollectionSection<FeatureDto, TermVocabularyDto> {

    public RecommendedModifierVocabulariesDtoCollectionSection(CdmFormFactory formFactory,
            ConversationHolder conversation, ICdmFormElement parentElement, int style) {
        super(formFactory, conversation, parentElement, "Recommended modifier vocabularies", style);
    }

    /* (non-Javadoc)
     * @see eu.etaxonomy.taxeditor.ui.section.occurrence.dna.AbstractNullEntityCollectionSection#getEntityCollection(java.lang.Object)
     */
    /** {@inheritDoc} */
    @Override
    protected Collection<TermVocabularyDto> getEntityCollection(FeatureDto entity) {
        return entity.getRecommendedModifierEnumeration();
    }

    @Override
    public Comparator<TermVocabularyDto> getComparator() {
        return new Comparator<TermVocabularyDto>() {
            @Override
            public int compare(TermVocabularyDto o1, TermVocabularyDto o2) {
                if(o1==null){
                    return -1;
                }
                if(o2==null){
                    return 1;
                }
                int diff = o1.getTitleCache().compareTo(o2.getTitleCache());
                if(diff==0){
                    diff = o1.getUuid().compareTo(o2.getUuid());
                }
                return diff;
            }
        };
    }

    /* (non-Javadoc)
     * @see eu.etaxonomy.taxeditor.ui.section.AbstractEntityCollectionSection#createNewElement()
     */
    /** {@inheritDoc} */
    @Override
    public TermVocabularyDto createNewElement() {
        TermVocabulary<State> voc = TermVocabulary.NewInstance(TermType.Modifier);
        return new TermVocabularyDto(voc.getUuid(), voc.getRepresentations(), voc.getTermType(), voc.getTitleCache(), voc.isAllowDuplicates(), voc.isOrderRelevant(), voc.isFlat());
    }

    /* (non-Javadoc)
     * @see eu.etaxonomy.taxeditor.section.AbstractEntityCollectionSection#addElement(eu.etaxonomy.cdm.model.common.IVersionableEntity)
     */
    /** {@inheritDoc} */
    @Override
    public void addElement(TermVocabularyDto element) {
        //never gets called
    }

    /* (non-Javadoc)
     * @see eu.etaxonomy.taxeditor.section.AbstractEntityCollectionSection#removeElement(eu.etaxonomy.cdm.model.common.IVersionableEntity)
     */
    /** {@inheritDoc} */
    @Override
    public void removeElement(TermVocabularyDto element) {
        if (element == null){
            return;
        }
        TermVocabularyDto remove = null;
        for (TermVocabularyDto voc: getEntity().getSupportedCategoricalEnumerations()){
            if (voc.getUuid().equals(element.getUuid())){
                remove = voc;
                break;
            }
        }
        getEntity().getSupportedCategoricalEnumerations().remove(remove);
    }

    /* (non-Javadoc)
     * @see eu.etaxonomy.taxeditor.section.AbstractEntityCollectionSection#getEmptyString()
     */
    /** {@inheritDoc} */
    @Override
    public String getEmptyString() {
        return "No modifiers vocabularies yet.";
    }

    /* (non-Javadoc)
     * @see eu.etaxonomy.taxeditor.section.AbstractEntityCollectionSection#getTooltipString()
     */
    /** {@inheritDoc} */
    @Override
    protected String getTooltipString() {
        return "Add a modifier vocabulary";
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public TermVocabularyDto addExisting() {
        // TODO Auto-generated method stub
        return null;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public boolean allowAddExisting() {
        // TODO Auto-generated method stub
        return false;
    }



}
