/**
* Copyright (C) 2015 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.ui.section.vocabulary;

import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Color;

import eu.etaxonomy.cdm.model.term.DefinedTerm;
import eu.etaxonomy.cdm.model.term.TermType;
import eu.etaxonomy.cdm.model.term.TermVocabulary;
import eu.etaxonomy.cdm.model.description.Feature;
import eu.etaxonomy.taxeditor.ui.combo.VocabularyComboElement;
import eu.etaxonomy.taxeditor.ui.element.AbstractFormSection;
import eu.etaxonomy.taxeditor.ui.element.CdmFormFactory;
import eu.etaxonomy.taxeditor.ui.element.ICdmFormElement;
import eu.etaxonomy.taxeditor.ui.section.AbstractEntityCollectionElement;

/**
 * @author pplitzner
 * @date Apr 27, 2015
 *
 */
public class RecommendedModifierVocabulariesElement extends AbstractEntityCollectionElement<TermVocabulary<DefinedTerm>> {


    private VocabularyComboElement<DefinedTerm, TermVocabulary<DefinedTerm>> comboStateVocabulary;

    /**
     * @param formFactory
     * @param section
     * @param entity
     * @param removeListener
     * @param isChoosableEntity
     * @param backgroundColor
     * @param style
     */
    public RecommendedModifierVocabulariesElement(CdmFormFactory formFactory, AbstractFormSection section, TermVocabulary<DefinedTerm> entity,
            SelectionListener removeListener, Color backgroundColor, int style) {
        super(formFactory, section, entity, removeListener, backgroundColor, style);

    }

    @Override
    public void createControls(ICdmFormElement element, int style) {
        comboStateVocabulary = formFactory.createVocabularyComboElement(TermType.Modifier, "Modifier Vocabulary", null, element, style);
        if (entity != null){
        	setEntity(entity);
        }
    }

    @Override
    public void setEntity(TermVocabulary<DefinedTerm> entity) {
        this.entity = entity;
        if(entity.getId()!=0 && comboStateVocabulary != null){
            comboStateVocabulary.setSelection(entity);
            comboStateVocabulary.removeEmptyElement();
        }
    }

    @Override
    public void handleEvent(Object eventSource) {
        if(eventSource==comboStateVocabulary && comboStateVocabulary.getSelection()!=null){
            if(getParentElement() instanceof RecommendedModifierVocabulariesCollectionSection){
                RecommendedModifierVocabulariesCollectionSection parentElement = (RecommendedModifierVocabulariesCollectionSection) getParentElement();
                Feature feature = parentElement.getEntity();
                feature.removeRecommendedModifierEnumeration(entity);
                TermVocabulary<DefinedTerm> vocabulary = comboStateVocabulary.getSelection();
                feature.addRecommendedModifierEnumeration(vocabulary);
                entity = vocabulary;
                //update the parent section to re-set the listeners to the persisted measurement unit
                parentElement.removeElementAndUpdate(null);
            }
        }
    }

}
