/**
* Copyright (C) 2015 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.ui.section.vocabulary;

import java.util.Collection;
import java.util.Comparator;

import eu.etaxonomy.cdm.api.conversation.ConversationHolder;
import eu.etaxonomy.cdm.model.term.TermType;
import eu.etaxonomy.cdm.model.term.TermVocabulary;
import eu.etaxonomy.cdm.model.description.Feature;
import eu.etaxonomy.cdm.model.description.State;
import eu.etaxonomy.taxeditor.ui.element.CdmFormFactory;
import eu.etaxonomy.taxeditor.ui.element.ICdmFormElement;
import eu.etaxonomy.taxeditor.ui.section.DefaultCdmBaseComparator;
import eu.etaxonomy.taxeditor.ui.section.occurrence.dna.AbstractUnboundEntityCollectionSection;

/**
 * @author pplitzner
 * @date Apr 27, 2015
 *
 */
public class StateVocabularyCollectionSection extends AbstractUnboundEntityCollectionSection<Feature, TermVocabulary<State>> {

    public StateVocabularyCollectionSection(CdmFormFactory formFactory,
            ConversationHolder conversation, ICdmFormElement parentElement, int style) {
        super(formFactory, conversation, parentElement, "Supported state vocabularies", style);
    }

    /* (non-Javadoc)
     * @see eu.etaxonomy.taxeditor.ui.section.occurrence.dna.AbstractNullEntityCollectionSection#getEntityCollection(java.lang.Object)
     */
    /** {@inheritDoc} */
    @Override
    protected Collection<TermVocabulary<State>> getEntityCollection(Feature entity) {
        return entity.getSupportedCategoricalEnumerations();
    }

    @Override
    public Comparator<TermVocabulary<State>> getComparator() {
        return new DefaultCdmBaseComparator<>();
    }

    /* (non-Javadoc)
     * @see eu.etaxonomy.taxeditor.ui.section.AbstractEntityCollectionSection#createNewElement()
     */
    /** {@inheritDoc} */
    @Override
    public TermVocabulary<State> createNewElement() {
        return TermVocabulary.NewInstance(TermType.State);
    }

    /* (non-Javadoc)
     * @see eu.etaxonomy.taxeditor.section.AbstractEntityCollectionSection#addElement(eu.etaxonomy.cdm.model.common.IVersionableEntity)
     */
    /** {@inheritDoc} */
    @Override
    public void addElement(TermVocabulary<State> element) {
        //never gets called
    }

    /* (non-Javadoc)
     * @see eu.etaxonomy.taxeditor.section.AbstractEntityCollectionSection#removeElement(eu.etaxonomy.cdm.model.common.IVersionableEntity)
     */
    /** {@inheritDoc} */
    @Override
    public void removeElement(TermVocabulary<State> element) {
        getEntity().removeSupportedCategoricalEnumeration(element);
    }

    /* (non-Javadoc)
     * @see eu.etaxonomy.taxeditor.section.AbstractEntityCollectionSection#getEmptyString()
     */
    /** {@inheritDoc} */
    @Override
    public String getEmptyString() {
        return "No state vocabularies yet.";
    }

    /* (non-Javadoc)
     * @see eu.etaxonomy.taxeditor.section.AbstractEntityCollectionSection#getTooltipString()
     */
    /** {@inheritDoc} */
    @Override
    protected String getTooltipString() {
        return "Add a state vocabulary";
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public TermVocabulary<State> addExisting() {
        // TODO Auto-generated method stub
        return null;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public boolean allowAddExisting() {
        // TODO Auto-generated method stub
        return false;
    }


}
