/**
* Copyright (C) 2015 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.ui.section.vocabulary;

import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Color;

import eu.etaxonomy.cdm.model.term.TermType;
import eu.etaxonomy.cdm.model.description.Feature;
import eu.etaxonomy.cdm.model.description.StatisticalMeasure;
import eu.etaxonomy.taxeditor.ui.combo.term.TermComboElement;
import eu.etaxonomy.taxeditor.ui.element.AbstractFormSection;
import eu.etaxonomy.taxeditor.ui.element.CdmFormFactory;
import eu.etaxonomy.taxeditor.ui.element.ICdmFormElement;
import eu.etaxonomy.taxeditor.ui.section.AbstractEntityCollectionElement;

/**
 * @author pplitzner
 * @date Apr 27, 2015
 *
 */
public class StatisticalMeasureCollectionElement extends AbstractEntityCollectionElement<StatisticalMeasure> {


    private TermComboElement<StatisticalMeasure> comboStatisticalMeasure;

    /**
     * @param formFactory
     * @param section
     * @param entity
     * @param removeListener
     * @param isChoosableEntity
     * @param backgroundColor
     * @param style
     */
    public StatisticalMeasureCollectionElement(CdmFormFactory formFactory, AbstractFormSection section, StatisticalMeasure entity,
            SelectionListener removeListener, Color backgroundColor, int style) {
        super(formFactory, section, entity, removeListener, backgroundColor, style);

    }

    @Override
    public void createControls(ICdmFormElement element, int style) {
        comboStatisticalMeasure = formFactory.createDefinedTermComboElementDto(TermType.StatisticalMeasure, element, "Statistical measure", null, style);
        if (entity != null){
        	setEntity(entity);
        }
    }

    @Override
    public void setEntity(StatisticalMeasure entity) {
        this.entity = entity;
        if(entity.getId()!=0 && comboStatisticalMeasure != null){
            comboStatisticalMeasure.setSelection(entity);
            comboStatisticalMeasure.removeEmptyElement();
        }
    }

    @Override
    public void handleEvent(Object eventSource) {
        if(eventSource==comboStatisticalMeasure && comboStatisticalMeasure.getSelection()!=null){
            if(getParentElement() instanceof StatisticalMeasureCollectionSection){
                StatisticalMeasureCollectionSection parentElement = (StatisticalMeasureCollectionSection) getParentElement();
                Feature feature = parentElement.getEntity();
                feature.removeRecommendedStatisticalMeasure(entity);
                StatisticalMeasure statisticalMeasure = comboStatisticalMeasure.getSelection();
                feature.addRecommendedStatisticalMeasure(statisticalMeasure);
                entity = statisticalMeasure;
                //update the parent section to re-set the listeners to the persisted measurement unit
                parentElement.removeElementAndUpdate(null);
            }
        }
    }

}
