/**
* Copyright (C) 2015 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.ui.section.vocabulary;

import java.util.Collection;
import java.util.Comparator;

import eu.etaxonomy.cdm.api.conversation.ConversationHolder;
import eu.etaxonomy.cdm.model.description.StatisticalMeasure;
import eu.etaxonomy.cdm.persistence.dto.FeatureDto;
import eu.etaxonomy.cdm.persistence.dto.TermDto;
import eu.etaxonomy.taxeditor.ui.element.CdmFormFactory;
import eu.etaxonomy.taxeditor.ui.element.ICdmFormElement;
import eu.etaxonomy.taxeditor.ui.section.occurrence.dna.AbstractUnboundEntityCollectionSection;

/**
 * @author pplitzner
 * @date Apr 27, 2015
 *
 */
public class StatisticalMeasureDtoCollectionSection extends AbstractUnboundEntityCollectionSection<FeatureDto, TermDto> {

    public StatisticalMeasureDtoCollectionSection(CdmFormFactory formFactory,
            ConversationHolder conversation, ICdmFormElement parentElement, int style) {
        super(formFactory, conversation, parentElement, "Statistical measures", style);
    }

    /* (non-Javadoc)
     * @see eu.etaxonomy.taxeditor.ui.section.occurrence.dna.AbstractNullEntityCollectionSection#getEntityCollection(java.lang.Object)
     */
    /** {@inheritDoc} */
    @Override
    protected Collection<TermDto> getEntityCollection(FeatureDto entity) {
        return entity.getRecommendedStatisticalMeasures();
    }

    @Override
    public Comparator<TermDto> getComparator() {
        return new Comparator<TermDto>() {
            @Override
            public int compare(TermDto o1, TermDto o2) {
                if(o1==null){
                    return -1;
                }
                if(o2==null){
                    return 1;
                }
                int diff = o1.getTitleCache().compareTo(o2.getTitleCache());
                if(diff==0){
                    diff = o1.getUuid().compareTo(o2.getUuid());
                }
                return diff;
            }
        };
    }

    /* (non-Javadoc)
     * @see eu.etaxonomy.taxeditor.ui.section.AbstractEntityCollectionSection#createNewElement()
     */
    /** {@inheritDoc} */
    @Override
    public TermDto createNewElement() {
        return TermDto.fromTerm(StatisticalMeasure.NewInstance());
    }

    /* (non-Javadoc)
     * @see eu.etaxonomy.taxeditor.section.AbstractEntityCollectionSection#addElement(eu.etaxonomy.cdm.model.common.IVersionableEntity)
     */
    /** {@inheritDoc} */
    @Override
    public void addElement(TermDto element) {
        //never gets called
    }

    /* (non-Javadoc)
     * @see eu.etaxonomy.taxeditor.section.AbstractEntityCollectionSection#removeElement(eu.etaxonomy.cdm.model.common.IVersionableEntity)
     */
    /** {@inheritDoc} */
    @Override
    public void removeElement(TermDto element) {
        if (element == null){
            return;
        }
        TermDto remove = null;
        for (TermDto unit: getEntity().getRecommendedStatisticalMeasures()){
            if (unit.getUuid().equals(element.getUuid())){
                remove = unit;
                break;
            }
        }
        getEntity().getRecommendedStatisticalMeasures().remove(remove);
    }

    /* (non-Javadoc)
     * @see eu.etaxonomy.taxeditor.section.AbstractEntityCollectionSection#getEmptyString()
     */
    /** {@inheritDoc} */
    @Override
    public String getEmptyString() {
        return "No statistical measures yet.";
    }

    /* (non-Javadoc)
     * @see eu.etaxonomy.taxeditor.section.AbstractEntityCollectionSection#getTooltipString()
     */
    /** {@inheritDoc} */
    @Override
    protected String getTooltipString() {
        return "Add a statistical measure";
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public TermDto addExisting() {
        // TODO Auto-generated method stub
        return null;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public boolean allowAddExisting() {
        // TODO Auto-generated method stub
        return false;
    }

}
