/**
* Copyright (C) 2014 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.ui.section.occurrence.dna;

import java.util.Collection;
import java.util.Collections;
import java.util.Comparator;

import eu.etaxonomy.cdm.api.conversation.ConversationHolder;
import eu.etaxonomy.cdm.model.media.Media;
import eu.etaxonomy.cdm.model.molecular.AmplificationResult;
import eu.etaxonomy.taxeditor.ui.element.CdmFormFactory;
import eu.etaxonomy.taxeditor.ui.element.ICdmFormElement;
import eu.etaxonomy.taxeditor.ui.section.AbstractEntityCollectionSection;
import eu.etaxonomy.taxeditor.ui.section.DefaultCdmBaseComparator;

/**
 * @author pplitzner
 * @date 26.02.2014
 *
 */
public class AmplificationGelPhotoCollectionDetailSection extends AbstractEntityCollectionSection<AmplificationResult, Media> {

    /**
     * @param formFactory
     * @param conversation
     * @param parentElement
     * @param title
     * @param style
     */
    public AmplificationGelPhotoCollectionDetailSection(CdmFormFactory formFactory, ConversationHolder conversation,
            ICdmFormElement parentElement, int style) {
        super(formFactory, conversation, parentElement, "Gel Photo", style);
    }



    /* (non-Javadoc)
     * @see eu.etaxonomy.taxeditor.ui.section.AbstractEntityCollectionSection#getCollection(java.lang.Object)
     */
    @Override
    public Collection<Media> getCollection(AmplificationResult entity) {
        if(entity.getGelPhoto()!=null){
            return Collections.singleton(entity.getGelPhoto());
        }
        return Collections.EMPTY_LIST;
    }

    @Override
    public Comparator<Media> getComparator() {
        return new DefaultCdmBaseComparator<>();
    }

    /* (non-Javadoc)
     * @see eu.etaxonomy.taxeditor.ui.section.AbstractEntityCollectionSection#createNewElement()
     */
    @Override
    public Media createNewElement() {
        if(getEntity().getGelPhoto()==null){
            return Media.NewInstance();
        }
        return null;
    }

    /* (non-Javadoc)
     * @see eu.etaxonomy.taxeditor.ui.section.AbstractEntityCollectionSection#addElement(java.lang.Object)
     */
    @Override
    public void addElement(Media element) {
        if(getEntity().getGelPhoto()==null){
            getEntity().setGelPhoto(element);
        }

    }

    /* (non-Javadoc)
     * @see eu.etaxonomy.taxeditor.ui.section.AbstractEntityCollectionSection#removeElement(java.lang.Object)
     */
    @Override
    public void removeElement(Media element) {
        getEntity().setGelPhoto(null);
    }

    /* (non-Javadoc)
     * @see eu.etaxonomy.taxeditor.ui.section.AbstractEntityCollectionSection#getEmptyString()
     */
    @Override
    public String getEmptyString() {
        return "No gel photo yet";
    }

    /* (non-Javadoc)
     * @see eu.etaxonomy.taxeditor.ui.section.AbstractEntityCollectionSection#getTooltipString()
     */
    @Override
    protected String getTooltipString() {
        return "Add a gel photo";
    }



    /**
     * {@inheritDoc}
     */
    @Override
    public Media addExisting() {
        return null;
    }



    /**
     * {@inheritDoc}
     */
    @Override
    public boolean allowAddExisting() {
        return false;
    }

}
