/**
* Copyright (C) 2014 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.ui.section.occurrence.dna;

import org.joda.time.DateTime;

import eu.etaxonomy.cdm.model.molecular.DnaQuality;
import eu.etaxonomy.cdm.model.molecular.DnaSample;
import eu.etaxonomy.cdm.model.term.OrderedTerm;
import eu.etaxonomy.cdm.model.term.TermType;
import eu.etaxonomy.taxeditor.ui.combo.term.TermComboElement;
import eu.etaxonomy.taxeditor.ui.element.CdmFormFactory;
import eu.etaxonomy.taxeditor.ui.element.ICdmFormElement;
import eu.etaxonomy.taxeditor.ui.element.NumberWithLabelElement;
import eu.etaxonomy.taxeditor.ui.element.TextWithLabelElement;
import eu.etaxonomy.taxeditor.ui.mvc.element.DateElement;
import eu.etaxonomy.taxeditor.ui.section.AbstractCdmDetailElement;

/**
 * @author pplitzner
 * @date 05.03.2014
 */
public class DnaQualityDetailElement extends AbstractCdmDetailElement<DnaSample> {

    private TextWithLabelElement textPurificationMethod;
    private NumberWithLabelElement numberRatioOfAbsorbance260_280;
    private NumberWithLabelElement numberRatioOfAbsorbance260_230;
    private NumberWithLabelElement numberConcentration;
    private TermComboElement<OrderedTerm> comboQualityTerm;
    private DateElement dateQualityCheck;

    public DnaQualityDetailElement(CdmFormFactory formFactory, ICdmFormElement formElement) {
        super(formFactory, formElement);
    }

    @Override
    protected void createControls(ICdmFormElement formElement, DnaSample entity, int style) {
        DnaQuality dnaQuality = entity.getDnaQuality();
        if(dnaQuality==null){
            dnaQuality = DnaQuality.NewInstance();
            entity.setDnaQuality(dnaQuality);
        }
        Double ratioOfAbsorbance260_230 = dnaQuality.getRatioOfAbsorbance260_230();
        Double ratioOfAbsorbance260_280 = dnaQuality.getRatioOfAbsorbance260_280();
        Double concentration = dnaQuality.getConcentration();
        OrderedTerm qualityTerm = dnaQuality.getQualityTerm();

        textPurificationMethod = formFactory.createTextWithLabelElement(formElement, "DNA Purification Details", dnaQuality.getPurificationMethod(), style);
        numberRatioOfAbsorbance260_280 = formFactory.createFloatTextWithLabelElement(formElement, "Ratio of absorbance 260/280", ratioOfAbsorbance260_280, style);
        numberRatioOfAbsorbance260_230 = formFactory.createFloatTextWithLabelElement(formElement, "Ratio of absorbance 260/230", ratioOfAbsorbance260_230, style);
        numberConcentration = formFactory.createFloatTextWithLabelElement(formElement, "Concentration", concentration, style);
        comboQualityTerm = formFactory.createDefinedTermComboElement(TermType.DnaQualityType, formElement, "Quality Term", qualityTerm, style);
        DateTime qualityCheckedDate = dnaQuality.getQualityCheckDate();

        dateQualityCheck = formFactory.createDateElement(formElement, "Quality Check", qualityCheckedDate, style, false);
    }

    @Override
    public void handleEvent(Object eventSource) {
        DnaQuality dnaQuality = getEntity().getDnaQuality();
        if(eventSource==textPurificationMethod){
            dnaQuality.setPurificationMethod(textPurificationMethod.getText());
        }
        else if(eventSource==numberRatioOfAbsorbance260_230){
            dnaQuality.setRatioOfAbsorbance260_230(numberRatioOfAbsorbance260_230.getDouble());
        }
        else if(eventSource==numberRatioOfAbsorbance260_280){
            dnaQuality.setRatioOfAbsorbance260_280(numberRatioOfAbsorbance260_280.getDouble());
        }
        else if(eventSource==numberConcentration){
            dnaQuality.setConcentration(numberConcentration.getDouble());
        }
        else if(eventSource==comboQualityTerm){
            dnaQuality.setQualityTerm(comboQualityTerm.getSelection());
        }
        else if(eventSource==dateQualityCheck.getController()){
            DateTime dateTime = dateQualityCheck.getController().getDateTime();
            dnaQuality.setQualityCheckDate(dateTime);
        }
    }
}