/**
* Copyright (C) 2014 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.ui.section.occurrence.dna;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionListener;

import eu.etaxonomy.cdm.model.molecular.Sequence;
import eu.etaxonomy.cdm.model.reference.Reference;
import eu.etaxonomy.taxeditor.ui.element.AbstractFormSection;
import eu.etaxonomy.taxeditor.ui.element.CdmFormFactory;
import eu.etaxonomy.taxeditor.ui.element.ICdmFormElement;
import eu.etaxonomy.taxeditor.ui.element.TextWithLabelElement;
import eu.etaxonomy.taxeditor.ui.section.AbstractEntityCollectionElement;
import eu.etaxonomy.taxeditor.ui.selection.EntitySelectionElement;

/**
 * @author pplitzner
 * @date 07.01.2014
 */
public class SequenceReferenceCollectionDetailElement
        extends AbstractEntityCollectionElement<Reference>{

    protected EntitySelectionElement<Reference> selection_reference;
    protected TextWithLabelElement text_referenceDetail;

    public SequenceReferenceCollectionDetailElement(CdmFormFactory formFactory,
            AbstractFormSection<?> section, Reference entity,
            SelectionListener removeListener, int style) {
        super(formFactory, section, entity, removeListener, null, style);
    }

    @Override
    public void createControls(ICdmFormElement formElement, int style) {
        selection_reference = formFactory
                .createSelectionElement(Reference.class,
                        formElement, "Reference",
                        null, EntitySelectionElement.SELECTABLE|EntitySelectionElement.EDITABLE, style);
        text_referenceDetail = formFactory.createTextWithLabelElement(
                formElement, "Reference Detail", null, SWT.NULL);
        if (entity != null){
        	setEntity(entity);
        }
    }

    @Override
    public void setEntity(Reference entity) {
        this.entity = entity;
        if(selection_reference != null){
        	selection_reference.setEntity(entity);
        	text_referenceDetail.setText(entity.getReferenceAbstract());
        }
        
    }

    @Override
    public void handleEvent(Object eventSource) {
        if(eventSource==selection_reference){
            if(getParentElement() instanceof SequenceReferenceCollectionDetailSection){
                Sequence sequence = ((SequenceReferenceCollectionDetailSection) getParentElement()).getEntity();
                sequence.removeCitation(entity);
                Reference newCitation = selection_reference.getSelection();
                sequence.addCitation(newCitation);
                entity = newCitation;

                text_referenceDetail.setEnabled(true);
            }
        } else if(eventSource==text_referenceDetail){
            entity.setReferenceAbstract(text_referenceDetail.getText());
        }
    }
}